(*  :Title:	Cost Functions  *)

(*  :Authors:	Brian Evans and James McClellan  *)

(*
    :Summary:	To implement cost functions to measure cost of
		signal processing operations
 *)

(*  :Context:	SignalProcessing`ObjectOriented`System`  *)

(*  :PackageVersion:  SPP 2.9  @(#)CostFunctions.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*
    :Source:	Myers, Cory.  {Signal Representation for Symbolic and
		  Numeric Processing}.  M.I.T. Ph. D. Thesis.  August,
		  1986.  Appendix D.

		Covell, Michele.  {An Algorithm Design Environment for
		  Signal Processing}.  M.I.T. Ph. D. Thesis.  December,
		  1989.

                Evans, Brian. {A Knowledge-Based Environment for the
		  Design and Analysis of Multidimensional Multirate Signal
		  Processing Algorithms}. Ph. D. Thesis, Georgia Institute
		  of Technology, Atlanta, GA, June, 1993.
 *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*
    :Discussion:

    The cost functions described here measure the number of additions,
    multiplications, and memory elements need to implement a signal
    processing operation.
 *)

(*
    :Functions:	AdditiveCosts
		AlgorithmCostFunction
		CostRange
		MemoryCosts
		MultiplicationCosts
		PerOutputSamplingRate
		SamplingRate
 *)



(*  B E G I N     P A C K A G E  *)

BeginPackage[ "SignalProcessing`ObjectOriented`CostFunctions`",
	      "SignalProcessing`ObjectOriented`System`",
	      "SignalProcessing`Support`FilterSupport`",
	      "SignalProcessing`Support`SigProc`",
	      "SignalProcessing`Support`SupCode`" ]


If [ TrueQ[ $VersionNumber >= 2.0 ],
     Off[ General::spell ];
     Off[ General::spell1 ] ];


(*  U S A G E     I N F O R M A T I O N  *)

AdditiveCosts::usage =
	"AdditiveCosts[ oper[p1, p2, ..., pn][sig1, sig2, ..., sign ] ] \
	returns the number of additions needed to implement the \
	operator oper (having parameters p1, p2, ..., pn) in terms \
	of the sampling rate of the input signals sig1, sig2, ..., sign."

AlgorithmCostFunction::usage =
	"AlgorithmCostFunction[ expr, {w1, w2, ..., wn}, {f1, f2, ..., fn} ] \
	computes the cost of expr as a weighted sum of cost functions, i.e. \
	sum (wi fi) over i. \
	The cost functions should be able to measure the cost of a \
	general expression."

ComputePerOutputSamplingRate::usage =
	"ComputePerOutputSamplingRate[expr, fun1, fun2, vars] in 1-D simply \
	returns the sampling rate for expr. \
	In m-D, this function first applies fun1 to each column of the \
	sampling matrix for expr in terms of variables vars and \
	then applies fun1 to the resulting vector."

CostRange::usage =
	"\"symbol/: CostRange[symbol] = range\" assigns a range of values \
	to the symbol.
	You can remove the definition by evaluating \
	\"symbol/: CostRange[symbol] = .\"."

MemoryCosts::usage =
	"MemoryCosts[ oper[p1, p2, ..., pN][sig1, sig2, ..., sigN ] ] \
	returns the number of memory elements needed to implement the \
	operator oper (having parameters p1, p2, ..., pN) in terms \
	of the sampling rate of the input signals sig1, sig2, ..., sigN."

MultiplicationCosts::usage =
	"MultiplicationCosts[ oper[p1, p2, ..., pN][sig1, sig2, ..., sigN ] ] \
	returns the number of multiplications needed to implement the \
	operator oper (having parameters p1, p2, ..., pN) in terms \
	of the sampling rate of the input signals sig1, sig2, ..., sigN."

PerOutputSamplingRate::usage =
	"PerOutputSamplingRate[inputrate, blocksize(s), numsamples, var(s)] \
	computes the sampling rate per output sample for block operations \
	given the input sampling rate (matrix) inputrate, the size of blocks \
	in each dimension blocksize(s), the total number of samples processed \
	numsamples, and the variable(s) var(s) on which the block operation \
	is performed."

SamplingRate::usage =
	"SamplingRate[expr, var] returns the sampling rate (matrix) of the \
	expression expr with respect to the discrete variable (variables)
	var. \
	SamplingRate assumes that the input signals for multi-input \
	operators have been sampled at the same rate."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin[ "`Private`" ]


Unprotect[ CircularShift, DFT, Downsample, FIR, IIR, Interleave,
	   InvDFT, Periodic, PolyphaseDownsample, PolyphaseUpsample,
	   PolyphaseResample, Rev, Shift, Summation, Upsample ]


(*  L O C A L     R O U T I N E S  *)

(*  average  *)
average[ x_List ] := ( Apply[Plus, x] / Length[x] ) /; VectorQ[x]
average[ x_ ] := x

(*  divideBy2  *)
divideBy2[ n_Integer ] := If [ OddQ[n], (n + 1) / 2, n / 2 ]

(*  filterCosts--- check for symmetry in coefficients  *)
filterCosts[ coeffs_ ] :=
	If [ symmetricMatrixQ[coeffs],
	     divideBy2[ Apply[Times, Dimensions[coeffs]] ],
	     Apply[Times, Dimensions[coeffs]] - 1 ]

(* matmult *)
matmult[a_, b_, n_List] := a . b
matmult[a_, b_, n_Symbol] := a b

(* matinv  *)
matinv[a_, n_List] := Inverse[a]
matinv[a_, n_Symbol] := 1/a

(*  sampleVolume  *)
sampleVolume[ x_?MatrixQ, n_ ] := Abs[Det[x]]
sampleVolume[ x_, n_Symbol ] := CostRange[x]
sampleVolume[ x_, n_ ] := Abs[x]

(*  pre-processing check  *)
(*  FIR and IIR coefficients need to be given *)
(*  rewrite Difference operators as FIR filters  *)
(*  check to see if signal inputs for multi-input *)
(*    operators have the sampling rate  *)

(*  symmetricMatrixQ  *)
symmetricMatrixQ[ x_ ] := symmetrixMatrixQ[ x, Dimensions[x] ]
symmetricMatrixQ[ x_, {len_Integer} ] := SameQ[ x, Reverse[x] ] 
symmetricMatrixQ[ x_, matrix_ ] :=
Block [	{},
	Print[ "symmetric m-D filter coefficient check not implemented." ]
	False ]


(*  C O S T     F U N C T I O N S  *)

(*  AdditiveCosts  *)
AdditiveCosts[ h_Plus ] = 1
AdditiveCosts[ h_Subtract ] = 1
AdditiveCosts[ h_Times ] = 0
AdditiveCosts[ h_Divide ] = 0
AdditiveCosts[ h_Conjugate ] = 0
AdditiveCosts[ h_Im ] = 0
AdditiveCosts[ h_Re ] = 0

CircularShift/: AdditiveCosts[ CircularShift[n0_, N_, n_][x_] ] := 0
DFT/: AdditiveCosts[ DFT[l_, n_, k_, ___][x_] ] :=
	( sampleVolume[l] - 1 )^2 *
	PerOutputSamplingRate[ SamplingRate[x, n], Dimensions[l], 
			       Apply[Times, Dimensions[l]], n ]
Downsample/: AdditiveCosts[ Downsample[m_, n_, ___][x_] ] := 0
FIR/: AdditiveCosts[ FIR[n_, coeffs_, ___][x_] ] :=
	( Apply[Times, Dimensions[coeffs]] - 1 ) *
	ComputePerOutputSamplingRate[x, average, average, n]
IIR/: AdditiveCosts[ IIR[n_, coeffs_, ___][x_] ] :=
	( Apply[Times, Dimensions[coeffs]] - 1 ) *
	ComputePerOutputSamplingRate[x, average, average, n]
Interleave/: AdditiveCosts[ Interleave[n_, ___][x1_, xrest__] ] :=
	Block [	{len, list},
		list = {x1, xrest};
		len = Length[list];
		Apply[Times, Map[ AdditiveCosts, lists ]] / len ]
InvDFT/: AdditiveCosts[ InvDFT[l_, k_, n_, ___][x_] ] :=
	( sampleVolume[l] - 1 )^2 *
	PerOutputSamplingRate[ SamplingRate[x, n], Dimensions[l], 
			       Apply[Times, Dimensions[l]], n ]
Periodic/: AdditiveCosts[ Periodic[k_, n_, ___][x_] ] := 0
PolyphaseDownsample/:
	AdditiveCosts[
		PolyphaseDownsample[m_, h_[n_, coeffs_, ___], n_, ___][x_] ] :=
	( Apply[Times, Dimensions[coeffs]] - 1 ) *
	PerOutputSamplingRate[ SamplingRate[ Downsample[m, n][x], n ],
			       Dimensions[coeffs],
			       Apply[Times, Dimensions[coeffs]], n ] /;
	SameQ[h, FIR] || SameQ[h, IIR]
PolyphaseUpsample/:
	AdditiveCosts[
		PolyphaseUpsample[l_, h_[n_, coeffs_, ___], n_, ___][x_] ] :=
	( Apply[Times, Dimensions[coeffs]] - 1 ) *
	PerOutputSamplingRate[ SamplingRate[x, n], Dimensions[coeffs],
			       Apply[Times, Dimensions[coeffs]], n ] /;
	SameQ[h, FIR] || SameQ[h, IIR]
Rev/: AdditiveCosts[ Rev[n_, ___][x_] ] := 0
Shift/: AdditiveCosts[ Shift[n0_, n_][x_] ] := 0
Summation/: AdditiveCosts[ Summation[i_, ibeg_, iend_, inc_, ___][x_] ] :=
	( (iend - ibeg) / inc + 1 ) * SamplingRate[x, i]
Upsample/: AdditiveCosts[ Upsample[l_, n_, ___][x_] ] := 0

(*  MemoryCosts--- independent of sampling rates  *)
MemoryCosts[ h_Plus ] = 1
MemoryCosts[ h_Subtract ] = 1
MemoryCosts[ h_Times ] = 1
MemoryCosts[ h_Divide ] = 1
MemoryCosts[ h_Conjugate ] = 1
MemoryCosts[ h_Im ] = 0
MemoryCosts[ h_Re ] = 0

CircularShift/: MemoryCosts[ CircularShift[n0_, N_, n_, ___][x_] ] :=
	sampleVolume[N, n]
DFT/: MemoryCosts[ DFT[l_, n_, k_, ___][x_] ] :=
	( 2 sampleVolume[l] - 1 )
Downsample/: MemoryCosts[ Downsample[m_, n_, ___][x_] ] :=
	sampleVolume[m, n]
FIR/: MemoryCosts[ FIR[n_, coeffs_, ___][x_] ] :=
	filterCosts[coeffs]
IIR/: MemoryCosts[ IIR[n_, coeffs_, ___][x_] ] :=
	filterCosts[coeffs]
Interleave/: MemoryCosts[ Interleave[n_, ___][x1_, xrest__] ] :=
	Length[ {x1, xrest} ]
InvDFT/: MemoryCosts[ InvDFT[l_, k_, n_, ___][x_] ] :=
	( 2 sampleVolume[l] - 1 )
Periodic/: MemoryCosts[ Periodic[k_, n_, ___][x_] ] :=
	sampleVolume[k, n]
Rev/: MemoryCosts[ Rev[n_, ___][x_] ] := 0
Shift/: MemoryCosts[ Shift[n0_, n_, ___][x_] ] :=
	sampleVolume[N, n]
Summation/: MemoryCosts[ Summation[i_, ibeg_, iend_, inc_, ___][x_] ] := 0
Upsample/: MemoryCosts[ Upsample[l_, n_, ___][x_] ] :=
	sampleVolume[l, n]


(*  MultiplicationCosts  *)
MultiplicationCosts[ h_Plus ] = 0
MultiplicationCosts[ h_Subtract ] = 0
MultiplicationCosts[ h_Times ] = 1
MultiplicationCosts[ h_Divide ] = 1
MultiplicationCosts[ h_Conjugate ] = 1
MultiplicationCosts[ h_Im ] = 0
MultiplicationCosts[ h_Re ] = 0

CircularShift/: MultiplicativeCosts[ CircularShift[n0_, N_, n_, ___][x_] ] := 0
DFT/: MultiplicativeCosts[ DFT[l_, n_, k_, ___][x_] ] :=
	sampleVolume[l]^2 *
	PerOutputSamplingRate[ SamplingRate[x, n], Dimensions[l], 
			       Apply[Times, Dimensions[l]], n ]
Downsample/: MultiplicativeCosts[ Downsample[m_, n_, ___][x_] ] := 0
FIR/: MultiplicativeCosts[ FIR[n_, coeffs_, ___][x_] ] :=
	filterCosts[coeffs] *
	ComputePerOutputSamplingRate[x, average, average, n]
IIR/: MultiplicativeCosts[ IIR[n_, coeffs_, ___][x_] ] :=
	filterCosts[coeffs] *
	ComputePerOutputSamplingRate[x, average, average, n]
Interleave/: MultiplicationCosts[ Interleave[n_, ___][x1_, xrest__] ] :=
	Block [	{len, list},
		list = {x1, xrest};
		len = Length[list];
		Apply[Times, Map[ MultiplicationCosts, lists ]] / len ]
InvDFT/: MultiplicativeCosts[ InvDFT[l_, k_, n_, ___][x_] ] :=
	sampleVolume[l] *
	PerOutputSamplingRate[ SamplingRate[x, n], Dimensions[l], 
			       Apply[Times, Dimensions[l]], n ]
Periodic/: MultiplicativeCosts[ Periodic[k_, n_, ___][x_] ] := 0
PolyphaseDownsample/:
    MultiplicativeCosts[
	PolyphaseDownsample[m_, h_[n_, coeffs_, ___], n_, ___][x_] ] :=
	filterCosts[coeffs] *
	PerOutputSamplingRate[ SamplingRate[ Downsample[m, n][x], n ],
			       Dimensions[coeffs],
			       Apply[Times, Dimensions[coeffs]], n ] /;
	SameQ[h, FIR] || SameQ[h, IIR]
PolyphaseUpsample/:
	MultiplicativeCosts[
		PolyphaseUpsample[l_, h_[n_, coeffs_, ___], n_, ___][x_] ] :=
	filterCosts[coeffs] *
	PerOutputSamplingRate[ SamplingRate[x, n], Dimensions[coeffs],
			       Apply[Times, Dimensions[coeffs]], n ] /;
	SameQ[h, FIR] || SameQ[h, IIR]
Rev/: MultiplicativeCosts[ Rev[n_, ___][x_] ] := 0
Shift/: MultiplicativeCosts[ Shift[n0_, n_][x_] ] := 0
Summation/: MultiplicativeCosts[
		Summation[i_, ibeg_, iend_, inc_, ___][x_] ] := 0
Upsample/: MultiplicativeCosts[ Upsample[l_, n_, ___][x_] ] := 0


(*  O T H E R     E X P O R T E D     F U N C T I O N S  *)

(*  AlgorithmCostFunction  *)
applyCostFunction[ expr_, fun_ ] := fun[expr]

AlgorithmCostFunction[ expr_, weightList_List, funList_List ] :=
	( weightList . Map[ applyCostFunction[expr, #]&, funList ] ) /;
	Length[weightList, funList]

(*  ComputePerOutputSamplingRate  *)
ComputePerOutputSamplingRate[ x_, fun_, n_Symbol ] := SamplingRate[x, n]
ComputePerOutputSamplingRate[ x_, fun1_, fun2_, n_List ] :=
	fun2[ Map[ fun1, Transpose[ SamplingRate[x, n] ] ] ]

(*  CostRange  *)
CostRange[x_?AtomQ] := x
CostRange[Hold[x__]] := Hold[x]
CostRange[h_[a_]] := h [ CostRange[a] ]
CostRange[h_[a1_, a__]] := Apply[ h, Map[CostRange, { a1, a }] ]

(*  PerOutputSamplingRate  *)
PerOutputSamplingRate[ sampleRate_, blockSize_, totalSamples_, n_Symbol ] :=
	sampleRate
PerOutputSamplingRate[ sampleMatrix_, blockSizes_, totalSamples_, n_List ] :=
	( sampleMatrix . blockSizes ) / totalSamples

(*  SamplingRate  *)
Downsample/: SamplingRate[ Downsample[m_, n_][x_], n_ ] :=
	matmult[m, SamplingRate[x, n], n]
Interleave/: SamplingRate[ Interleave[n_][x1_, xrest__], n_ ] :=
	Length[{x1, xrest}] SamplingRate[x1]
PolyphaseDownsample/: SamplingRate[ PolyphaseDownsample[m_, h_, n_][x_], n_ ] :=
	matmult[m, SamplingRate[x, n], n]
PolyphaseUpsample/: SamplingRate[ PolyphaseUpsample[l_, h_, n_][x_], n_ ] :=
	matmult[matinv[m], SamplingRate[x, n], n]
PolyphaseResample/: SamplingRate[ PolyphaseResample[l_, n_][x_], n_ ] :=
	matmult[matinv[m], matmult[l, SamplingRate[x, n]], n]
Upsample/: SamplingRate[ Upsample[l_, n_][x_], n_ ] :=
	matmult[matinv[m], SamplingRate[x, n], n]

SamplingRate[ x_[params__][arg1_, args__], n_ ] := 
	SamplingRate[arg1, n] /;
	LINEAR[ x[params] ] && MEMORYLESS[ x[params] ]

SamplingRate[ x_[params__][args__], n_ ] :=
	QueryTheUser[ x[params][args], x, SamplingRate ]
SamplingRate[ x_[args__], n_ ] :=
	QueryTheUser[ x[args], x, SamplingRate, n ]


(*  E N D     P A C K A G E  *)

Protect[ CircularShift, DFT, Downsample, FIR, IIR, Interleave,
	 InvDFT, Periodic, PolyphaseDownsample, PolyphaseUpsample,
	 PolyphaseResample, Rev, Shift, Summation, Upsample ]

End[]
EndPackage[]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell ];
     On[ General::spell1 ] ];


(*  W R I T E     P R O T E C T I O N  *)

Block [	{newfuns},
	newfuns = 
	      {	AdditiveCosts,	AlgorithmCostFunction,	CostRange,
		MemoryCosts,	MultiplicationCosts,	PerOutputSamplingRate,
		SamplingRate };
	Combine[ SPfunctions, newfuns ];
	Apply[ Protect, newfuns ] ]


(*  E N D I N G     M E S S A G E  *)

Print["Routines for measuring algorithm cost have been loaded."]
Null
