(*  :Title:	Piecewise Convolution Algorithms  *)

(*  :Authors:	Kevin West, Brian Evans, James McClellan, Robert Bamberger *)

(*
    :Summary:	To take the continuous-time and discrete-time
		convolution of two piecewise-defined functions and return
		another piecewise-defined function.
 *)

(*  :Context:	SignalProcessing`Support`Convolution`  *)

(*  :PackageVersion:  SPP 2.9  @(#)Convolution.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*
    :Source:	  Oppenheim, A., and Schafer, R.  {Discrete-Time Signal
		Processing}.  1989.

		  Z. Zuhao, "The Square Matrix Rule of the Convolution
		Integral", IEEE Trans. on Education, Nov., 1990, pp. 369-372

		  B. L. Evans, "A Knowledge-Based Environment for the Design
		and Analysis of Multidimensional Multirate Signal Processing
		Algorithms", Ph. D. Thesis, Georgia Institute of Technology,
		Atlanta, GA, June, 1993.

		  K. A. West and J. H. McClellan, "Symbolic Convolution",
		IEEE Trans. on Education, vol. 36, no. 4, pp. 386-393, 
		Nov., 1993.
 *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*
    :Discussion:  This package originally performed piecewise continuous
		  convolution and therefore was part of the analog signal
		  processing packages (spp).  We've added discrete convolution,
		  so it is now in the Support packages.  It's old context
		  was "SignalProcessing`Analog`Convolution`"---  if you
		  ask Mathematica to load Convolution from the analog spp,
		  it will still load.
 *)

(*
    :Functions:	AutoCorrelation
		ConvertFromList
		ConvertToList
		FormulaToPiecewise
		CTPiecewiseConvolution
		DTPiecewiseConvolution
		IntervalQ
		PiecewiseConvolution
		PiecewisePlot
		PiecewiseToFormula
		PlotList
		SimplifyList
		ValidIntervalQ
 *)



(*  B E G I N     P A C K A G E  *)

BeginPackage [	"SignalProcessing`Support`Convolution`",
		"SignalProcessing`Support`SigProc`",
		"SignalProcessing`Support`Lisp`",
		"SignalProcessing`Support`SupCode`" ]


If [ TrueQ[ $VersionNumber >= 2.0 ],
     Off[ General::spell ];
     Off[ General::spell1 ] ]

			
(*  U S A G E     I N F O R M A T I O N  *)

$ConvolutionDomain::usage = 
	"$ConvolutionDomain is the default domain for all of the routines \
	in the Convolution package (except for DTPiecewiseConvolution and \
	CTPiecewiseConvolution which imply a domain). \
	Use SetConvolutionDomain to reset its value."

Area::usage =
	"Area[ar] is an object representing the area under a Dirac delta \
	function. \
	A Dirac delta function is usually written as C Delta[t - t0], \
	where C is the area under the delta function and t0 is the location \
	of the delta function. \
	In F-interval form, C Delta[t - t0] becomes { Area[C], t0, t0 }."

AutoCorrelation::usage =
	"AutoCorrelation[e, v] computes the autocorrelation of expression e \
	(in the form of a list of F-intervals, an F-interval, or a function) \
	with respect to v. \
	The variable v is treated as either discrete and continuous \
	according to the value of the Domain option. \
	The default domain is the value of $ConvolutionDomain."

ConvertFromList::usage =
	"FormulaToPiecewise[l, t] converts a list of F-intervals in the form \
	{f, t1, t2} to a signal processing expression. \
	The new expression will contain either Delta, CStep, and CPulse \
	functions (continuous-time) or Impulse, Step, and Pulse functions \
	(discrete-time) according to the value of the Domain option. \
	The default domain is the value of $ConvolutionDomain. \
	The old name for this function is ConvertFromList."

ConvertToList::usage =
	"PiecewiseToFormula[e, t] converts the expression e to a \
	list of F-intervals in the form {fun, left, right}. \
	The domain of the new expression is established according to the \
	value of the Domain option. \
	The default domain is the value of $ConvolutionDomain. \
	The old name for this function is ConvertToList."

ConvolutionOutputForm::usage =
	"ConvolutionOutputForm is an option for CTPiecewiseConvolution \
	PiecewiseConvolution, and DTPiecewiseConvolution. \
	It can take a value of Automatic, Formula, or Piecewise. \
	If the option is set to Automatic, then the output will depend on
	the form of the inputs:  if either input was specified as a \
	formula (or expression), then the output will also be a formula; \
	otherwise, the output will be a list of F-intervals. \
	If the option is set to Piecewise, the output will be a list of \
	F-intervals no matter the way the input is specified. \
	If it is set to Formula, the output will be an expression \
	no matter the input form."

CTPiecewiseConvolution::usage =
	"CTPiecewiseConvolution[h, x, t] carries out the one-dimensional, \
	continuous-time piecewise convolution of h and x with respect to t. \
	See PiecewiseConvolution for valid representation of h and x. \
	Using F-interval notation {function, left_end_point, right_end_point}, \
	a Dirac delta is meant when function = Area[<area>] \
	(where <area> equals the area of under the impulse function) and \
	left_end_point = right_end_point (the location of the impulse). \
	A Dialogue option of True or All will cause the graphs for an \
	animation to be generated in which h is flipped in the variable t \
	and then slid across x. \
	See also PiecewiseConvolution."

DTPiecewiseConvolution::usage = 
	"DTPiecewiseConvolution[h, x, n] carries out the one-dimensional, \
	discrete-time piecewise convolution of h and x with respect to n. \
	See PiecewiseConvolution for the valid representations of h and x. \
	Using F-interval notation {function, left_end_point, right_end_point}, \
	an impulse (Kronecker delta) function is meant when \
	left_end_point = right_end_point (the location of the impulse) \
	and the strength of the impulse function is given by function. \
	A Dialogue option of True or All will cause the graphs for an \
	animation to be generated in which h is flipped in the variable n \
	and then slid across x. \
	Real-valued interval end points will be truncated to integers. \
	See also PiecewiseConvolution."

Formula::usage =
	"Formula is an option for the piecewise convolution routines. \
	See ConvolutionOutputForm for more information."

IntervalQ::usage =
	"IntervalQ[x] returns True if the argument x is an interval, and \
	gives False otherwise. \
	An interval has the form {f, t1, t2} such that none of f, t1, or t2 \
	are lists. \
	See also ValidIntervalQ."

Piecewise::usage =
	"Piecewise is an option for the piecewise convolution routines. \
	See ConvolutionOutputForm for more information."

PiecewiseConvolution::usage =
	"PiecewiseConvolution[f, g, v] convolves the piecewise functions \
	f and g with respect to the variable v. \
	In this context, a function is represented in a piecewise fashion: \
	(1) as F-intervals of the form {fun, left, right}, \
	(2) as a list of F-intervals, or (3) as an expression. \
	An F-interval has the form \
	{function, left_end_point, right_end_point}. \
	The F-interval notation represents a finite-extent function or \
	sequence when the end points do not equal infinity. \
	The result of convolution is always returned as a list of F-intervals. \
	The variable v is treated as either discrete and continuous \
	according to the value of the Domain option. \
	The default domain is the value of $ConvolutionDomain, \
	which can be reset by SetConvolutionDomain. \
	See also CTPiecewiseConvolution and DTPiecewiseConvolution."

PlotList::usage =
	"PiecewisePlot[l, {v, vstart, vend}] plots the piecewise function \
	l vs. v. \
	The variable v is treated as either discrete and continuous \
	according to the value of the Domain option. \
	The default domain is the value of $ConvolutionDomain. \
	The old name for this function is PlotList."

SetConvolutionDomain::usage =
	"SetConvolutionDomain[domain] establishes the default convolution \
	domain as either Continuous or Discrete. \
	It resets the global variable $ConvolutionDomain. \
	It returns domain on success, and Null on error."

SimplifyList::usage =
	"SimplifyList[interval_list, v] takes a list of F-intervals \
	each having the form {f, t1, t2} and returns a simplified sorted \
	list of F-intervals. \
	The interval_list is treated as either a continuous-time or a \
	discrete-time function according the value of the Domain option. \
	The variable v is treated as either discrete and continuous \
	according to the value of the Domain option. \
	The default domain is the value of $ConvolutionDomain."

SymmetricPulseConvolutions::usage =
	"SymmetricPulseConvolutions[c, a, t, n] convolves a symmetric pulse \
	with itself n - 1 times. \
	The symmetric pulse has amplitude c and is defined for the \
	continuous real variable t over the interval [-a, a]. \
	The function returns a formula. \
	Therefore, SymmetricPulseConvolutions[c, a, t, 1] simply returns \
	the symmetric pulse formula c CPulse[2 a, t + a]."

ValidIntervalQ::usage =
	"ValidIntervalQ[i] returns True if i is a valid F-interval, and \
	gives False otherwise. \
	An F-interval is an interval of the form {f, leftendp, rightendp} \
	where f is a function and leftendp < rightendp unless f is an \
	impulse function in which case leftendp must equal rightendp. \
	In a valid F-interval, leftendp and rightendp must be numbers. \
	See also IntervalQ."

VectorConvolution::usage =
	"VectorConvolution[vector1, vector2] convolves two vectors that \
	represent samples of function."

(*  E N D     U S A G E     I N F O R M A T I O N  *)



Begin[ "`Private`" ]


(*  S Y S T E M     S E T T I N G S  *)

$ConvolutionDomain = Continuous

Protect[ $ConvolutionDomain ]



(*  M E S S A G E S  *)

AutoCorrelation::domain =
	"The option Domain must be equal to Discrete or Continuous."
ConvertFromList::domain =
	"The option Domain must be equal to Discrete or Continuous."
ConvertToList::domain =
	"The option Domain must be equal to Discrete or Continuous."
CTPiecewiseConvolution::argct =
	"CTPiecewiseConvolution requires exactly three arguments."
DTPiecewiseConvolution::argct =
	"DTPiecewiseConvolution requires exactly three arguments."
PiecewiseConvolution::animation =
	"Generating the animation ... Please wait."
PiecewiseConvolution::argct =
	"PiecewiseConvolution requires at least three arguments."
PiecewiseConvolution::domain =
	"The option Domain must be equal to Discrete or Continuous."
PiecewiseConvolution::infext =
	"Cannot animate the flip-and-slide approach to convolution for \
	the function `` because it goes from -Infinity to Infinity."
PiecewiseConvolution::noanim =
	"No animation was generated because the result of the convolution \
	is zero."
PiecewiseConvolution::animnot =
	"The F-intervals `` of the `` will not appear in the animation frames."
PlotList::domain =
	"The option Domain must be equal to Discrete or Continuous."
SimplifyList::domain =
	"The option Domain must be equal to Discrete or Continuous."
SetConvolutionDomain::domain =
	"The option Domain must be equal to Discrete or Continuous."
ValidIntervalQ::domain =
	"The option Domain must be equal to Discrete or Continuous."

(*  E N D     M E S S A G E S  *)


(*  S U P P O R T I N G     R O U T I N E S  *)

(*  animateConvolution  *)
If [ TrueQ[ $VersionNumber >= 2.0 ],
     fontform[ expr_, font_, size_ ] := FontForm[ expr, {font, size} ],
     fontform[ expr_, font_, size_ ] := FontForm[ expr, font, size ] ]

makeLegendLine[plotstyles_, line_, string_] :=
	Graphics [ { ToCollection[ plotstyles[[line]] ],
		     Line[{{-2, 2 - line}, {-0.5, 2 - line}}],
		     Text[string, {0, 2 - line}, {-1, 0}] } ]

splitAndTell[ piecewise_, string_ ] :=
	Block [	{keep, reject},
		keep = Select[ piecewise, validEndPointsQ ];
		reject = Select[ piecewise, Not[validEndPointsQ[#]]& ];
		If [ ! EmptyQ[reject],
		     Message[ PiecewiseConvolution::animnot, reject, string ] ];
		N[ keep ] ]

validEndPointsQ[ {f_, a_, b_} ] :=
	( SameQ[a, -Infinity] || NumberQ[N[a]] ) &&
	( SameQ[b, Infinity] || NumberQ[N[b]] )

plotall[graphlist_] :=
	Map[ Show[#, DisplayFunction :> $DisplayFunction]&, graphlist ]

animateConvolution[ffunp_, gfunp_, convp_, t_, domain_, oplist_] :=
	Block [	{a, aa, animateFunction, animationSequence, aPlot, b, bb,
		 bPlot, c, cEndPoints, conv, convPlot, cwidth, endPoints,
		 fEndPoints, fPiecewise, frames = 25, fstr, fwidth, gEndPoints,
		 gPiecewise, graphOptions, gstr, gwidth, hstr, inf, inNotebook,
		 istep, minusInf, multiple, nEndPoints, nullPlot1, nullPlot2,
		 overlapstr, overlapctlstr, plot, plotfun, plotrange,
		 plotstyles, rangelist, stepfun, tcleft, tcright, tfleft,
		 tfright, tgleft, tgright, tmax, tmin, toffset, width, xstr},

		fPiecewise = splitAndTell[ffunp, "first function"];
		gPiecewise = splitAndTell[gfunp, "second function"];
		conv = splitAndTell[convp, "convolution result" ];

		If [ EmptyQ[conv],
		     Message[ PiecewiseConvolution::noanim ];
		     Return[ conv ] ];

		If [ TrueQ[$VersionNumber >= 2.0],
		     inNotebook = TrueQ[ $Notebooks ],
		     inNotebook = SameQ[$System, "NeXT"] ||
				  SameQ[$System, "Macintosh"] ];

		If [ inNotebook,
		     animationFunction = plotall,
		     Needs[ "Graphics`Animation`" ];
		     animationFunction = If [ TrueQ[ $VersionNumber >= 2.0 ],
					      Graphics`Animation`ShowAnimation,
					      Graphics`Animation`Animate ] ];

		If [ SameQ[domain, Continuous],   (* then *)
		     plotfun = Plot;
		     stepfun = CStep;
		     hstr = "h(``)";
		     xstr = "x(``)";
		     overlapctlstr = "x(u) h(``-u)",  (* else *)
		     plotfun = SequencePlot;
		     stepfun = Step;
		     hstr = "h[``]";
		     xstr = "x[``]";
		     overlapctlstr = "x[m] h[``-m]" ];
		overlapstr = ToString[StringForm[overlapctlstr, t]];
		hstr = ToString[StringForm[hstr, t]];
		xstr = ToString[StringForm[xstr, t]];

		multiple = TrueQ[ $VersionNumber >= 2.0 ] &&
			   SameQ[ Replace[Dialogue, oplist], All ];

		(*  Extract the end points for each of the three functions  *)
		fEndPoints = Map[getsecthird, fPiecewise];
		gEndPoints = Map[getsecthird, gPiecewise];
		cEndPoints = Map[getsecthird, conv];

		(*  Abort for two-sided infinite extent functions *)
		If [ MemberQ[cEndPoints, -Infinity] && 
		       MemberQ[cEndPoints, Infinity],
		     Message[ PiecewiseConvolution::infext, conv ];
		     Return[ conv ] ];

		(*  Animation takes much memory, so have Mathematica cleanup  *)
		Share[];

		(* Define the plot styles and graph options for all plots *)
		plotstyles = { {Thickness[.01], RGBColor[1,0,0]},
			       {Thickness[.005], RGBColor[0,1,0]},
			       {Thickness[.0025], RGBColor[0,0,1]} };

		graphOptions = { DisplayFunction -> Identity,
				 PlotLabel -> "Flip-and-Slide Convolution",
				 PlotRange -> plotrange,
				 PlotStyle -> plotstyles };

		If [ TrueQ[ $VersionNumber >= 2.0 ],
		     PrependTo[ graphOptions, Compiled -> False ] ];

		(* Display a legend and tell user that the animation is going *)
		fstr = "first function " ~StringJoin~ hstr;
		gstr = "second function " ~StringJoin~ xstr;
		Show[ { makeLegendLine[plotstyles, 1, fstr],
			makeLegendLine[plotstyles, 2, gstr],
			makeLegendLine[plotstyles, 3, "convolution result"] },
		      PlotLabel -> fontform["LEGEND", "Times", 18],
		      PlotRange -> {{-2, 2}, {-2, 2}} ];
		Message[ PiecewiseConvolution::animation ];

		(* Pick a numeric value for +/- Infinity end points *)
		endPoints = Join[fEndPoints, gEndPoints, cEndPoints];
		nEndPoints = Select[ N[endPoints], NumberQ ];
		minusInf = Min[nEndPoints];
		minusInf -= 0.5 Abs[minusInf];
		inf = Max[nEndPoints];
		inf += 0.5 Abs[inf];

		(*  Find the start and end points of each function  *)
		tfleft = Min[fEndPoints] /. ( -Infinity -> minusInf );
		tfright = Max[fEndPoints] /. ( Infinity -> inf );
		tgleft = Min[gEndPoints] /. ( -Infinity -> minusInf );
		tgright = Max[gEndPoints] /. ( Infinity -> inf );
		tcleft = Min[cEndPoints] /. ( -Infinity -> minusInf );
		tcright = Max[cEndPoints] /. ( Infinity -> inf );

		(*  Find the width of the first function and convolution  *)
		fwidth = tfright - tfleft;
		gwidth = tgright - tgleft;
		cwidth = tcright - tcleft;
		If [ SameQ[domain, Discrete], fwidth++; gwidth++; cwidth++ ];

		(*  Find the time interval to use for all of the plots  *)
		tmin = Min[tgleft - fwidth, tcleft];
		tmax = Max[tgright + fwidth, tcright]; 

		If [ SameQ[domain, Continuous],
		     tmin = tmin - 0.1 Abs[tmin];
		     tmax = tmax + 0.1 Abs[tmax],
		     tmin--;
		     tmax++ ];

		(*  Find the range to use for all plots  *)
		a = ConvertFromList[ fPiecewise, t, Domain -> domain ];
		b = ConvertFromList[ gPiecewise, t, Domain -> domain ];
		aPlot = plotfun[ a, {t, tmin, tmax},
				 DisplayFunction -> Identity,
				 PlotRange -> Automatic ];
		bPlot = plotfun[ b, {t, tmin, tmax},
				 DisplayFunction -> Identity,
				 PlotRange -> Automatic ];
		convPlot = PlotList[ conv, {t, tmin, tmax},
				     DisplayFunction -> Identity,
				     Domain -> domain,
				     PlotRange -> Automatic ];
		rangelist = Transpose[ { PlotRange[aPlot][[2]],
					 PlotRange[bPlot][[2]],
					 PlotRange[convPlot][[2]] } ];
		plotrange = { Min[ rangelist[[1]] ], Max[ rangelist[[2]] ] };

		(* Make a null plot to use later *)
		nullPlot1 = Apply[ plotfun,
				   Join[ { 0, {t, tmin, tmax},
				   	   PlotLabel -> overlapstr },
					 graphOptions ] ];
		nullPlot2 = Apply[ plotfun,
				   Join[ { 0, {t, tmin, tmax},
					   PlotLabel -> "Convolution Output" },
					 graphOptions ] ];

		(*  Plot both functions being convolved and hold for 5 frames *)
		plot = Apply[plotfun, {{a, b}, {t, tmin, tmax}, graphOptions}];
		If [ multiple,
		     plot = GraphicsArray[{{plot}, {nullPlot1}}] ];
		animationSequence = Table[ plot, {5} ];

		(*  Flip f so that it is next to g and hold for 4 frames  *)
		toffset = tgleft + tfleft;
		If [ SameQ[domain, Discrete], toffset-- ];
		aa = a /. t -> ( toffset - t );
		plot = Apply[plotfun, {{aa, b}, {t, tmin, tmax}, graphOptions}];
		If [ multiple,
		     plot = GraphicsArray[{{plot}, {nullPlot1}}] ];
		animationSequence = Join[animationSequence, Table[plot, {4}]];

		(*  Slide the first function across the second *)
		aa = a /. t -> (i + toffset - t);
		cc = ConvertFromList[ conv, t, Domain -> domain ];
		width = fwidth + gwidth;
		istep = width / frames;
		If [ SameQ[domain, Discrete],
		     istep = Floor[istep];
		     If [ istep == 0, istep = 1 ] ];
		Do [ plot = Apply[ plotfun,
				   {{aa, b, cc stepfun[i + toffset - t]},
				    {t, tmin, tmax}, graphOptions} ];
		     If [ multiple,
			  overlapstr =
			    ToString[StringForm[overlapctlstr, i + toffset]];
			  plot1 = Apply[ plotfun,
					 { (aa b),
					   {t, tmin, tmax},
					   PlotLabel -> overlapstr,
					   PlotStyle -> {RGBColor[1,0,1]},
					   PlotRange -> plotrange,
					   DisplayFunction -> Identity } ];
			  plot2 = Apply[ plotfun,
					 { (cc stepfun[i + toffset -t]),
					   {t, tmin, tmax},
					   PlotLabel -> "Convolution Output",
					   PlotStyle -> { Thickness[.0025],
							  RGBColor[0,0,1] },
					   PlotRange -> plotrange,
					   DisplayFunction -> Identity } ];
			  plot = GraphicsArray[{{plot}, {plot1}}] ];
		     AppendTo[ animationSequence, plot ],
		     {i, 0, width, istep} ];

		animationSequence = Join[animationSequence, Table[plot, {5}]];

		animationFunction[ animationSequence ];

		conv ]

(*  areaOfImpulse -- return the impulse area at a particular point *)
areaOfImpulse[list_, a_] :=
   Block[ {areaofimp = 0, i},
	  Do[ If [ Head[list[[i]][[1]]] == Area,
		   areaofimp += list[[i]][[1]][[1]] * impulseHere[list[[i]], a],
		   Null ],
	      {i, Length[list]} ];
	  areaofimp ]

(*  callFunction --  switch between discrete and continuous domains  *)
SetAttributes[callFunction, {HoldFirst}]

callFunction[ message_, discretefun_, continuousfun_, oplist_List, args___ ] :=
	Block [	{domain},
		domain = Replace[Domain, oplist];
		Which [	SameQ[domain, Discrete],
			  Apply[discretefun, {args}],
			SameQ[domain, Continuous],
			  Apply[continuousfun, {args}],
			True,
			  message ] ]

(*  ConvertToList routines  *)

Options[ConvertToList] := { Domain :> $ConvolutionDomain }

ConvertToList[f_, v_, options___] :=
	callFunction[ Message[ConvertToList::domain],
		      DTConvertToList, CTConvertToList,
		      ToList[options] ~Join~ Options[ConvertToList],
		      f, v ]

CTConvertToList[f_, t_] := CTSimplifyList[ unsimpCTL[f, t], t ]
DTConvertToList[f_, n_] := DTSimplifyList[ unsimpCTL[f, n], n ]

unsimpCTL[f_?IntervalQ, t_] := {f}
unsimpCTL[f_List, t_] := f
unsimpCTL[f_ + g_, t_] := Join[unsimpCTL[f,t], unsimpCTL[g,t]]
unsimpCTL[0, t_] := {}
unsimpCTL[Delta[t_ + a_.] ar_., t_] :=
	{{Area[ Limit[ar, t -> -a] ], -a, -a}} /; FreeQ[a,t]
unsimpCTL[Delta[-t_ + a_.] ar_., t_] :=
	{{Area[ Limit[ar, t -> a] ], a, a}} /; FreeQ[a,t]
unsimpCTL[Impulse[t_ + a_.] ar_., t_] :=
	{{Limit[ ar, t -> -a], -a, -a}} /; FreeQ[a,t]
unsimpCTL[Impulse[-t_ + a_.] ar_., t_] :=
	{{Limit[ar, t -> a], a, a}} /; FreeQ[a,t]
unsimpCTL[CStep[t_ + a_.] f_., t_] := {{f,-a,Infinity}} /; FreeQ[a, t]
unsimpCTL[CStep[-t_ + a_.] f_., t_] := {{f,-Infinity,a}} /; FreeQ[a, t]
unsimpCTL[Step[t_ + a_.] f_., t_] := {{f,-a,Infinity}} /; FreeQ[a, t]
unsimpCTL[Step[-t_ + a_.] f_., t_] := {{f,-Infinity,a}} /; FreeQ[a, t]
unsimpCTL[CPulse[l_, t_ + a_.] f_.,t_] := {{f, -a, l - a}} /; FreeQ[{l,a}, t]
unsimpCTL[CPulse[l_, -t_ + a_.] f_.,t_] := {{f, a - l, a}} /; FreeQ[{l,a}, t]
unsimpCTL[Pulse[l_, t_ + a_.] f_.,t_] := {{f, -a, l - 1 - a}} /; FreeQ[{l,a}, t]
unsimpCTL[Pulse[l_, -t_ + a_.] f_.,t_] := {{f, a - l + 1, a}} /; FreeQ[{l,a}, t]
unsimpCTL[f_ term_Plus, t_] := unsimpCTL[ Distribute[f term], t ] /;
	! FreeQ[term, t] && ! SameQ[f term, Distribute[f term]]
unsimpCTL[f_, t_] := {{f,-Infinity,Infinity}}

(*  ConvertFromList routines  *)

Options[ConvertFromList] := { Domain :> $ConvolutionDomain }

ConvertFromList[list_?VectorQ, v_, options___] :=
	ConvertFromList[ {list}, v, options ]

ConvertFromList[list_, v_, options___] :=
	callFunction[ Message[ConvertFromList::domain],
		      DTConvertFromList, CTConvertFromList,
		      ToList[options] ~Join~ Options[ConvertFromList],
		      list, v ]

CTConvertFromList[list_, t_] :=
	Apply[Plus, Map[CTConvertFromInterval[#1, t]&, list]]

CTConvertFromInterval[{Area[ar_], a_, a_}, t_] := ar Delta[t - a]
CTConvertFromInterval[{f_, -Infinity, Infinity}, t_] := f
CTConvertFromInterval[{f_, a_, Infinity}, t_] := f CStep[t - a]
CTConvertFromInterval[{f_, -Infinity, a_}, t_] := f CStep[a - t]
CTConvertFromInterval[{f_, a_, b_}, t_] := f CPulse[b - a, t - a]

DTConvertFromList[list_, t_] :=
	Apply[Plus, Map[DTConvertFromInterval[#1, t]&, list]]

DTConvertFromInterval[{Area[ar_], a_, a_}, t_] := ar Impulse[t-a]  
DTConvertFromInterval[{ar_, a_, a_}, t_] := ar Impulse[t-a]
DTConvertFromInterval[{f_, -Infinity, Infinity}, t_] := f
DTConvertFromInterval[{f_, a_, Infinity}, t_] := f Step[t - a]
DTConvertFromInterval[{f_, -Infinity, a_}, t_] := f Step[a - t]
DTConvertFromInterval[{f_, a_, b_}, t_] := f Pulse[b - a + 1, t - a]


(*  convolveTwoContinuousIntervals  *)
(*  -- algorithm to compute symbolic convolution [Evans, p. 68]   *)
(*  -- in the second if statement, t1 + t4 and t2 + t3 are safe,  *)
(*     even though t1 could be -Infinity and t2 could be Infinity *)
(*     because in Mathematica, Infinity + <anything> = Infinity   *)
convolveTwoContinuousIntervals[ fg_, t_, t1_, t2_, t3_, t4_, T_ ] :=
	Block [	{convlist},
		convlist = {};
		If [ ! SameQ[t1, -Infinity] && ! SameQ[t3, -Infinity],
		     AppendTo[ convlist,
			       { Integrate[fg, {T, t1, t - t3}],
				 t1 + t3, t1 + t4 } ] ];
		If [ ! SameQ[t3, -Infinity] && ! SameQ[t4, Infinity],
		     AppendTo[ convlist,
			       { Integrate[fg, {T, t - t4, t - t3}],
				 t1 + t4, t2 + t3 } ] ];
		If [ ! SameQ[t2, Infinity] && ! SameQ[t4, Infinity],
		     AppendTo[ convlist,
			       { Integrate[fg, {T, t - t4, t2}],
				 t2 + t3, t2 + t4 } ] ];

		(* f(t) and g(t) are one-sided in opposite directions *)
		If [ ! SameQ[t1, -Infinity] && SameQ[t2, Infinity] &&
		     SameQ[t3, -Infinity] && ! SameQ[t4, Infinity],
		     AppendTo[ convlist,
			       { Integrate[fg, {T, t1, Infinity}],
				 -Infinity, t1 + t4 } ];
		     AppendTo[ convlist,
			       { Integrate[fg, {T, t - t4, Infinity}],
				 t1 + t4, Infinity } ] ];

		(* Handle the six degenerate cases *)
		If [ EmptyQ[convlist],
		     convlist = { Integrate[ fg, {T, t1, t2} ],
				  -Infinity, Infinity } ];

		convlist ]

(*  convolveTwoDiscreteIntervals  *)
(*  -- algorithm to compute symbolic convolution [Evans, p. 68]   *)
(*  -- in the second if statement, n1 + n4 and n2 + n3 are safe,  *)
(*     even though n1 could be -Infinity and n2 could be Infinity *)
(*     because in Mathematica, Infinity + <anything> = Infinity   *)
convolveTwoDiscreteIntervals[ fg_, n_, n1_, n2_, n3_, n4_, T_ ] :=
	Block [	{a, b, convlist, sumhead},
		sumhead = If [ TrueQ[$VersionNumber >= 2.0],
			       Algebra`SymbolicSum`SymbolicSum,
			       Algebra`GosperSum`GosperSum ];
		convlist = {};
		If [ ! SameQ[n1, -Infinity] && ! SameQ[n3, -Infinity],
		     AppendTo[ convlist,
			       { Cancel[ sumhead[fg, {T, n1, n - n3}] ],
				 n1 + n3, n1 + n4 - 1 } ] ];
		If [ ! SameQ[n3, -Infinity] && ! SameQ[n4, Infinity],
		     AppendTo[ convlist,
			       { Cancel[ sumhead[fg, {T, n - n4, n - n3}] ],
				 n1 + n4, n2 + n3 } ] ];
		If [ ! SameQ[n2, Infinity] && ! SameQ[n4, Infinity],
		     AppendTo[ convlist,
			       { Cancel[ sumhead[fg, {T, n - n4, n2}] ],
				 n2 + n3 + 1, n2 + n4 } ] ];

		(* f[n] and g[n] are one-sided in opposite directions *)
		If [ ! SameQ[n1, -Infinity] && SameQ[n2, Infinity] &&
		     SameQ[n3, -Infinity] && ! SameQ[n4, Infinity],
		     AppendTo[ convlist,
			       { Cancel[ sumhead[fg, {T, n1, Infinity}] ],
				 -Infinity, n1 + n4 - 1 } ];
		     AppendTo[ convlist,
			       { Cancel[ sumhead[fg, {T, n - n4, Infinity}] ],
				 n1 + n4, Infinity } ] ];

		(* Handle the six degenerate cases *)
		If [ EmptyQ[convlist],
		     convlist = { Cancel[ sumhead[ fg, {T, n1, n2} ] ],
				  -Infinity, Infinity } ];

		(*  Fix problem with SymbolicSum  *)
		convlist /. 
		  ( Algebra`SymbolicSum`SymbolicSum[x_, {i_, i1_, i2_}] :>
		      ( Algebra`SymbolicSum`SymbolicSum[ x, {i, a, b} ] /.
			{ a -> i1, b -> i2 } ) ) ]

(*  convolveTwoIntervals  *)
effectiveLength[ -Infinity, Infinity ] = { Infinity, Infinity }
effectiveLength[ -Infinity, u_ ] := { Infinity, Indeterminate }
effectiveLength[ l_, Infinity ] := { Infinity, Indeterminate }
effectiveLength[ l_, u_ ] := { u - l, u - l }

outOfOrder[t1_, t2_, t3_, t4_] :=
	Block [	{len11, len12, len21, len22},
		{len11, len12} = effectiveLength[t1, t2];
		{len21, len22} = effectiveLength[t3, t4];
		TrueQ[ N[ len11 < len21 ] ] ||
		  TrueQ[ SameQ[len11, len21] &&
			 SameQ[len12, Indeterminate] &&
			 SameQ[len22, Infinity] ] ||
		  TrueQ[ SameQ[len11, len21] &&
			 SameQ[len12, len22, Indeterminate] &&
			 myLess[t1, t3] ] ]

(*
outOfOrder[t1_, t2_, t3_, t4_] :=
Block [	{allfour, nallfour},
	allfour = {t1, t2, t3, t4};
	nallfour = N[allfour];
	Which [ SameQ[Length[ Select[ nallfour, NumberQ ] ], 4],
		  N[(t4 - t3) > (t2 - t1)],
		SameQ[t1, -Infinity] && ! SameQ[t3, -Infinity],
		  True,
		SameQ[t1, t3, -Infinity] && TrueQ[t2 < t4],
		  True,
		True,
		  False ] ]
 *)

convolveTwoIntervals[pair_List, t_, time_] :=
	convolveTwoIntervals[ pair[[1]], pair[[2]], t, time]

convolveTwoIntervals[{Area[ar1_], a_, a_}, {Area[ar2_], b_, b_}, t_, time_] :=
	{{Area[ar1 ar2], a+b, a+b}} 
convolveTwoIntervals[{Area[ar_], a_, a_}, {f_, b_, c_}, t_, time_] :=
	{{ar (f /. t -> t-a), b+a, c+a}}
convolveTwoIntervals[{f_, a_, b_}, {Area[ar_], c_, c_}, t_, time_] :=
	{{ar (f /. t -> t-c), a+c, b+c}}
convolveTwoIntervals[{ar_, a_, a_}, {f_, b_, c_}, n_, Discrete] :=
	{{ar (f /. n -> n-a), b+a, c+a}}
convolveTwoIntervals[{f_, b_, c_}, {ar_, a_, a_}, n_, Discrete] :=
	{{ar (f /. n -> n-a), b+a, c+a}}

	(* Jan-92 JMc: make sure g(t) is smaller in extent; if f(t) and  *)
	(* g(t) have same extent, then make sure that g(t) is left-sided *)
convolveTwoIntervals[{f_, t1_, t2_}, {g_, t3_, t4_}, t_, time_] :=
	convolveTwoIntervals[{g, t3, t4}, {f, t1, t2}, t, time] /;
	outOfOrder[t1, t2, t3, t4]

	(* Rewrote after generalizing Square Matrix Rule *)
	(* validated in "misc_notebooks/exhaustiveConvTester" Notebook *)
	(* separated the computation so it could be used by m-D conv *)
convolveTwoIntervals[{f_, t1_, t2_}, {g_, t3_, t4_}, t_, Continuous] := 
	Block [	{f2, g2, T},
		f2 = f /. t -> T;
		g2 = g /. t -> t - T;
		convolveTwoContinuousIntervals[ f2 g2, t, t1, t2, t3, t4, T ] ]

	(* Rewrote after generalizing Square Matrix Rule *)
	(* validated in "misc_notebooks/exhaustiveConvTester" Notebook *)
	(* separated the computation so it could be used by m-D conv *)
convolveTwoIntervals[{f_, n1_, n2_}, {g_, n3_, n4_}, n_, Discrete] := 
	Block [	{fg, f2, g2, T},
		f2 = f /. n -> T;
		g2 = g /. n -> n - T;
		fg = Cancel[f2 g2];
		convolveTwoDiscreteIntervals[ fg, n, n1, n2, n3, n4, T ] ]

(*  ConvTwoLists  *)
pairIntervalAndList[int_, list_] := Map[ {int, #}&, list ]

CTConvTwoLists[a_, b_, v_] := ConvTwoLists[a, b, v, Continuous]
DTConvTwoLists[a_, b_, v_] := 
	Block [	{},
		If [ TrueQ[$VersionNumber >= 2.0],
		     Needs[ "Algebra`SymbolicSum`" ],
		     Needs[ "Algebra`GosperSum`" ] ];
		ConvTwoLists[a, b, v, Discrete] ]

ConvTwoLists[{}, b_, v_, time_] := {}
ConvTwoLists[a_, {}, v_, time_] := {}
ConvTwoLists[a_, b_, v_, time_] :=
	Block [	{convolution, groupedinto3s, nestedlist, pairedlists},
		nestedlist = Flatten[ Map[ pairIntervalAndList[#, b]&, a ] ];
		pairedlists = Partition[ Partition[nestedlist, 3], 2 ];
		convolution = Map[ convolveTwoIntervals[#, v, time]&,
				   pairedlists ];
		groupedinto3s = Simplify[ Partition[Flatten[convolution], 3] ];
		SimplifyList[ groupedinto3s, v, Domain -> time ] ]

(*  continuousEndPointsQ  *)
continuousEndPointsQ[ {f_, a_, b_} ] :=
	( NumberQ[N[a]] || SameQ[a, -Infinity] ) &&
	( NumberQ[N[b]] || SameQ[b, Infinity] )

(*  discreteEndPointsQ  *)
discreteEndPointsQ[ {f_, a_, b_} ] :=
	( IntegerQ[a] || SameQ[a, -Infinity] ) &&
	( IntegerQ[b] || SameQ[b, Infinity] )

(*  impulseHere -- return 1 if the points coincide *)
impulseHere[{Area[ar_], a_, a_}, a_] := 1
impulseHere[___] := 0

(*  inIntervalQ; inInterval returns a 1 if the intervals cross  *)    
inIntervalQ[{f_, left_, right_}, a_, b_] :=
	TrueQ[ N[myLessEqual[left, a, b, right]] ]
inInterval[args__] := If [ inIntervalQ[args], 1, 0 ]

(*  IntervalQ  *)
IntervalQ[{f_, a_, b_}] := ! ( ListQ[f] || ListQ[a] || ListQ[b] )
IntervalQ[___] := False

(*  listOfEndPoints  *)
getsecthird[ {a_, b_, c_} ] := ToCollection[b, c]

listOfEndPoints[list_List] := Sort[ Union[ Map[getsecthird, list] ], myLess ]

(*  myLess  *)
myLess[a_, a_] := False	
myLess[-Infinity, Infinity] := True
myLess[Infinity, -Infinity] := False
myLess[Infinity, a_] := False
myLess[a_, Infinity] := True
myLess[a_, -Infinity] := False
myLess[-Infinity, a_] := True
myLess[a_, b_] := N[a < b]
myLess[a_, b_, rest__] := myLess[a, b] && myLess[b, rest]

(*  myLessEqual  *)
myLessEqual[a_, a_] := True
myLessEqual[a_, b_] := myLess[a, b]
myLessEqual[a_, b_, rest__] := myLessEqual[a, b] && myLessEqual[b, rest]

(*  recursive continuous symmetric pulse convolution  *)
recursivePulseConvolution[ 1 ] =
  {1, -a, a}
recursivePulseConvolution[ 2 ] =
  { {w + 2 a, -2 a, 0},
    {2 a - w, 0, 2 a} }
recursivePulseConvolution[ 3 ] = 
  { { (3 a + w)^2/2, -3 a, -a},
    {   3 a^2 - w^2,  -a,  a},
    {(-3 a + w)^2/2,   a, 3 a} }
recursivePulseConvolution[ 4 ] = 
  { {((4 a + w)^3)/6, -4 a, -2 a},
    {(16 a^3)/3 - 2 a w^2 - 1/2 w^3, -2 a, 0}, 
    {2 a^2 w - a w^2 + 1/6 w^3 + (16 a^3 - 6 a^2 w - 3 a w^2 + w^3)/3, 0, 2 a}, 
    {((4 a - w)^3)/6, 2 a, 4 a} }

recursivePulseConvolution[ n_ ] :=
  stripAndCleanup[
    Simplify[
      CTPiecewiseConvolution[
	recursivePulseConvolution[n-1],
	recursivePulseConvolution[1],
	w ] ] ]

(*  removeBadInterval--- remove completely invalid intervals  *)
keepRealIntervalQ[ {Area[ar_], a_, b_} ] := False /; ZeroQ[ar] || ! Same[a, b]
keepRealIntervalQ[ {f_?ZeroQ, a_, b_} ] := False
keepRealIntervalQ[ x_ ] := True

keepIntIntervalQ[ {Area[ar_], a_, b_} ] := False /; ZeroQ[ar] || ! Same[a, b]
keepIntIntervalQ[ {f_?ZeroQ, a_, b_} ] := False
keepIntIntervalQ[ {f_, a_?NumberQ, b_} ] := False /; ! IntegerQ[a]
keepIntIntervalQ[ {f_, a_, b_?NumberQ} ] := False /; ! IntegerQ[b]
keepIntIntervalQ[ x_ ] := True

myCeiling[x_Real] := If [ x < 0, -Floor[-x], Ceiling[x] ]
myCeiling[x_] := x
myFloor[x_Real] := If [ x < 0, -Ceiling[-x], Floor[x] ]
myFloor[x_] := x

passIt[-Infinity, fun_] := -Infinity
passIt[a_Integer, fun_] := a
passIt[a_Real, fun_] := fun[a]
passIt[a_, fun_] := fun[N[a]] /; NumberQ[N[a]]
passIt[a_, fun_] := a

toIntegerEndPoints[ {f_, a_, b_} ] :=
	{ f, passIt[a, myCeiling], passIt[b, myFloor] }

removeBadInterval[oldlist_, Continuous] := Select[oldlist, keepRealIntervalQ]
removeBadInterval[oldlist_, Discrete] :=
	Select[ Map[toIntegerEndPoints, oldlist], keepIntIntervalQ ]

(*  removeDomain  *)
keepq[x_] := SameQ[Domain, Replace[Domain, x]]
removeDomain[oplist_] := Select[oplist, keepq]

(*  stripAndCleanup  *)
stripAndCleanup[ lists_ ] :=
	Block [	{entry, f, left, newlist, oldlist, position, right},
		newlist = {};
		oldlist = lists;
		While [ ! EmptyQ[oldlist],
			{f, left, right} = entry = First[oldlist];
			oldlist = Rest[oldlist];
			If [ ! SameQ[left, right],
			     position = Position[newlist, {_, left, right}];
			     If [ EmptyQ[position],
				  newlist = Append[newlist, entry],
				  newlist[[ position[[1,1]], 1 ]] += f ] ] ];
		newlist ]

(*  valueOfInterval -- return the value of a particular interval  *)
valueOfInterval[list_, a_, b_] :=
	Apply[Plus, Map[First, Select[list, inIntervalQ[#1, a, b]&]]]

(*  ValidIntervalQ  *)
Options[ValidIntervalQ] := { Domain :> $ConvolutionDomain }

ValidIntervalQ[list_, options___] := 
	callFunction[ Message[ValidIntervalQ::domain],
		      DTValidIntervalQ, CTValidIntervalQ,
		      ToList[options] ~Join~ Options[ValidIntervalQ],
		      list ]

CTValidIntervalQ[i_] := False /; Not[IntervalQ[i]]
CTValidIntervalQ[{Area[ar_], a_, b_}] := SameQ[a,b] && NumberQ[a] && !ZeroQ[ar]
CTValidIntervalQ[{f_, a_, b_}] := TrueQ[ N[(b > a) && ! SameQ[f, 0]] ]

DTValidIntervalQ[i_] := False /; Not[IntervalQ[i]]
DTValidIntervalQ[{Area[ar_], a_, b_}] :=
	SameQ[a, b] && (! InfinityQ[b]) && (! SameQ[ar, 0])
DTValidIntervalQ[{f_, a_, a_}] := (! InfinityQ[a]) && (! SameQ[f, 0])
DTValidIntervalQ[{f_, a_, b_}] := TrueQ[ N[(b >= a) && (! SameQ[f, 0])] ]


(*  C O N V O L U T I O N     R O U T I N E S  *)


(*  AutoCorrelation  *)

Options[AutoCorrelation] := { Domain :> $ConvolutionDomain }

AutoCorrelation[f_, v_, options___] :=
	callFunction[ Message[AutoCorrelation::domain],
		      DTPiecewiseConvolution, CTPiecewiseConvolution,
		      ToList[options] ~Join~ Options[AutoCorrelation],
		      f, flipInTime[f, v], v ]

flipInTime[int_, v_] := flipInterval[int, v] /; IntervalQ[int]
flipInTime[list_List, v_] := Reverse[ Map[flipInterval[#1, v]&, list] ]
flipInTime[f_, v_] := f /. v -> -v

flipInterval[{f_, v1_, v2_}, v_] := {(f /. v -> -v), -v2, -v1}

(*  PiecewiseConvolution  *)
Options[PiecewiseConvolution] :=
	{ Dialogue -> False,
	  Domain :> $ConvolutionDomain,
	  ConvolutionOutputForm -> Piecewise }

PiecewiseConvolution[f_, g_, v_, options___] :=
	Block [	{oplist},
		oplist = ToList[options] ~Join~ Options[PiecewiseConvolution];
		callFunction[ Message[PiecewiseConvolution::domain],
			      DTPiecewiseConvolution, CTPiecewiseConvolution,
			      oplist, f, g, v, oplist ] ]

PiecewiseConvolution[___] := Message[PiecewiseConvolution::argct]

(*  CTPiecewiseConvolution  *)
CTPiecewiseConvolution[f_, g_, t_, options___] :=
	Block [	{animate, conv, fPiecewise, gPiecewise, oplist, outputform},

		(*  Parse the options passed into the routine  *)
		oplist = ToList[options];
		animate = InformUserQ[oplist];
		outputform = Replace[ ConvolutionOutputForm, oplist ];
		If [ SameQ[outputform, Automatic],
		     If [ ! SameQ[Head[f], List] || ! SameQ[Head[g], List],
			  outputform = Formula,
			  outputform = Piecewise ] ];

		(*  Compute and animate (if enabled) the convolution *)
		fPiecewise = CTConvertToList[f, t];
		gPiecewise = CTConvertToList[g, t];
		conv = CTConvTwoLists[ fPiecewise, gPiecewise, t ];
		If [ animate,
		     animateConvolution[ fPiecewise, gPiecewise, conv,
					 t, Continuous, oplist ] ];

		(* return the form of the answer desired by the user *)
		If [ SameQ[outputform, Formula],
		     conv = ConvertFromList[conv, t, Domain->Continuous] ];

		conv ]

CTPiecewiseConvolution[___] := Message[CTPiecewiseConvolution::argct]

(*  DTPiecewiseConvolution  *)
DTPiecewiseConvolution[f_, g_, n_, options___] :=
	Block [	{animate, conv, fPiecewise, gPiecewise, oplist, outputform},

		(*  Parse the options passed into the routine  *)
		oplist = ToList[options];
		animate = InformUserQ[oplist];
		outputform = Replace[ ConvolutionOutputForm, oplist ];
		If [ SameQ[outputform, Automatic],
		     If [ ! SameQ[Head[f], List] || ! SameQ[Head[g], List],
			  outputform = Formula,
			  outputform = Piecewise ] ];

		(*  Compute and animate (if enabled) the convolution *)
		fPiecewise = DTConvertToList[f, n];
		gPiecewise = DTConvertToList[g, n];
		conv = DTConvTwoLists[ fPiecewise, gPiecewise, n ];
		If [ animate,
		     animateConvolution[ fPiecewise, gPiecewise, conv,
					 n, Discrete, oplist ] ];

		(* return the form of the answer desired by the user *)
		If [ SameQ[outputform, Formula],
		     conv = ConvertFromList[conv, n, Domain->Discrete] ];

		conv ]

DTPiecewiseConvolution[___] := Message[DTPiecewiseConvolution::argct]

(* SymmetricPulseConvolutions *)
SymmetricPulseConvolutions[ scale_, varscale_, var_, n_Integer ] :=
	Block [	{conv},
		conv = recursivePulseConvolution[n] /.
			 { a -> varscale, w -> var };
		(scale^n) ConvertFromList[ conv, var ] ] /;
	( n > 0 )


(*  P L O T T I N G     R O U T I N E S  *)

(*  PlotList  *)
Options[PlotList] := { Domain :> $ConvolutionDomain }

PlotList[list_, {v_, v1_, v2_}, options___] :=
	Block [	{oplist},
		oplist = ToList[options] ~Join~ Options[PlotList];
		callFunction[ Message[PlotList::domain],
			      DTPlotList, CTPlotList, oplist,
			      list, {v, v1, v2}, removeDomain[oplist] ] ]

(* CTPlotList  *)
CTPlotList[list_List, {t_, t1_, t2_}, options___] :=
	SignalPlot[CTConvertFromList[list, t], {t, t1, t2}, options]
CTPlotList[fun_, {t_, t1_, t2_}, options___ ] :=
	SignalPlot[fun, {t, t1, t2}, options]

(* DTPlotList *)
DTPlotList[list_List, {n_, n1_, n2_}, options___] :=
	SequencePlot[DTConvertFromList[list, n], {n, n1, n2}, options]
DTPlotList[fun_, {n_, n1_, n2_}, options___] :=
	SequencePlot[fun, {n, n1, n2}, options]


(*  S I M P L I F I C A T I O N     R O U T I N E S  *)

(*  SimplifyList  *)
Options[SimplifyList] := { Domain :> $ConvolutionDomain }

SimplifyList[f_?VectorQ, v_, options___] :=
	SimplifyList[ {f}, v, options ]

SimplifyList[{{}}, v_, options___] := {}

SimplifyList[list_, v_, options___] :=
	callFunction[ Message[SimplifyList::domain],
		      DTSimplifyList, CTSimplifyList,
		      ToList[options] ~Join~ Options[SimplifyList],
		      list, v ]

(*  CTSimplifyList  *)
CTSimplifyList[list_, t_] :=
	Block [	{funsSymbolicEndPoints, funsNumericEndPoints, validfuns},
		validfuns = removeBadInterval[list, Continuous];
		funsNumericEndPoints =
			Select[ validfuns, continuousEndPointsQ ];
		funsSymbolicEndPoints =
			Select[ validfuns, Not[continuousEndPointsQ[#]]& ];
		CTSimplifyList[funsNumericEndPoints, funsSymbolicEndPoints, t] ]

CTSimplifyList[{}, extra_, t_] := extra
CTSimplifyList[list_, extra_, t_] :=
	Block[ { curpoint, k, nextpoint, points, simplelist},
		points = listOfEndPoints[list];		
		nextpoint = points[[1]];
		simplelist = {{ Area[areaOfImpulse[list, nextpoint]],
				nextpoint, nextpoint }};
		Do [ curpoint = nextpoint;
		     nextpoint = points[[k]];
		     AppendTo[ simplelist,
			       { valueOfInterval[list, curpoint, nextpoint],
				 curpoint, nextpoint } ];
		     AppendTo[ simplelist,
			       { Area[areaOfImpulse[list, nextpoint]],
	       			 nextpoint, nextpoint } ],
		     {k, 2, Length[points]} ];
		Join[ removeBadInterval[ SPSimplify[simplelist,
						    Variables -> ToList[t]],
					 Continuous],
		      extra ] ]

(*  DTSimplifyList  *)
openList[{Area[A_], a_, a_}] := {A, a, a+1}
openList[{Area[A_], a_, b_}] := {} /; ! SameQ[a, b]
openList[{f_, a_, b_}] := {f, a, b+1}
closeList[{f_, a_, b_}] := {f, a, b-1}

DTSimplifyList[list_, n_] :=
	Block [	{funsSymbolicEndPoints, funsNumericEndPoints, validfuns},
		validfuns = removeBadInterval[list, Discrete];
		funsNumericEndPoints =
			Select[ validfuns, discreteEndPointsQ ];
		funsSymbolicEndPoints =
			Select[ validfuns, Not[discreteEndPointsQ[#]]& ];
		DTSimplifyList[funsNumericEndPoints, funsSymbolicEndPoints, n] ]

DTSimplifyList[{}, extra_, n_] := extra
DTSimplifyList[list_, extra_, n_] :=
	Block [	{curpoint, ival, k, lastpoint, nextpoint, numpoints,
		 openlist, points, simplelist = {}, simplified},

		openlist = Map[openList, list];
		points = listOfEndPoints[openlist];
		numpoints = Length[points];
		nextpoint = points[[1]];
		Do [ curpoint = nextpoint;
		     nextpoint = points[[k]];
		     ival = valueOfInterval[openlist, curpoint, nextpoint];
		     AppendTo[ simplelist, {ival, curpoint, nextpoint} ],
		     {k, 2, numpoints} ];
		simplified = SPSimplify[ Map[closeList, simplelist],
					 Variables -> {n} ];
		simplelist = removeBadInterval[simplified, Discrete];

		openlist = {};
		While [	Length[simplelist] > 0,
			curpoint = First[ simplelist ];
			simplelist = Rest[ simplelist ];

			If [ ! SameQ[ curpoint[[2]], curpoint[[3]] ],
			     AppendTo[ openlist, curpoint ];
			     curpoint = {},

			     If [ Length[openlist] > 0,
				  lastpoint = Last[openlist];
				  If [ SameQ[curpoint[[2]], 1+lastpoint[[3]]] &&
				       SameQ[curpoint[[1]] /. n->curpoint[[2]],
					   lastpoint[[1]] /. n->curpoint[[2]]],
				       openlist = Append[ Drop[openlist, -1],
							  { lastpoint[[1]],
							    lastpoint[[2]],
							    curpoint[[2]] } ];
				       curpoint = {} ] ];

			     If [ Length[simplelist] > 0 &&
				    Length[curpoint] > 0,
				  nextpoint = First[simplelist];
				  If [ SameQ[curpoint[[2]], nextpoint[[2]]-1] &&
				       SameQ[curpoint[[1]] /. n->curpoint[[2]],
					  nextpoint[[1]] /. n->curpoint[[2]]],
				       AppendTo[ openlist,
						 { nextpoint[[1]],
						   nextpoint[[2]] - 1,
						   nextpoint[[3]] } ];
				       simplelist = Rest[simplelist];
				       curpoint = {} ] ];

			     If [ Length[curpoint] > 0,
				  AppendTo[openlist, curpoint] ] ] ];

		lastpoint = Last[openlist];
		While [ SameQ[First[lastpoint] /. n -> lastpoint[[3]], 0] &&
			! SameQ[lastpoint[[3]], Infinity],
			openlist = If [ SameQ[lastpoint[[2]], lastpoint[[3]]],
					Drop[openlist, -1],
		    			Append[ Drop[openlist, -1],
						{ lastpoint[[1]],
						  lastpoint[[2]],
						  lastpoint[[3]] - 1 } ] ];
			lastpoint = Last[ openlist ] ];

		SPSimplify[ Join[openlist, extra], Variables -> {n} ] ]

(*  SetConvolutionDomain  *)
SetConvolutionDomain[domain_] :=
	Block [	{},
		Unprotect[ $ConvolutionDomain ];
		$ConvolutionDomain = domain;
		Protect[ $ConvolutionDomain ];
		domain ] /;
	SameQ[domain, Discrete] || SameQ[domain, Continuous]

SetConvolutionDomain[bogusdomain_] := Message[SetConvolutionDomain::domain]

(*  VectorConvolution  *)
(*  -- Algorithm is linear in memory used and quadratic in execution time  *)
(*  -- {The Mathematica Journal} vol. 2 no. 2 offers a version that is     *)
(*     quadratic in memory used and execution time.			   *)
(*  -- For the convolution of a 100-element vector and a 1000-element one, *)
(*     the timing difference on a Mac Quadra was 85.3 sec vs. 68.3 sec	   *)
(*  -- Although the MJ implementation was faster, it used too much memory. *)
VectorConvolution[ vector1_?VectorQ, vector2_?VectorQ ] :=
	vectorconvolution[ vector1, vector2, Length[vector1], Length[vector2] ]

vectorconvolution[ vector1_, vector2_, len1_, len2_ ] :=
	Block [	{h, i, len, x, y},
		len = len1 + len2 - 1;
		h = Reverse[vector2] ~Join~ Table[0, {len1 - 1}];
		x = Table[0, {len2 - 1}] ~Join~ vector1;
		y = Table[0, {len}];
		For [ i = 1, i <= len, i++,
		      y[[i]] = x . h;
		      h = RotateRight[h] ];
		y ]


(*  E N D     P A C K A G E  *) 

End[]
EndPackage[]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell1 ];
     On[ General::spell ] ]


(*  A L I A S E S  *)

FormulaToPiecewise = ConvertToList
FormulaToPiecewise::usage = ConvertToList::usage

PiecewisePlot = PlotList
PiecewisePlot::usage = PlotList::usage

PiecewiseToFormula = ConvertFromList
PiecewiseToFormula::usage = ConvertFromList::usage


(*  H E L P     I N F O R M A T I O N  *)

Block [	{newfuns},
	 newfuns =
	 { AutoCorrelation,		ConvertFromList,
	   ConvertToList,		CTPiecewiseConvolution,
	   DTPiecewiseConvolution,	FormulaToPiecewise,
	   IntervalQ,			PiecewiseConvolution,
	   PiecewisePlot,		PiecewiseToFormula,
	   PlotList,			SimplifyList,
	   ValidIntervalQ };
	 Combine[ SPfunctions, newfuns ];
	 Apply[ Protect, newfuns ];
	 Protect[ ConvolutionOutputForm, Formula, Piecewise ] ]


(*  E N D I N G     M E S S A G E  *)

Print["Piecewise convolution rules have been loaded."]
Null
