(*  :Title:	Deriving Multidimensional Decimation Systems  *) 

(*  :Authors:	Brian Evans and James McClellan  *)

(*
    :Summary:	To provide the ability to derive multirate systems to
		decimate a signal with an arbitrary bandpass shape
		described by a convex polygon.
 *)

(*  :Context:	SignalProcessing`Support`Decimation`  *)

(*  :PackageVersion:  SPP 2.9  @(#)Decimation.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*
    :Source:	Chen and Vaidyanathan. "On the Choice of Rational Decimation
		Systems for Multidimensional Signals."  ICASSP '93.  vol. V.
		pp. 527-530.

		Dudgeon and Mersereau. {Multidimensional Signal Processing}.
		1984.  ISBN 0-13-604959-1.  400 pages.
 *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*  :Discussion:  *)

(*
    :Functions:	DesignDecimationSystem
		DownsamplingAliasing
		FactorRationalResamplingMatrix
		FindBoundingRegion
		FindMinimumRectangle
		ValidDecimatorBasebandQ
 *)


(*  B E G I N     P A C K A G E  *)

BeginPackage [	"SignalProcessing`Support`Decimation`",
		"SignalProcessing`Support`LatticeTheory`",
		"SignalProcessing`Support`Polygons`",
		"SignalProcessing`Support`Lisp`",
		"SignalProcessing`Support`SupCode`" ]


If [ TrueQ[ $VersionNumber >= 2.0 ],
     Off[ General::spell ];
     Off[ General::spell1 ] ]

			
(*  U S A G E     I N F O R M A T I O N  *)

DesignDecimationSystem::usage =
	"DesignDecimationSystem[Polygon[pointlist]] returns the decimation \
	system to resample the bandpass signal (which is a convex hull \
	represented by the polygon) at the Nyquist rate. \
	The complete decimation system consists of four parts in cascade: \
	modulator, upsampler, filter, and downsampler. \
	The modulator is defined as Exp[I shiftVector . timeIndices ]. \
	This routine returns a list: \
	{ shiftVector, upsampling matrix, downsampling matrix }. \
	The Mod option controls the grid size and therefore the size \
	of the entries of the resampling matrices. \
	DesignDecimationSystem[ratmat] converts the rational resampling \
	matrix into a decimation system."

DownsamplingAliasing::usage =
	"DownsamplingAliasing[M, polylist] graphically illustrates the effect \
	of downsampling by the integer matrix M on the frequency domain \
	of the input signal represented as the union of polygons expressed \
	as the list of polygons polylist. \
	The input and output frequency domains, although representing \
	different sampling grids, are both graphed in the fundamental \
	tile -Pi < w1 < Pi and -Pi < w2 < Pi. \
	The Dialogue option controls how much analysis will be performed: \
	False for none, True for some, and All for all."

FactorRationalResamplingMatrix::usage =
	"FactorRationalResamplingMatrix[ratmat] factors the non-singular \
	rational matrix into two matrix factors L^-1 M where L and M are \
	both non-singular integer matrices."

FindBoundingRegion::usage =
	"FindBoundingRegion[majorAxis, polygon] finds the smallest rectangle \
	enclosing the convex hull represented by polygon with the line \
	majorAxis as the rectangle's major axis."

FindMinimumRectangle::usage =
	"FindMinimumRectangle[ Polygon[pointlist] ] finds the rectangle of \
	minimum area that circumscribes the convex hull represented as a \
	polygon. \
	The rectangle, which may be rotated about an axis, is returned \
	as a polygon."

ValidDecimatorBaseband::usage =
	"ValidDecimatorBaseband[ Polygon[pointlist] ] determines whether or \
	not the baseband represented by the polygon is a valid passband \
	for a integer decimator structure. \
	The routine returns { valid, passband, matrix } where valid \
	is True or False, passband is the baseband whose centroid has been \
	shifted to the origin, and matrix is the downsampling matrix \
	of the decimator."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin[ "`Private`" ]


(*  C O N S T A N T S  *)

DesignDecimationSystem::mod =
	"The Mod option is neither an integer or a pair of integers: ``. \
	The routine will use a default value of ``."

DesignDecimationSystem::badpoint =
	"The point `` occurs multiple times in the polygon ``."

(* piTickMarks--- give the tick marks for the 2-D fundamental frequency tile *)
(* each entry is has the form loc or {loc, str}.  A tick mark will be placed *)
(* at location loc on the axis; str will be printed next to the tick mark    *)
piTickMarks =
	Block [	{i, j, negpi, pospi, zero},		
		If [ TrueQ[ $VersionNumber >= 2.0 ],
		     pospi = FontForm["p", {"Symbol", 14}];
		     zero  = FontForm["0", {"Symbol", 14}];
		     negpi = FontForm["-p", {"Symbol", 14}],
		     pospi = "Pi";
		     zero  = "0";
		     negpi = "-Pi" ];
		Join[ {{-Pi, negpi}},
		      Table[ {(-1 + i/10) Pi, ""}, {i, 1, 9} ],
		      {{0, zero}},
		      Table[ {j Pi / 10, ""}, {j, 1, 9} ],
		      {{Pi, pospi}} ] ]


(*  L O C A L     F U N C T I O N S  *)

(* aliasedRegions  *)
aliasedRegions[ polygonList_ ] :=
Block [	{i, j, numPoly, poly, region},
	region = {};
	numPoly = Length[polygonList];
	For [ i = 1, i <= numPoly, i++,
	      For [ j = i+1, j <= numPoly, j++,
		    poly = ConvexPolygonIntersection[ polygonList[[i]],
						      polygonList[[j]] ];
		    If [ Length[poly] >= 3, AppendTo[region, poly] ] ] ];
	region ]

(* approxVertex *)
SetAttributes[ approxVertex, Listable ]

approxVertex[ r_Integer, incflag_, mod_ ] := r
approxVertex[ r_Rational, incflag_, mod_ ] := r
approxVertex[ r_, True, mod_ ] := Ceiling[mod N[r]] / mod
approxVertex[ r_, False, mod_ ] := Floor[mod N[r]] / mod 

(*  clockwiseOrder  *)
clockwiseOrder[ pointlist_ ] :=
Block [ {i, insort, len, newpointlist},
        newpointlist = {};
        len = Length[pointlist];
	insort = SignalProcessing`Support`Polygons`Private`insertionSort;
        For [ i = 1, i <= len, i++,
              newpointlist = insort[ newpointlist, pointlist[[i]] ] ];
        newpointlist ]

(* designDialogue *)
designDialogue[ dialogue_, center_, polyPoints_, h_,
		Polygon[minRectPoints_], options___ ] :=
Block [	{bestPacking, dark, efficiency, halt, light, minArea, minRect,
	 minRectList, polyArea, poly, rectPolyList, rectArea, thickness,
	 tileCorners},

	tileCorners = {{-Pi, -Pi}, {Pi, Pi}};
	light = RGBColor[1, 1/2, 0];
	dark = RGBColor[0, 0, 0];
	thickness = Thickness[0.006];
	poly = Polygon[polyPoints];
	polyArea = PolygonArea[poly];
	minRect = Polygon[minRectPoints];
	minArea = PolygonArea[minRect];
	bestPacking = N[ Round[1000*polyArea/minArea] / 10 ];

	If [ SameQ[dialogue, All],
	     Print[ "Best packing efficiency with rotated rectangle having ",
		    "real-valued coordinates: ", bestPacking, "%" ];
	     minRectList =
		 rectModWithFundamentalTile[N[minRectPoints], N[tileCorners]];
	     Show[ Graphics[ { Prepend[minRectList, dark],
			       {light, poly} } ],
		   options, AspectRatio -> 1, Axes -> True, Frame -> True,
		   FrameTicks -> {piTickMarks, piTickMarks},
		   PlotRange -> {{-Pi, Pi}, {-Pi, Pi}} ] ];

	halt = Pi Transpose[Inverse[h]];
	rectPoints =
		{ halt . {-1, -1} + center,
		  halt . {-1,  1} + center,
		  halt . { 1,  1} + center,
		  halt . { 1, -1} + center };
	rectPolyList = rectModWithFundamentalTile[rectPoints, tileCorners];
	rectArea = PolygonArea[ Polygon[ rectPoints ] ];
	efficiency = N[ Round[1000*polyArea/rectArea] / 10 ];
	Print[ "Actual packing efficiency: ", efficiency, "%  ",
	       "(out of a best possible ", bestPacking, "%)" ];

	(* must group color with polygon so that skeleton polyOutline shows *)
	Show[ Graphics[ { Prepend[rectPolyList, dark],
			  {light, poly} } ],
	      options, AspectRatio -> 1, Axes -> True, Frame -> True,
	      FrameTicks -> {piTickMarks, piTickMarks},
	      PlotRange -> {{-Pi, Pi}, {-Pi, Pi}} ] ]

(*  evaluateTiling  *)
evaluateTiling[ l_, factor_ ] :=
Block [	{lact, str},
	lact = If [ IntegerQ[l], l, Round[ N[l] ] ];
	str = Which [ lact == factor,
			"resamples the baseband signal at its Nyquist rate \
			(assuming that no aliasing is present)",
		      factor > lact,
			"introduces aliasing in the baseband signal",
		      True,
			"does not map the baseband signal onto the entire \
			frequency domain." ];
	Print[ StringJoin[ "Initial analysis: the downsampler ", str, "." ] ] ]

evaluateTiling[ l_, factor_, intraflag_, interflag_, area_ ] :=
Block [	{aliasflag, andstr, bothstr, interstr, intrastr, lact},
	aliasflag = intraflag || interflag;
	lact = If [ IntegerQ[l], l, Round[ N[l] ] ];
	If [ aliasflag,
	     bothstr  = If [ intraflag && interflag, "both ", "" ];
	     andstr   = If [ intraflag && interflag, "and ", "" ];
	     intrastr = If [ intraflag, "intra-band ", "" ];
	     interstr = If [ interflag, "inter-band ", "" ];
	     Print[ "The downsampler introduces ", bothstr, intrastr,
		    andstr, interstr, "aliasing." ];
	     Print[ "Aliasing occurs in ",
		    N[ Round[ N[1000 area / (4 Pi^2)] ] / 10 ],
		    "% of the frequency domain." ] ];
	If [ ! aliasflag,
	     Print[ "The downsampler does not introduce aliasing." ];
	     If [ factor == lact,
		  Print[ "It resamples the baseband at the Nyquist rate." ] ];
	     If [ factor < lact,
		  Print[ "However, the downsampler does not map the baseband\
			 signal onto the entire frequency domain.  The\
			 baseband could be downsampled by a factor as high\
			 as ", lact ] ] ] ]

(* findBoundingRectangle *)
findBoundingRectangle[ Polygon[poly_] ] :=
Block [	{trans, xmin, xmax, ymin, ymax},
	trans = Transpose[poly];
	{xmin, ymin} = Map[ Min, trans ];
	{xmax, ymax} = Map[ Max, trans ];
	Polygon[ {{xmin, ymin}, {xmin, ymax}, {xmax, ymax}, {xmax, ymin}} ] ]

(* findCenter *)
findCenter[ pointlist_ ] :=
	Apply[ Plus, pointlist ] / Length[pointlist]
findCenter[ pointlist_, numpoints_ ] :=
	Apply[ Plus, pointlist ] / numpoints

(* findResamplingMatrix--- assume that the corners in rectangle are in order *)
findResamplingMatrix[ Polygon[ { pt1_, pt2_, pt3_, pt4_ } ] ] :=
Block [	{Anorm},
	Anorm = Transpose[ {pt1, pt2} ] . Inverse[ {{-1,-1}, {-1,1}} ];
	Inverse[Transpose[Anorm]] ] /;
ParallelogramQ[ Polygon[ {pt1, pt2, pt3, pt4} ] ] &&
  N[ findCenter[{pt1, pt2, pt3, pt4}] == {0, 0} ]

(* idealParallelogramList *)
idealParallelogramList[ { pt1_, pt2_, pt3_, pt4_ }, mod_ ] :=
Block [	{adjustment, curPolyList, diagSlope1, diagSlope2,
	 floatPointVertices, insideFlag, mypt1, mypt2, mypt3, mypt4,
	 newpt1, newpt2, newpt3, newpt4},

	(* put rectangle vertices in order: upper left,  *)
	(*   upper right, lower right, and lower left.   *)
	floatPointVertices =
	    {newpt1, newpt2, newpt3, newpt4} =
		parallelogramSort[ {pt1, pt2, pt3, pt4} ];

	(* compute new vertices for the grid to make a parallelogram *)
	mypt1 = approxVertex[ newpt1, { False, True}, mod ];
	mypt2 = approxVertex[ newpt2, { True,  True}, mod ];
	mypt3 = approxVertex[ newpt3, { True,  False}, mod ];
	mypt4 = mypt3 - ( mypt2 - mypt1 );	(* parallelogram rule *)

	(* see if each floating-point vertex is inside the current *)
	(* polygon which now has rational-valued vertices *)
	adjustment = {{0, -1/mod[[2]]}, {-1/mod[[1]], 0},
		      {0,  1/mod[[2]]}, { 1/mod[[1]], 0}};
	curPolyList = { mypt1, mypt2, mypt3, mypt4 };
	insideFlag =
	  Apply[ And,
		 Map[ InsidePolygonQ[Polygon[curPolyList], Point[#], True]&,
		      floatPointVertices ] ];
	If [ ! insideFlag,
	     curPolyList += adjustment;
	     insideFlag =
	       Apply[ And,
		 Map[ InsidePolygonQ[Polygon[curPolyList], Point[#], True]&,
		      floatPointVertices ] ];
	     If [ ! insideFlag, curPolyList -= adjustment ] ];

	curPolyList ]

(* intrabandDialogue *)
intrabandDialogue[aliasing_, graphics_, graphoptions_, i_] :=
Block [	{bandstr, colorstr, posnegstr},
	If [ EmptyQ[aliasing],
	     posnegstr = " is free of"; colorstr = ".",
	     posnegstr = " does suffer from"; colorstr = " (shown in black)." ];
	bandstr = StringJoin[ "Band #", ToString[i], " of ", ToString[factor] ];
	Print[ bandstr, posnegstr, " intra-band aliasing", colorstr ];
	Apply[ Show, Join[{graphics}, graphoptions, {PlotLabel -> bandstr}] ] ]

(* inverseRotationMatrixFromSlope *)
inverseRotationMatrixFromSlope[ slope_ ] :=
Block [	{a, b},
	a = Numerator[slope];
	b = Denominator[slope];
	sin = a / Sqrt[a^2 + b^2];
	cos = b / Sqrt[a^2 + b^2];
	{{ cos, -sin }, { sin, cos }} ]

(* orderPolygon *)
orderPolygon[ Polygon[pointlist_] ] := Polygon[ Sort[pointlist] ]

(* parallelogramOrder *)
lookup[ { 1,  1} ] = 1		    (* lookup traverses vertices clockwise *)
lookup[ { 1,  0} ] = 1		    
lookup[ { 1, -1} ] = 2
lookup[ { 0, -1} ] = 2
lookup[ {-1, -1} ] = 3
lookup[ {-1,  0} ] = 3
lookup[ {-1,  1} ] = 4
lookup[ { 0,  1} ] = 4
parallelogramOrder[ {a_, b_} ] := lookup[ Sign[ b - a ] ]

(* parallelogramSort *)
parallelogramSort[ points_ ] :=
Block [	{ordering, pos},
	ordering = Map[ parallelogramOrder,
			Transpose[ {points, RotateLeft[points]} ] ];
	pos = First[ Position[ordering, 1] ];
	RotateLeft[ points, pos - 1 ] ]

(* polygonUnion *)
polygonUnion[ polylist_ ] :=
Block [	{i, keep, len, removed, sorted, union},
	keep = removed = {};
	len = Length[polylist];
	sorted = Map[ orderPolygon, polylist ];
	For [ i = 1, i <= len, i++,
	      If [ ! MemberQ[removed, i],
		   AppendTo[keep, i];
		   thepoly = sorted[[i]];
		   pos = Apply[ Join, Position[ sorted, thepoly ] ];
		   removed = Join[removed, pos] ] ];
	polylist[[keep]] ]

(* projectPointOntoLine *)
projectPointOntoLine[ p2_, Line[ {p0_, p1_} ] ] :=
Block [	{u, v, z},
	v = p1 - p0;
	u = p2 - p0;
	z = ((u . v) / (v . v)) v;
	p0 + z ]

(* rectModWithFundamentalTile *)
rectModWithFundamentalTile[ rectPoints_, {ll_, ur_} ] :=
Block [	{cycle, i, last, next, point, rectangles, rectAsTris},
	rectAsTris = PolygonToTriangles[ Polygon[rectPoints] ];
	ToList[ TriangleModWithSquare[rectAsTris, {ll, ur}] ] ]

(* rotationMatrixFromSlope *)
rotationMatrixFromSlope[ slope_ ] :=
Block [	{a, b},
	a = Numerator[slope];
	b = Denominator[slope];
	sin = a / Sqrt[a^2 + b^2];
	cos = b / Sqrt[a^2 + b^2];
	{{ cos, sin }, { -sin, cos }} ]

(* slope *)
slope[ Line[{{x0_, y0_}, {x1_, y1_}}] ] :=
	If [ SameQ[x0, x1], Infinity, ( y0 - y1 ) / ( x0 - x1 ) ]

(* tilingDialogue *)
tilingDialogue[ intrabandFlag_, interbandFlag_, factor_ ] :=
Block [ {aliasedFlag, endstr, interstr, intrastr},
	aliasedFlag = intrabandFlag || interbandFlag;
	endstr = If [ aliasedFlag,
		      intrastr = If [ intrabandFlag, "intra-band ", "" ];
		      interstr = If [ interbandFlag, "inter-band ", "" ];
		      andstr =
			If [ intrabandFlag && interbandFlag, "and ", "" ];
		      StringJoin[ " with ", intrastr, andstr,
				  interstr, "aliasing shown in black" ],
		      "" ];
	Print[ "Tiling of the input frequency domain",
	       " for the ", factor, " bands", endstr, ":" ] ]

(* xIntercept *)
xIntercept[ 0, {x0_, y0_} ] := Infinity
xIntercept[ Infinity, {x0_, y0_} ] := x0
xIntercept[ m_, {x0_, y0_} ] := ( y0 - m x0 ) / m

(* yIntercept *)
yIntercept[ 0, {x0_, y0_} ] := y0
yIntercept[ Infinity, {x0_, y0_} ] := Infinity
yIntercept[ m_, {x0_, y0_} ] := y0 - m x0


(*  G L O B A L     F U N C T I O N S  *)

(* DesignDecimationSystem *)
Options[ DesignDecimationSystem ] := { Dialogue -> False, Mod -> 100 }

DesignDecimationSystem[ arg_, {n1_Symbol, n2_Symbol}, op___ ] :=
Block [	{h = Unique["filter"], l, m, shift, x = Unique["x"]},
	If [ InformUserQ[ ToList[op] ~Join~ Options[DesignDecimationSystem] ],
	     Print[
	       StringJoin[
		 "From the pair of discrete-time index variables ",
		 ToString[n1], " and ", ToString[n2],
		 " you have provided, this routine will return the ",
		 "rational decimator as a cascade of a modulator, ",
		 "upsampler, filter, and downsampler in cascade.  "
		 "This routine leaves the impulse response of the filter ",
		 ToString[ h[n1,n2] ], " and the input signal ",
		 ToString[ x[n1,n2] ], " unspecified." ] ] ];
	{shift, l, m} = designDecimationSystem[ arg, op ];
	Downsample[ m, {n1, n2}] [
	  Convolve[{n1,n2}][
	    h[n1, n2],
	    Upsample[ l, {n1, n2}] [
	      Exp[ I shift . {n1, n2} ] x[n1, n2] ] ] ] ]

DesignDecimationSystem[ Polygon[pointlist_], op___ ] :=
	designDecimationSystem[ Polygon[pointlist], op ]

DesignDecimationSystem[ ratmat_List, op___ ] :=
	designDecimationSystem[ ratmat, op ] /;
	MatrixQ[ ratmat ] && Apply[ SameQ, Dimensions[ratmat] ]

designDecimationSystem[ Polygon[pointList_], op___ ] :=
Block [	{center, dialogue, idealRectPoints, l, linv, m, minRect, mod, newRect,
	 numpoints, oplist, pointlist, ratmat, rejected, thepoint},

	(* make sure that the polygon does not contain redundant points *)
	numpoints = Length[pointList];
	pointlist = rejected = {};
	For [ i = 1, i <= numpoints, i++,
	      If [ ! MemberQ[rejected, i],
		   thepoint = pointList[[i]];
		   AppendTo[pointlist, thepoint];
		   positions = Apply[Join, Position[pointList, thepoint]];
		   If [ Length[positions] > 1,
			Message[ DesignDecimationSystem::badpoint,
				 thepoint, Polygon[pointList] ] ];
		   rejected = Join[rejected, positions] ] ];

	(* process options Dialogue & Mod; all other opts are for graphics *)
	oplist = ToList[op] ~Join~ ToList[Options[DesignDecimationSystem]];
	dialogue = Replace[Dialogue, oplist];
	mod = Replace[Mod, oplist];
	graphoptions = ToCollection[ RemoveOptions[oplist, {Dialogue, Mod}] ];
	If [ SameQ[mod, Mod], mod = 100 ];
	If [ IntegerQ[mod], mod = { mod, mod } ];
	If [ ! MatchQ[mod, {_Integer, _Integer}],
	     Message[ DesignDecimationSystem::mod, mod, 100 ];
	     mod = {100, 100} ];

	(* design the decimator system *)
	minRect = FindMinimumRectangle[ Polygon[pointlist] ];
	idealRectPoints = idealParallelogramList[ First[minRect] / Pi, mod ];
	center = findCenter[idealRectPoints];
	newRect = Polygon[ Map[ (# - center)&, idealRectPoints ] ];
	ratmat = findResamplingMatrix[newRect];
	If [ InformUserQ[dialogue],
	     designDialogue[ dialogue, Pi center, pointlist, ratmat,
			     minRect, graphoptions ] ];
	{ linv, m } = FactorRationalResamplingMatrix[ ratmat ];
	l = Inverse[linv];
	If [ InformUserQ[oplist],
	     Print[ "The input-output compression ratio is ",
		    Abs[Det[m]], "-to-", Abs[Det[l]], "." ] ];
	{ Pi center, l, m } ]

designDecimationSystem[ ratmat_List, op___ ] :=
Block [	{ l, linv, m, oplist },
	oplist = ToList[op] ~Join~ ToList[Options[DesignDecimationSystem]];
	{ linv, m } = FactorRationalResamplingMatrix[ ratmat ];
	l = Inverse[linv];
	If [ InformUserQ[oplist],
	     Print[ "The compression ratio is ",
		    Abs[Det[m]] "-to-", Abs[Det[l]], "." ] ];
	{ {0,0}, l, m } ] /;
	MatrixQ[ ratmat ] && Apply[ SameQ, Dimensions[ratmat] ]

(*  DownsamplingAliasing  *)
Options[ DownsamplingAliasing ] :=
	{ AspectRatio -> 1, Axes -> True, Dialogue -> All,
	  Frame -> True, FrameTicks -> {piTickMarks, piTickMarks},
	  PlotRange -> {{-Pi, Pi}, {-Pi, Pi}} }

DownsamplingAliasing[ M_, Polygon[pointlist_], options___ ] :=
Block [ {adjustArea, aliasing, aliasedList, colors, cosets, factor, graphics,
	 graphoptions, i, interband, interbandFlag, intrabandFlag, ki,
	 maximalfactor, minvtrans, newpointlist, oplist, polyarea, redundant,
	 regions, regionPlots, square, tiling, tilingList, triangles},

	oplist = ToList[options] ~Join~ Options[DownsamplingAliasing];
	dialogue = Replace[Dialogue, oplist];
	graphoptions = RemoveOptions[oplist, Dialogue];
	aliasedList = regions = regionPlots = {};

	If [ InformUserQ[dialogue],
	     Print[ "Analyzing the aliasing for the downsampling matrix  ",
		    MatrixForm[M],
		    " for a baseband whose domain is described by the",
		    " polygon with vertices ", pointlist ] ];

	cosets = DistinctCosetVectors[M];
	factor = Length[cosets];
	minvtrans = Inverse[Transpose[M]];
	square = {{-Pi, -Pi}, {Pi, Pi}};
	colors = DistinctRGBColors[factor];
	black = RGBColor[0, 0, 0];

	polyarea = Simplify[ PolygonArea[ Polygon[pointlist] ] ];
	maximalfactor = Simplify[ 4 Pi^2 / polyarea ];
	If [ SameQ[dialogue, All],
	     Print[ "The downsampling will yield the baseband plus ",
		    factor - 1, " shifted/skewed copies of the baseband." ];
	     Print[ "The shifting vectors are ",
		    Map [ (2 Pi (minvtrans . #))&, cosets ] ];
	     Print[ "The area of the baseband is" ];
	     Print[ polyarea ];
	     Print[ "which has a numerical value of ", N[polyarea], "." ];
	     evaluateTiling[maximalfactor, factor] ];

	(* Check intra-band aliasing *)
	intrabandFlag = interbandFlag = False;
	For [ i = 1, i <= factor, i++,
	      ki = cosets[[i]];
	      newpointlist = Map [ (# + 2 Pi (minvtrans . ki))&, pointlist ];
	      triangles = ToList[ PolygonToTriangles[ Polygon[newpointlist] ] ];
	      tilingList = Map[ ToList[TriangleModWithSquare[#, square]]&,
				triangles ];
	      tiling = Apply[Join, tilingList];
	      If [ ! VectorQ[tiling],
		   Print[ "Tiling list for band #", i, " is not flat" ] ];
	      aliasing = aliasedRegions[tiling];
	      graphics = { Graphics[ Prepend[ tiling, colors[[i]] ] ],
			   Graphics[ Prepend[ aliasing, black ] ] };
	      If [ ! EmptyQ[aliasing],
		   intrabandFlag = True; AppendTo[aliasedList, aliasing] ];
	      If [ InformUserQ[dialogue],
		   intrabandDialogue[aliasing, graphics, graphoptions, i] ];
	      regionPlots = Join[regionPlots, graphics];
	      AppendTo[regions, tiling] ];

	(* Check inter-band aliasing & adjust for redundant aliased regions *)
	If [ InformUserQ[dialogue],
	     interband = OverlappingRegions[regions];
	     interbandFlag = ! EmptyQ[interband];
	     aliasedList = polygonUnion[ Union[Join[aliasedList, interband]] ];
	     If [ interbandFlag || intrabandFlag,
	          aliasedArea = Apply[ Plus, Map[PolygonArea, aliasedList] ];
		  redundant = OverlappingRegions[aliasedList];
	          adjustArea = Apply[ Plus, Map[PolygonArea, redundant] ];
		  aliasedArea -= adjustArea ] ];

	(* Show tiling of the frequency domain with aliasing in black *)
	If [ InformUserQ[dialogue],
	     tilingDialogue[intrabandFlag, interbandFlag, factor] ];
	Show[ Append[ regionPlots, Graphics[Prepend[interband, black]] ],
	      graphoptions,
	      PlotLabel -> "Tiling of Input Frequency Domain" ];
	If [ InformUserQ[dialogue],
	     evaluateTiling[ maximalfactor, factor, intrabandFlag,
			     interbandFlag, aliasedArea ] ];

	regions ]

(* FindBoundingRegion  *)
FindBoundingRegion[ Line[{{x0_, y0_}, {x1_, y1_}}], Polygon[poly_] ] :=
	findBoundingRectangle[ Polygon[poly] ] /;
	SameQ[ x0, x1 ] || SameQ[ y0, y1 ]

FindBoundingRegion[ Line[points_], Polygon[poly_] ] :=
Block [	{invmatrix, m, matrix, newPoly, newRect, rect},
	m = slope[ Line[points] ];
	matrix = rotationMatrixFromSlope[m];
	newPoly = Transpose[ Chop [ N[ matrix . Transpose[poly] ] ] ];
	rect = findBoundingRectangle[ Polygon[ newPoly ] ];
	invmatrix = inverseRotationMatrixFromSlope[m];
	newRect = Transpose[ Chop[ N[ invmatrix . Transpose[ rect[[1]] ] ] ] ];
	Polygon[ newRect ] ]

(*  FindMinimumRectangle  *)
FindMinimumRectangle[ Polygon[pointlist_] ] :=
Block [	{center, curpoint, edge, i, lastpoint, majorAxis, minArea,
	 minRect, newArea, newRect, numpoints},
	numpoints = Length[pointlist];
	center = findCenter[pointlist, numpoints];
	lastpoint = pointlist[[numpoints]];
	minArea = Infinity;
	For [ i = 1, i <= numpoints, i++,
	      curpoint = pointlist[[i]];
	      majorAxis = edge = Line[ {lastpoint, curpoint} ];
	      newRect = FindBoundingRegion[majorAxis, Polygon[pointlist]];
	      newArea = PolygonArea[newRect];
	      If [ N[ newArea < minArea ],
		   minRect = newRect; minArea = newArea ];
	      lastpoint = curpoint ];
	minRect ]

(* FactorRationalResamplingMatrix *)
FactorRationalResamplingMatrix[ ratmat_List ] :=
Block [	{d, diagVector, i, leftd, mindim, rightd, u, v},
	{u, d, v} = SmithMcMillanForm[ ratmat ];
	mindim = Min[ Dimensions[d] ];
	diagVector = Table[ d[[i,i]], {i, 1, mindim} ];
	rightd = DiagonalMatrix[ Map[ Numerator, diagVector ] ];
	leftd = DiagonalMatrix[ 1 / Map[ Denominator, diagVector ] ];
	{ u . leftd, rightd . v } ] /;
MatrixQ[ ratmat ] && Apply[ SameQ, Dimensions[ratmat] ]

(* ParallelogramQ *)
ParallelogramQ[ Polygon[ { pt1_, pt2_, pt3_, pt4_ } ] ] :=
	SameQ[ slope[ Line[ {pt1, pt2} ] ], slope[ Line[ {pt4, pt3} ] ] ] &&
	SameQ[ slope[ Line[ {pt2, pt3} ] ], slope[ Line[ {pt1, pt4} ] ] ]

ParallelogramQ[ h_ ] := False

(* ValidDecimatorBaseband *)
ValidDecimatorBaseband[ Polygon[pointlist_] ] :=
	{ False, Polygon[pointlist], DiagonalMatrix[{1,1}] } /;
	(Length[pointlist] == 4) || (Length[pointlist] == 6)

ValidDecimatorBaseband[ Polygon[pointlist_] ] :=
Block [	{area, centroid, matrix, numpoints, passband},
	numpoints = Length[pointlist];
	centroid  = Apply[Plus, pointlist] / numpoints;
	passband  = Polygon[ Map[ (# - centroid)&, pointlist ] ];
	area = Simplify[ 4 Pi^2 / PolygonArea[passband] ];
	If [ IntegerQ[area], valid = True ];
	{ valid, passband, matrix } ]


(*  E N D     P A C K A G E  *) 

End[]
EndPackage[]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell1 ];
     On[ General::spell ] ]


(*  H E L P     I N F O R M A T I O N  *)

Block [	{newfuns},
	 newfuns =
	      { DesignDecimationSystem,		DownsamplingAliasing,
		FactorRationalResamplingMatrix, FindBoundingRegion,
		FindMinimumRectangle,		ValidDecimatorBaseband };
	 Combine[ SPfunctions, newfuns ];
	 Apply[ Protect, newfuns ] ]


(*  E N D I N G     M E S S A G E  *)

Print["Interactive design of decimation systems is now supported."]
Null
