(*  :Title:	Intersection of Polygons	*) 

(*  :Authors:	Brian Evans and James McClellan  *)

(*
    :Summary:	To provide the ability to take intersections of regions
		described by a union of triangles.
 *)

(*  :Context:	SignalProcessing`Support`Polygons`  *)

(*  :PackageVersion:  SPP 2.9  @(#)Polygons.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*  :Source:	{Algorithms}. Sedgewick. Second Edition. Chapter 24.  *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*  :Discussion:  *)

(*
    :Functions:	ConvexPolygonIntersection
		DistinctRGBColors
		DistinctCMYKColors
		InsidePolygonQ
		OverlappingRegions
		PolygonArea
		PolygonToTriangles
		TriangleModWithSquare
 *)


(*  B E G I N     P A C K A G E  *)

BeginPackage [	"SignalProcessing`Support`Polygons`",
		"SignalProcessing`Support`Lisp`",
		"SignalProcessing`Support`SupCode`" ]


If [ TrueQ[ $VersionNumber >= 2.0 ],
     Off[ General::spell ];
     Off[ General::spell1 ] ]

			
(*  U S A G E     I N F O R M A T I O N  *)

ConvexPolygonIntersection::usage =
	"ConvexPolygonIntersection[ p1, p2, ... ] returns the intersection \
	of polygons p1, p2, ... \
	The algorithm exhaustive enumerates all line segments connecting \
	the vertices in the two polygons. \
	This function returns an empty list if there are no intersections."

DistinctRGBColors::usage =
	"DistinctRGBColors[n] will generate a list of n distinct \
	reg, green, and blue colors as distinct form one another as possible \
	but also with as much contrast to white and black as possible."

DistinctCMYKColors::usage =
	"DistinctCMYKColors[n] will generate a list of n distinct \
	cyan, magenta, yellow, and black colors."

InsidePolygonQ::usage =
	"InsidePolygonQ[ Polygon[vertices], Point[coordinates] ] returns \
	true if the point is inside the polygon, and False otherwise. \
	 A third argument is a flag, which defaults to False, indicating \
	what to return if the point lies on an edge of the polygon."

OverlappingRegions::usage =
	"OverlappingRegions[r] enumerates each pair of non-identical \
	regions in r and returns the union of all overlapping regions \
	as a list of polygons (an empty list means no overlap). \
	OverlappingRegions[region1, region2] returns a list of polygons \
	that represents the overlap between region1 and region2. \
	Multiple overlapping regions may be returned. \
	Remove redundant regions with the Union command."

PolygonArea::usage =
	"PolygonArea[ Polygon[ {p0, p1, ...} ] ] will compute the area \
	of the polygon with vertices p0, p1, ..."

PolygonToTriangles::usage =
	"PolygonToTriangles[polygon] will slice the polygon into \
	triangles by connecting each vertex with the centroid."

RearrangePolygon::usage =
	"RearrangePolygon[ polygon ] rearranges the listing of vertices \
	in order to place the vertex that has the smallest x-coordinate \
	among those that have the smallest y-coordinates as the first vertex."

TriangleModWithSquare::usage =
	"TriangleModWithSquare[ Polygon[{v1, v2, v3}], {ll, ur} ] will \
	compute the triangle with vertices v1, v2, and v3 modulo the square \
	with lower-left corner coordinates ll and upper-right corner \
	coordinates ur."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin[ "`Private`" ]


TriangleModWithSquare::shift =
	"Cannot determine the shift necessary to put one of the points \
	in the triangle `` into the square with lower left-hand corner \
	`` and upper right-hand corner ``."

TriangleModWithSquare::notri =
	"The following polygon is not a triangle: ``"


(*  L O C A L     F U N C T I O N S  *)

(*  counterClockwise--- returns 1 if points are counter-clockwise, *)
(*  0 if points are co-linear, and -1 if points are clockwise.	   *)
counterClockwise[ {x0_, y0_}, {x1_, y1_}, {x2_, y2_} ] :=
Block [	{ccw, dx1, dx2, dy1, dy2},
	dx1 = N[x1 - x0];
	dy1 = N[y1 - y0];
	dx2 = N[x2 - x0];
	dy2 = N[y2 - y0];
	Which [ dx1 dy2 > dy1 dx2,
		  ccw = 1,
		dx1 dy2 < dy1 dx2,
		  ccw = -1,
		True,
		  If [ (dx1 dx2 < 0) || (dy1 dy2 < 0),
		       ccw = -1,
		       If [ (dx1 dx1 + dy1 dy1) >= (dx2 dx2 + dy2 dy2),
			    ccw = 0,
			    ccw = 1 ] ] ];
	ccw ]

(*  inPolygonQ--- couldn't get Sedgewick's algorithm to work for all cases  *)
lastPolygonList = {}
lastRearrangedPolygonList = {}

inPolygonQ[polygonlist_, pointlist_, maxcoords_, mincoords_] :=
Block [	{count, edge, j, numVertices, pointline, polylist, outsidePoint,
	 testline, vertex, vertexCoordinates},

	polylist = If [ SameQ[polygonlist, lastPolygonList],
			lastRearrangedPolygonList,
			lastRearrangedPolygonList =
			    rearrangePolygonList[polygonlist] ];
	outsidePoint = { 2 maxcoords[[1]] - mincoords[[1]], pointlist[[2]] };
	testline = { pointlist, outsidePoint };
	numVertices = Length[ polylist ];
	count = 0;
	j = numVertices;
	For [ vertex = 1, vertex <= numVertices, vertex++,
	      vertexCoordinates = polylist[[vertex]];
	      pointline = {vertexCoordinates, vertexCoordinates};
	      If [ ! intersectQ[pointline, testline],
		   edge = { vertexCoordinates, polylist[[j]] };
		   j = vertex;
		   If [ intersectQ[edge, testline], count++ ] ] ];

	OddQ[count] ]

(*  insert and insertionSort--- insertionSort is front-end for insert  *)
insertionSort[ list_, v_?VectorQ ] :=
	insert[ list, v ]
insertionSort[ list_, {v_?VectorQ} ] :=
	insert[ list, v ]
insertionSort[ list_, {v1_?VectorQ, v2_?VectorQ} ] :=
	insert[ insert[ list, v1 ], v2 ]

insert[ list_, entry_ ] := Append[list, entry] /; Length[list] <= 1

insert[ list_, entry_ ] :=
Block [	{i, ilast, len, pos},
	len = Length[list];
	ilast = len;
	For [ i = 1, i <= len, i++,
	      If [ counterClockwise[ list[[ilast]], entry, list[[i]] ] >= 0,
		   pos = i; Break[] ];
	      ilast = i ];
	If [ SameQ[pos, 1],
	     Prepend[list, entry],
	     Join[ Take[ list, {1, pos - 1} ],
		   {entry},
		   Take[ list, {pos, len} ] ] ] ]

(*  intersectPointAndLineQ  *)
intersectPointAndLineQ[ {x_, y_}, {{x0_, y0_}, {x1_, y1_}} ] :=
  TrueQ[ N[ ( x0 <= x <= x1 ) || ( x0 >= x >= x1 ) ] &&
	 N[ ( y0 <= y <= y1 ) || ( y0 >= y >= y1 ) ] &&
	 N[ ( x0 == x == x1 ) || ( y0 == y == y1 ) ||
	    ( y == ((y1 - y0) / ( x1 - x0 )) * ( x - x0 ) + y0 ) ] ]

(*  intersectQ  *)
intersectQ[ {point_List, point_List}, {point1_List, point2_List} ] :=
	intersectPointAndLineQ[ point, {point1, point2} ]

intersectQ[ {point1_List, point2_List}, {point_List, point_List} ] :=
	intersectPointAndLineQ[ point, {point1, point2} ]

intersectQ[ {l1p1_List, l1p2_List}, {l2p1_List, l2p2_List} ] :=
	( counterClockwise[l1p1, l1p2, l2p1] *
	  counterClockwise[l1p1, l1p2, l2p2] <= 0 ) &&
	( counterClockwise[l2p1, l2p2, l1p1] *
	  counterClockwise[l2p1, l2p2, l1p2] <= 0 )

(*  join  *)
join[ master_, {x_List, y_List} ] := Join[ master, {x, y} ]
join[ master_, point_ ] := Append[ master, point ]

(*  lineIntersection--- assumes lines do intersect; see intersectQ  *)
(*  it returns either a point or a pair of points as a sequence	    *)
lineIntersection[ {point_List, point_List}, {{bx0_, by0_}, {bx1_, by1_}} ] :=
	point
lineIntersection[ {{bx0_, by0_}, {bx1_, by1_}}, {point_List, point_List} ] :=
	point
lineIntersection[ {{ax0_,ay0_}, {ax1_,ay1_}}, {{bx0_, by0_}, {bx1_, by1_}} ] :=
Block [	{allpoints, ba, bb, intersection, ma, mb, points, xilist, yilist},
	ma = slope[ {ax0, ay0}, {ax1, ay1} ];
	mb = slope[ {bx0, by0}, {bx1, by1} ];
	Which [ SameQ[ma, mb],		(* intersection may be line segment *)
		  allpoints = {{ax0, ay0}, {ax1, ay1}, {bx0, by0}, {bx1, by1}};
		  points = Take[ Sort[allpoints, sortByXorY], {2, 3} ];
		  intersection =
		      If [ SameQ[ points[[1]], points[[2]] ],
			   points[[1]],
			   toCollection[ points ] ],
		SameQ[ma, Infinity],
		  intersection = { ax0, mb ( ax0 - bx0 ) + by0 },
		SameQ[mb, Infinity],
		  intersection = { bx0, ma ( bx0 - ax0 ) + ay0 },
		True,
		  ba = ay0 - ma ax0;
		  bb = by0 - mb bx0;
		  intersection = { ( ba - bb ) / ( mb - ma ),
				   ( ba mb - bb ma ) / ( mb - ma ) } ];
	intersection ]

(*  makePolyLines  *)
makePolyLines[ poly_, 1 ] := { Join[poly, poly] }
makePolyLines[ poly_, len_ ] := Transpose[ { poly, RotateLeft[poly] } ]

(*  mymod  *)
SetAttributes[mymod, {Listable}]
mymod[x_, y_] := x - y Floor[x/y] 

(*  newPointOrLine  *)
newPointOrLine[ master_, point_ ] :=
	If [ ! MemberQ[master, point], point, {} ]

newPointOrLine[ master_, point1_, point2_ ] :=
	Block [	{return = {}},
		If [ ! MemberQ[master, point1], AppendTo[return, point1] ];
		If [ ! MemberQ[master, point2], AppendTo[return, point2] ];
		return ]

(*  norm  *)
norm[v_] := Sqrt[v . v]

(*  normsquared  *)
normsquared[v_] := v . v

(*  onPolygonEdgeQ  *)
onPolygonEdgeQ[polylist_List, point_List] := True /;
	MemberQ[polylist, point]

onPolygonEdgeQ[polylist_List, point_List] :=
Block [	{curVertex, flag = False, lastVertex, numVertices, vertex},
	numVertices = Length[polylist];
	lastVertex = polylist[[numVertices]];
	For [ vertex = 1, vertex <= numVertices, vertex++,
	      curVertex = polylist[[vertex]];
	      If [ intersectPointAndLineQ[point, {lastVertex, curVertex}],
		   flag = True; Break[] ];
	      lastVertex = curVertex ];
	flag ]

onPolygonEdgeQ[ x___ ] :=
	MyMessage[ General::argct, False, onPolygonEdgeQ, Length[{x}] ]

(*  slope  *)
slope[ {x0_, y0_}, {x1_, y1_} ] :=
	If [ SameQ[x0, x1], Infinity, ( y0 - y1 ) / ( x0 - x1 ) ]

(*  sortByXorY  *)
sortByXorY[ {x1_, y1_}, {x2_, y2_} ] :=
	If [ SameQ[x1, x2], N[ y1 < y2 ], N[ x1 < x2 ] ]


(*  toCollection  *)
toCollection[x_?AtomQ] := x
toCollection[h_[args___]] := args
toCollection[a__] := a

(*  toList  *)
toList[] := {}
toList[arg_List] := arg
toList[arg_] := List[arg] /; ! SameQ[Head[arg], List]
toList[arg1_, args__] := List[arg1, args]

(*  triangleArea  *)
triangleArea[ Polygon[{p_, p_, q_}] ] := 0

triangleArea[ Polygon[{p0_, p1_, p2_}] ] :=
Block [	{u, v, z},
	v = p1 - p0;
	u = p2 - p0;
	z = ((u . v) / normsquared[v]) v;
	1/2 norm[u - z] norm[v] ]


(*  M O D U L O     O P E R A T I O N S  *)

(* addVertices *)
addVertices[ intersection_, poly1_, len1_, poly2_, len2_ ] :=
Block [	{entry, i, newintersection}, 
	newintersection = intersection;
	For [ i = 1, i <= len1, i++,
	      entry = poly1[[i]];
	      If [ InsidePolygonQ[ Polygon[poly2], Point[entry], True ] &&
		     ! MemberQ[newintersection, entry],
		   newintersection = insertionSort[newintersection, entry] ] ];
	newintersection ]

(*  breakUpPolygon--- into other tri's by drawing edges to an interior point  *)
breakUpPolygon[ Polygon[pointlist_], point_ ] :=
Block [ {numpoints, repeatedPoints},
	numpoints = Length[pointlist];
	repeatedPoints = Table[point, {numpoints}];
	Map[ Polygon,
	     Transpose[{pointlist, RotateLeft[pointlist], repeatedPoints}] ] ]

(*  categorizePoints--- as inside, on, or outside the square's boundary  *)
categorizePoints[ pointlist_, {lowerleft_, upperright_}, corners_ ] :=
Block [ {i, index, insideFlag, lists, numpoints, point},
	lists = { {}, {}, {} };
	numpoints = Length[pointlist];
	For [ i = 1, i <= numpoints, i++,
	      point = pointlist[[i]];
	      insideFlag =
		  Apply[ And, N[Thread[lowerleft < point < upperright]] ];
	      index = Which [ insideFlag,			 1,
			      onPolygonEdgeQ[corners, point],	 2,
			      True,				 3 ];
	      lists[[index]] = Append[ lists[[index]], point ] ];
	lists ]

(*  combinePolygons  *)
combinePolygons[ Polygon[{}], poly_ ] := poly
combinePolygons[ poly1_, poly2_ ] := Join[ toList[poly1], toList[poly2] ]

(*  combineRecursive  *)
combineRecursive[ overlapPoly_, triangle_, triangle_, _, _ ] := triangle
combineRecursive[ overlapPoly_, triangle_, {triangle_}, _, _ ] := triangle

combineRecursive[ overlapPoly_, triangle_, newtriangles_,
		  lowerleft_, upperright_ ] :=
	combinePolygons[ overlapPoly,
			 TriangleModWithSquare[ newtriangles,
						{lowerleft, upperright} ] ]

(* containedQ *)
containedQ[ {{x1_, y1_}, {x2_, y2_}, {x3_, y3_}},
	    boundary_, outside_, {xll_, yll_}, {xur_, yur_} ] :=
Block [	{},
	N[ x1 <= xll && x2 <= xll && x3 <= xll ] ||
	N[ x1 >= xur && x2 >= xur && x3 >= xur ] || 
	N[ y1 <= yll && y2 <= yll && y3 <= yll ] ||
	N[ y1 >= yur && y2 >= yur && y3 >= yur ] ] /;
	Length[boundary] + Length[outside] == 3

containedQ[ triPoints_, boundary_, outside_, ll_, ur_ ] := False

(* findClosest *)
findClosest[intersection_, pt_, prevpt_] :=
	Which [ Length[intersection] == 1,
		  First[intersection],
		N[ norm[ intersection[[1]] - pt ] <
		   norm[ intersection[[2]] - pt ] ] &&
		! SameQ[ intersection[[1]], prevpt ],
		  intersection[[1]],
		True,
		  intersection[[2]] ]

(*  intersectLineAndSquare--- always return a list of one or two points  *)
intersectLineAndSquare[ {pt1_, pt2_}, corners_ ] :=
Block [	{edge, intersection, j, lastCorner, theCorner},
	intersection = {};
	lastCorner = corners[[4]];
	For [ j = 1, j <= 4, j++,
	      theCorner = corners[[j]];
	      edge = {lastCorner, theCorner};
	      If [ intersectQ[ edge, {pt1, pt2} ],
		   intersection =
		       Join[ intersection,
			     { lineIntersection[ edge, {pt1, pt2} ] } ] ];
	      lastCorner = theCorner ];
	intersection ]

(*  keepLine---  check if two lines intersect	 *)
keepLine[ line2_, line1_ ] :=
	If [ intersectQ[line1, line2], lineIntersection[line1, line2], {} ]

(*  myListMod--- take a set of coordinates pt modulo with an offset ll  *)
myListMod[ pt_, { ll_, len_ } ] := mymod[ (pt - ll), len] + ll

(*  outsideCheck  *)
outsideCheck[pt_, pt1_, pt2_, corners_, poly_] :=
Block [	{addpoints, intersection, intersection1, newpointlist, newpoly},
	newpoly = poly;
	intersection = intersectLineAndSquare[{pt, pt1}, corners];
	addpoints =
	    If [ SameQ[ intersection, {} ],
		 intersection1 = intersectLineAndSquare[{pt1, pt2}, corners];
		 newpointlist = If [ Length[intersection1] == 2,
				     { findClosest[intersection1, pt1, {}] },
				     intersection1 ];
		 Append[ newpointlist, pt1 ],
		 { findClosest[intersection, pt, {}] } ];
	If [ ! MemberQ[ newpoly, addpoints[[1]] ],
	     newpoly = insertionSort[ newpoly, addpoints[[1]] ] ];
	If [ Length[addpoints] == 2,
	     If [ ! MemberQ[ newpoly, addpoints[[2]] ],
		  newpoly = insertionSort[ newpoly, addpoints[[2]] ] ] ];
	newpoly ]

(*  outsideTriangles  *)
(*  1 point outside square--- always produces one triangle outside square  *)
outsideTriangles[{outpt_}, triPoints_, corners_] :=
Block [	{pt1, pt2},
	{pt1, pt2} = Complement[triPoints, {outpt}];
	Polygon[ { outpt,
		   First[ intersectLineAndSquare[{outpt, pt1}, corners] ],
		   First[ intersectLineAndSquare[{outpt, pt2}, corners] ] } ] ]

(*  2-3 points outside square--- produces 1-3 tri's outside the square  *)
outsideTriangles[outside_, triPoints_, corners_] :=
Block [	{i, newpoly, newtriangles, pt, pt1, pt2, triangles},
	triangles = {};
	For [ i = 1, i <= 3, i++,
	      pt = triPoints[[i]];
	      If [ MemberQ[outside, pt],
		   newpoly = {pt};
		   {pt1, pt2} = Drop[ triPoints, {i,i} ];
		   newpoly = outsideCheck[pt, pt1, pt2, corners, newpoly];
		   newpoly = outsideCheck[pt, pt2, pt1, corners, newpoly];
		   newtriangles =
		   	toList[ PolygonToTriangles[ Polygon[newpoly] ] ];
		   triangles = Union[ Join[ triangles, newtriangles ] ] ] ];
	triangles ] /;
Length[outside] >= 2

outsideTriangles[ x___ ] :=
	MyMessage[ General::argct, False, outsideTriangles, Length[{x}] ]

(*  polygonQ  *)
polygonQ[ Polygon[pointlist_] ] := Length[pointlist] >= 3
polygonQ[ pointlist_List ] := Length[pointlist] >= 3
polygonQ[ x_ ] := False

(* rearrangePolygonList *)
(* Move the vertex which has the smallest x-coordinate among those *)
(* that have the smallest y-coordinate to the first position	   *)
rearrangePolygonList[ polylist_ ] :=
Block [	{minpos, npolylist, smallestYpoints, xmin, ymin},
	npolylist = N[ polylist ];
	ymin = Min[ Map[ Second, npolylist ] ];
	smallestYpoints = Cases[ npolylist, {_, ymin} ];
	xmin = Min[ Map[ First, smallestYpoints ] ];
	minpos = Position[npolylist, {xmin, ymin}][[1,1]];
	RotateLeft[polylist, minpos - 1] ]

(*  shiftCheckQ--- check to see if shifted vertex is inside polygon   *)
(*   or if the shifted point is on the boundary and others are inside *)
shiftCheckQ[ pointlist_, pt_, tripoints_, {lower_, width_} ] :=
Block [	{flag = False, newpoints, newpt, poly, shift},
	newpt = myListMod[ pt, {lower, width} ];
	If [ ! SameQ[newpt, pt],
	     flag = inPolygonQ[pointlist, newpt, lower, lower + width];
	     If [ (! flag) && onPolygonEdgeQ[pointlist, newpt],
		  shift = newpt - pt;
		  newpoints = Map[ (# + shift)&, tripoints ];
		  poly = Polygon[pointlist];
		  flag = InsidePolygonQ[poly, Point[newpoints[[1]]], True] &&
			 InsidePolygonQ[poly, Point[newpoints[[2]]], True] &&
			 InsidePolygonQ[poly, Point[newpoints[[3]]], True] ] ];
	flag ]

(*  shiftFactor  *)
shiftFactor[ pt_, {lower_, width_}, numdims_ ] :=
Block [	{factor},
	factor = myListMod[pt, {lower, width}] - pt;
	Table[factor, {numdims}] ]

(*  shiftTriangle  *)
shiftTriangle[ polylist_, {pt1_, pt2_, pt3_}, {lower_, width_} ] :=
	{pt1, pt2, pt3} + shiftFactor[pt1, {lower, width}, 3] /;
	shiftCheckQ[ polylist, pt1, {pt1, pt2, pt3}, {lower, width} ]

shiftTriangle[ polylist_, {pt1_, pt2_, pt3_}, {lower_, width_} ] :=
	{pt1, pt2, pt3} + shiftFactor[pt2, {lower, width}, 3] /;
	shiftCheckQ[ polylist, pt2, {pt1, pt2, pt3}, {lower, width} ]

shiftTriangle[ polylist_, {pt1_, pt2_, pt3_}, {lower_, width_} ] :=
	{pt1, pt2, pt3} + shiftFactor[pt3, {lower, width}, 3] /;
	shiftCheckQ[ polylist, pt3, {pt1, pt2, pt3}, {lower, width} ]

shiftTriangle[ polylist_, {pt1_, pt2_, pt3_}, {lower_, width_} ] :=
	Message[ TriangleModWithSquare::shift,
		 {pt1, pt2, pt3}, lower, lower + width ]

(*  triangleModWithSquare--- recursive modulo operations on triangle  *)
triangleModWithSquare[ { pt1_List, pt2_List, pt3_List },
		       { lowerleft_List, upperright_List } ] :=
Block [	{boundary, corners, inside, insideCorners, newtriangles,
	 outside, overlap, overlapPoly, shift, tileWidth,
	 triangle, triPoints},

	triPoints = {pt1, pt2, pt3};
	triangle = Polygon[triPoints];
	corners = { lowerleft, { lowerleft[[1]], upperright[[2]] },
		    upperright, { upperright[[1]], lowerleft[[2]] } };

	{ inside, boundary, outside } =
	    categorizePoints[ triPoints, {lowerleft, upperright}, corners ];

	If [ Length[boundary] + Length[inside] == 3,
	     triangle,				   (* terminating condition  *)
	     insideCorners =
		 Select[ corners, InsidePolygonQ[triangle, Point[#], False]& ];
	     newtriangles =
	         If [ ! SameQ[insideCorners, {}],   (* square vertex in tri *) 
		      overlapPoly = Polygon[{}];
		      breakUpPolygon[ triangle, First[insideCorners] ],
		      overlap = ConvexPolygonIntersection[triPoints, corners];
		      If [ ( Length[overlap] <= 2 ) || 	      (* no overlap *)
			   containedQ[triPoints, boundary, outside,
				      lowerleft, upperright],
			   overlapPoly = Polygon[{}];
			   tileWidth = upperright - lowerleft;
			   Polygon[ shiftTriangle[ corners, triPoints,
						   {lowerleft, tileWidth} ] ],
							 (* process overlap *)
			   overlapPoly = Polygon[overlap];
			   outsideTriangles[outside, triPoints, corners] ] ];
	     combineRecursive[ overlapPoly, triangle, newtriangles,
			       lowerleft, upperright ] ] ] /;
	VectorQ[pt1] && VectorQ[pt2] && VectorQ[pt3]


(*  G L O B A L     F U N C T I O N S  *)


(*  ConvexPolygonIntersection  *)
ConvexPolygonIntersection[ Polygon[poly1_], Polygon[poly2_] ] :=
	Polygon[ ConvexPolygonIntersection[ poly1, poly2 ] ]

ConvexPolygonIntersection[ poly1_List, poly2_List ] :=
Block [	{i, intersection = {}, j, len1, len2, line1, line2,
	 linelist1, linelist2, newEntry},
	len1 = Length[poly1];
	len2 = Length[poly2];
	linelist1 = makePolyLines[ poly1, len1 ];
	linelist2 = makePolyLines[ poly2, len2 ];

	(* First add all vertices that are in the intersection *)
	intersection = addVertices[ intersection, poly1, len1, poly2, len2 ];
	intersection = addVertices[ intersection, poly2, len2, poly1, len1 ];

	(* Next enumerate all possible lines composing first polygon *)
	For [ i = 1, i <= len1, i++,
	      line1 = linelist1[[i]];

	      (* Enumerate all possible lines composing second polygon  *)
	      For [ j = 1, j <= len2, j++,
		    line2 = linelist2[[j]];
		    newEntry = newPointOrLine[ intersection,
		    			       keepLine[line2, line1] ];
		    If [ ! SameQ[newEntry, {}],
			 intersection =
			     insertionSort[intersection, newEntry] ] ] ];

	intersection ]

ConvexPolygonIntersection[ p1_, p2_, prest__ ] :=
	ConvexPolygonIntersection[ ConvexPolygonIntersection[p1, p2], prest ]

(*  DistinctRGBColors--- {0,0,0} is black and {1,1,1} is white  *)
colorList =
	{ {1, 1, 0}, {0, 1, 1}, {1, 0, 1}, {1, 0, 0}, {0, 1, 0}, {0, 0, 1} }

DistinctRGBColors[ n_Integer ] :=
Block [	{colors = {}, i, newcolors, scale, term},
	scale = Quotient[n-1, 6] + 1;
	term = 1 / scale;
	For [ i = 1, i <= n, i += 6,
	      newcolors = scale term colorList;
	      colors = Join[ colors, newcolors ];
	      scale-- ];
	Take[ Map[ Apply[RGBColor, #]&, colors ], {1, n} ] ] /;
	( n > 0 )

(*  DistincCMYKColors  *)
DistinctCMYKColors[ n_Integer ] :=
	Block [	{i, max},
		max = If [ IntegerQ[n/4], Ceiling[(n+1)/4], Ceiling[n/4] ];
		Table [ CMYKColor[ Quotient[i,4] / max,
				   Quotient[i+1,4] / max,
				   Quotient[i+2,4] / max,
				   Quotient[i+3,4] / max],
			{i, 1, n} ] ] /;
	( n > 0 )

(* InsidePolygonQ--- from Sedgewick, pp. 354-355. *)
InsidePolygonQ[ Polygon[ polylist_ ], Point[ point_ ], boundary_:False ] :=
Block [	{inflag = False, maxCoordinates, minCoordinates, transposedPoints},

	(* compute the rectangular bounding box of the vertices *)
	transposedPoints = Transpose[ polylist ];
	maxCoordinates = N[ Map[ Max, transposedPoints ] ];
	minCoordinates = N[ Map[ Min, transposedPoints ] ];

	(* Make sure that the point is in the bounding square first *)
	If [ Apply[ And, Thread[minCoordinates <= N[point] <= maxCoordinates] ],
	     inflag = If [ onPolygonEdgeQ[polylist, point],
			   boundary,
			   inPolygonQ[ polylist, point,
				       maxCoordinates, minCoordinates ] ] ];

	inflag ] /;
SameQ[ Dimensions[polylist][[2]], Length[point] ]

(*  OverlappingRegions  *)
OverlappingRegions[ regions_ ] :=
Block [ {intersection, len1, len2, numregions, overlap = {}, poly1, poly2,
	 p1, p2, region1, region2, r1, r2, valid},

	numregions = Length[regions];
	For [ r1 = 1, r1 <= numregions, r1++,
	  region1 = toList[ regions[[r1]] ];
	  len1 = Length[region1];
	  For [ r2 = r1+1, r2 <= numregions, r2++,
	    region2 = toList[ regions[[r2]] ];
	    len2 = Length[region2];
	    For [ p1 = 1, p1 <= len1, p1++,
	      poly1 = region1[[p1]];
	      For [ p2 = 1, p2 <= len2, p2++,
		poly2 = region2[[p2]];
		intersection = ConvexPolygonIntersection[poly1, poly2];
		If [ (! SameQ[ intersection, Polygon[{}] ]) &&
		       (! SameQ[intersection, {}]),
		     valid = Select[ toList[intersection], polygonQ ];
		     If [ ! SameQ[valid, {}],
			  overlap = Join[overlap, valid] ] ] ] ] ] ];

	overlap ]

OverlappingRegions[ first_, second_ ] :=
Block [ {intersection, len1, len2, numregions1, numregions2, overlap = {},
	 poly1, poly2, p1, p2, region1, region2, r1, r2, valid},

	numregions1 = Length[first];
	numregions2 = Length[second];
	For [ r1 = 1, r1 <= numregions1, r1++,
	  region1 = toList[ first[[r1]] ];
	  len1 = Length[region1];
	  For [ r2 = 1, r2 <= numregions2, r2++,
	    region2 = toList[ second[[r2]] ];
	    len2 = Length[region2];
	    For [ p1 = 1, p1 <= len1, p1++,
	      poly1 = region1[[p1]];
	      For [ p2 = 1, p2 <= len2, p2++,
		poly2 = region2[[p2]];
		intersection = ConvexPolygonIntersection[poly1, poly2];
		If [ (! SameQ[ intersection, Polygon[{}] ]) &&
		       (! SameQ[intersection, {}]),
		     valid = Select[ toList[intersection], polygonQ ];
		     If [ ! SameQ[valid, {}],
			  overlap = Join[overlap, valid] ] ] ] ] ] ];

	overlap ]

(*  PolygonArea  *)
PolygonArea[ Polygon[poly_] ] :=
	triangleArea[ Polygon[poly] ] /;
	Length[poly] == 3

PolygonArea[ Polygon[poly_] ] :=
	Apply[ Plus, Map[ triangleArea, PolygonToTriangles[Polygon[poly]] ] ] /;
	Length[poly] > 3

(*  PolygonToTriangles  *)
PolygonToTriangles[ Polygon[pointlist_] ] :=
	Polygon[ pointlist ] /;
	Length[ pointlist ] <= 3

PolygonToTriangles[ Polygon[pointlist_] ] :=
	breakUpPolygon[ Polygon[pointlist], 
			Apply[Plus, pointlist] / Length[pointlist] ]

(* RearrangePolygon *)
RearrangePolygon[ Polygon[ polylist_ ] ] :=
	Polygon[ rearrangePolygonList[polylist] ]

(*  TriangleModWithSquare  *)
TriangleModWithSquare[ Polygon[pointlist_], {lowLeft_, upRight_} ] :=
	MyMessage[ TriangleModWithSquare::notri, Polygon[pointlist],
		   Polygon[pointlist] ] /;
	(! SameQ[Head[pointlist], List]) && Length[pointlist] != 3

TriangleModWithSquare[ Polygon[{pt1_, pt2_, pt3_}], {lowLeft_, upRight_} ] :=
	triangleModWithSquare[ {pt1, pt2, pt3}, {lowLeft, upRight} ]

TriangleModWithSquare[ polylist_List, {lowLeft_, upRight_} ] :=
	Apply[ Join,
	       Map[ toList[ TriangleModWithSquare[#, {lowLeft, upRight}] ]&,
		    polylist ] ]


(*  E N D     P A C K A G E  *) 

End[]
EndPackage[]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell1 ];
     On[ General::spell ] ]


(*  H E L P     I N F O R M A T I O N  *)

Block [	{newfuns},
	 newfuns =
	   { ConvexPolygonIntersection,	  DistinctRGBColors,
	     DistinctCMYKColors,	  InsidePolygonQ,
	     OverlappingRegions,	  PolygonArea,
	     PolygonToTriangles,	  RearrangePolygon,
	     TriangleModWithSquare };
	 Combine[ SPfunctions, newfuns ];
	 Apply[ Protect, newfuns ] ]


(*  E N D I N G     M E S S A G E  *)

Print["Routines to intersect polygons have been loaded."]
Null
