(*  :Title:	Signal Processing Primitive Objects  *)

(*
    :Authors:	Brian L. Evans, James H. McClellan, and Joseph M. Winograd 
		with Roberto H. Bamberger.
 *)

(*
    :Summary:   Provide functions which are common in DSP but
		not standard in Mathematica.  Functions like
		Step and Impulse, as well as operators like
		Upsample and Shift.
 *)

(*  :Context:	SignalProcessing`Support`SigProc`  *)

(*  :PackageVersion:  SPP 2.9  @(#)SigProc.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	signal processing, operators, functions, symbolic processing *)

(*  :Keywords:	*)

(*
    :Source:	     Oppenheim, A., and Schafer, R.  {Discrete-Time
		Signal Processing}.  1989.
    
		     Myers, C.  {Signal Representation for Symbolic and
		Numeric Processing}.  MIT Ph.D. Thesis.  1986.

		     Covell, M.  {An Algorithm Design Environment}.
		MIT Ph. D. Thesis, 1989.

		     Bamberger, R. {The Directional Filter Bank:  A Multirate
		Filter Bank for the Directional Decompostion of Images},
		Georgia Tech Ph. D. Thesis, 1990.
 *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*
    :Discussion:	This package introduces the functions (signals) and
		  operators (systems) common in signal processing but missing
		  in Mathematica.
			Thanks to John Mitchell at the University of Chicago
		  for writing patches for the Summation operator and the Delta
		  function.
*)

(*  :Functions: Aliasby			operator
		AliasSinc		alias
		ASinc			alias
		CConvolve		operator
		CircularShift		operator
		Convolve		operator
		Continuous		option value
		CPulse			function
		CStep			function
		Delta			function
		DeltaPlot		function
		DFT			operator
		DTFT			operator
		Discrete		option value
		Difference		operator
		Dirichlet		function
		DiscreteGraphics	routine
		Domain			option
		Downsample		operator
		DummyVariables		function
		Extent1D		function
		FT			operator
		GetDeltaFunctions	function
		Impulse			function
		Interleave		operator
		IntervalsToFunction	routine
		InvDFT			operator
		InvFT			operator
		InvL			operator
		InvZ			operator
		L			operator
		LineImpulse		function
		OperatorInOperExpr	routine
		OperatorName		routine
		ParametersInOperExpr	routine
		PolyphaseDownsample	operator
		PolyphaseUpsample	operator
		Periodic		operator
		Pulse			function
		RationalGCD		routine
		Rev			operator
		RewriteSummations	routine
		RotateAxes		operator
		ScaleAxis		operator
		ScalingFactor		routine
		SequenceToFunction	routine
		SequencePlot		routine
		Shift			operator
		SignalCleanup		routine
		SignalPlot		function
		SignalsInOperExpr	routine
		Sinc			function
		SPIntegrationRules	data
		SPSimplify		function
		Step			function
		Summation		operator
		TheFunction		routine
		ToContinuous		routine
		ToDiscrete		routine
		Unit			function
		Upsample		operator
		UpsampledFunction	routine
		UpsampleFactor		routine
		UpsampleSequence	routine
		Z			operator

	This package provides signal primitives (functions), system
	primitives (operators), and supporting routines.   Each new
	object is placed into the SPsignals, SPsystems, or SPfunctions
	list accordingly (at end of file).
	
	Each signal primitive (function) is specified as follows:

	(1) default arguments (if any),
	(2) definition (so that it behaves as a function),
	(3) multidimensional separability (if any),
	(4) automatic simplification rules,
	(5) manual simplification rules (augment Simplify),
	(6) output form rules (including TeX form rules), and
	(7) derivative and integration rules (if any)

	Each system primitive (operator) is specified as follows:

	(1) multidimensional separability (if any),
	(2) definition (so that it can be converted into a function form),
	(3) output form rules (including TeX form rules), and
	(4) specify which parameter is (are) the variable(s)
	(5) any special extent that the operator has

	All simplification rules for operators should be added to
	SPSimplificationRules (see the end of the file).
 *)




(*  B E G I N     P A C K A G E  *)


If [ TrueQ[ $VersionNumber >= 2.1 ],

     BeginPackage[ "SignalProcessing`Support`SigProc`",
		   "Calculus`DiracDelta`",
		   "SignalProcessing`Support`SupCode`",
		   "SignalProcessing`Support`Lisp`" ];
     Unprotect[Derivative],

     BeginPackage[ "SignalProcessing`Support`SigProc`",
		   "SignalProcessing`Support`SupCode`",
		   "SignalProcessing`Support`Lisp`" ] ]

If [ TrueQ[ $VersionNumber >= 2.1 ], Unprotect[DiracDelta, UnitStep] ]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     Off[ General::spell ];
     Off[ General::spell1 ] ]


(*  U S A G E     I N F O R M A T I O N  *)

$DeltaFunctionScaling::usage =
	"The variable $DeltaFunctionScaling dictates how to scale the height \
	of Delta functions for the purposes of plotting:  None or Scaled."

Aliasby::usage =
	"Aliasby[m,w][x] represents the aliasing of x, a continuous-frequency \
	signal, by replicating it every (2 Pi / m) in w and dividing the \
	result by m. \
	It is always rewritten in terms of the Periodic operator."

CConvolve::usage =
	"CConvolve[t][x1, x2, ...] represents the continuous convolution in t \
	of functions x1, x2, .... \
	CConvolve[{t1, t2, ...}][x1, x2, ...] represents the multidimensional \
	continuous convolution in t1, t2, ... of functions x1, x2, ..."

CircularShift::usage =
	"CircularShift[n0, N, n] represents the circular shifting of a \
	sequence of length N in index variable n by n0. \
	That is, the sequence is treated as a ring buffer of length N \
	so each sample at index n is shifted to the right by
	(n + n0) mod N places. \
	See also the SPP function Shift and the Mathematica function \
	RotateRight."

Convolve::usage =
	"Convolve[n][x1, x2, ...] represents the discrete convolution in n of \
	functions x1, x2, .... \
	Convolve[{n1, n2, ...}][x1, x2, ...] represents the multidimensional \
	discrete convolution in n1, n2, ... of functions x1, x2, ..."

Continuous::usage =
	"Continuous is a possible value for a signal's domain."

CPulse::usage = 
	"CPulse[l,t] defines a pulse which begins at t=0 and ends at t = l. \
	The CPulse has value 1 within the range (0,l), \
	0 outside this range, and 1/2 at the points t=0 and t=l. \
	A continuous-pulse center at the origin is written as \
	CPulse[l, t + l/2] or Shift[-l/2, t][CPulse[l, t]]."

CStep::usage =
	"CStep[t], a.k.a. Unit[-1][t], is the unit step function \
	which is 1 for t > 0, 0 for t < 0, and 1/2 at t = 0. \
	It is commonly used for continuous expressions t. \
	Under Mathematica 2.1 and higher, CStep is defined as UnitStep, \
	but UnitStep will always be displayed as CStep so the change \
	should be transparent to the user. \
	See also Step and Unit."

Delta::usage =
	"Delta[expr] is the Dirac delta function. \
	The area under this functions is 1 but it only has value \
	at the origin. \
	That is, Integrate[ Delta[t] g[t], {t, t1, t2} ] is g[0] \
	if t1 <= 0 <= t2, 0 otherwise. \
	Under Mathematica 2.1 and higher, Delta is defined as DiracDelta, \
	but DiracDelta will always be displayed as Delta so the change \
	should be transparent to the user. \
	It differs from the Kronecker delta function Impulse[t]."

DeltaIntegrate::usage =
	"DeltaIntegrate[fun, {x, xlower, xupper}] and DeltaIntegrate[fun, x] \
	are special hooks for the built-in Integrate routine. \
	Under Mathematica 2.1, the rules in Calculus`DiracDelta` \
	take precedence over this function."

DeltaPlot::usage =
	"DeltaPlot is a object that can generate a plot of only Dirac delta \
	functions or a combination of Dirac delta functions and another plot. \
	In the first case, either four or six arguments are passed: \
	deltafuns, t, tmin, tmax, ymin, ymax.\
	In the second case, either five or seven arguments are passed: \
	deltafuns, t, tmin, tmax, plot, ymin, ymax. \
	In both cases, ymin and ymax are the optional arguments. \
	DeltaPlot returns a list of plots suitable for sending to Show. \
	Note that 1-D Delta functions are plotted as arrows. \
	The variable $DeltaFunctionScaling dictates how Delta functions \
	are scaled when plotted:  None or Scaled."

DFT::usage =
	"DFT[L, n, k] is the forward discrete Fourier transform operator. \
	Here, L is the DFT length(s), n is the time-domain variable(s), and \
	k is the frequency-domain variable(s). \
	Multidimensional DFT's are supported. \
	Applying TheFunction to the DFT operator will invoke \
	the DFT rule base (object DFTransform)."

DTFT::usage =
	"DTFT[n, w] is the forward discrete-time Fourier transform operator. \
	Here, n is the time-domain variable(s), and w is the frequency-domain \
	variable(s). \
	Multidimensional DTFT's are supported. \
	Applying TheFunction to the DTFT operator will invoke the DTFT \
	rule base (object DTFTransform)."

Difference::usage =
	"Difference[k,n] represents the k-th backward difference with respect \
	to the variable n. \
	The first-order difference of f equals f[n] - f[n-1], \
	the second-order difference is f[n] - 2 f[n-1] + f[n-2], and so on. \
	Applying this operator to the sequence x[n] would be written as \
	Difference[k,n][ x[n] ]."

Dirichlet::usage =
	"Dirichlet[N, w] is the normalized aliased sinc function. \
	which is defined as Sin[N w / 2] / ( N Sin [w / 2] ). \
	It is periodic in w with a period of 2 Pi for N odd and a \
	period of 4 Pi for N even. \
	Note that Dirichlet[N, 0] is 1. \
	Aliases for Dirichlet are ASinc, AliasSinc, and AliasedSinc."

Discrete::usage =
	"Discrete is a possible value for a signal's domain."

DiscreteGraphics::usage =
	"DiscreteGraphics[{{x1,y1}, {x2,y2}, ... {xN,yN}}] will plot \
	the points as lines drawn from the x-axis to the y-value. \
	It takes two optional arguments:  a line thickness (or plot style) \
	and a point thickness. \
	Each is a real value from 0 to 1 indicating a width that \
	is a fraction of the size of the entire graph."

Domain::usage =
	"Domain is an option for many signal processing functions. \
	It is either Discrete or Continuous."

Downsample::usage =
	"Downsample[m,n] represents the downsampling operator. \
	Downsampling resamples a function at the points n = 0, m, 2m, 3m, ... \
	In N dimensions, n is a list of N indices and m is an N x N matrix. \
	Applying TheFunction to f[n] which is downsampled in one dimension, \
	(written as Downsample[m, n][ f[n] ]) will produce f[m n]."

DummyVariables::usage =
	"DummyVariables[dimension, variable] generates dimension dummy \
	variables with variable as the root. \
	For example, n can be generated by DummyVariables[1, n] or \
	DummyVariables[0, n]. \
	As another example, DummyVariables[3, t] returns {t1, t2, t3}. \
	In this cases, the generated variables have the same context \
	as variable. \
	DummyVariables[dimension, variable, context] can \
	be used to specify another context/package."

Extent1D::usage =
	"Extent1D[expression, var] returns an interval \
	{lower-limit, upper-limit} outside of which the function is zero. \
	This routine assumes that var is a real variable. \
	The routine returns None if the function is zero-valued for all var."

FT::usage =
	"FT[t, w] is the forward continuous Fourier transform operator. \
	Here, t is the time-domain variable(s) and \
	w is the frequency-domain variable(s). \
	Applying TheFunction to this operator will invoke the continuous-time \
	Fourier transform rule base (object CTFTransform)."

GetDeltaFunctions::usage =
	"GetDeltaFunctions[function, variable(s)] will extract \
	all delta functions from the argument function.  See also DeltaPlot."

Impulse::usage =
	"Impulse[n] is the unit Kronecker delta function. \
	At n = 0, the function evaluates to 1. \
	Elsewhere, it evaluates to 0."

Interleave::usage =
	"Interleave[n][x0, x1, ...] interleaves samples of signals \
	x0, x1, ..., which are functions of n. \
	This is only applicable to discrete signals."

IntervalsToFunction::usage =
	"IntervalsToFunction[interlist, n, step, pulse] \
	will translate the list of functions and intervals (interlist) \
	into a signal expression as a function of n. \
	Each element in the interlist is a list of three elements: \
	<function_of_n>, <n_minus>, and <n_plus>. \
	The function <function_of_n> will be defined for \
	<n_minus> <= n <= <n_plus>. \
	Note that <n_minus> can be an integer or -Infinity and \
	that <n_plus> can be an integer or Infinity. \
	The default values are n = Global`n, \
	step = Step, and pulse = Pulse. \
	This function also works for when n represents a continuous variable \
	(step = CStep and pulse = CPulse)."

InvDFT::usage =
	"InvDFT[L, k, n] is the inverse discrete Fourier transform operator. \
	Here, L is the DFT length(s), n is the time-domain variable(s), and \
	k is the frequency-domain variable(s). \
	Multidimensional inverse DFT's are supported. \
	Applying TheFunction to the InvDFT operator will invoke the \
	inverse DFT rule base (object InvDFTransform)."

InvDTFT::usage =
	"InvDTFT[w, n] is the inverse discrete-time Fourier transform \
	operator. \
	Here, n is the time-domain variable(s), and w is the \
	frequency-domain variable(s). \
	Multidimensional inverse DTFT's are supported. \
	Applying TheFunction to the InvDTFT operator will invoke the \
	inverse DTFT rule base InvDTFTransform."

InvFT::usage =
	"InvFT[w, t] is the inverse discrete Fourier transform operator. \
	Here, t is the time-domain variable(s) and w is the \
	frequency-domain variable(s). \
	Multidimensional Fourier transforms are supported. \
	Applying TheFunction to this operator will invoke the inverse \
	continuous-time Fourier transform rule base InvCTFTransform."

InvL::usage =
	"InvL[s, t] is the inverse Laplace transform operator. \
	Applying TheFunction to the InvL operator will invoke the \
	bilateral inverse Laplace rule base InvLaPlace."

InvZ::usage =
	"InvZ[z, n] is the inverse z-transform operator. \
	Applying TheFunction to the InvZ operator will invoke the inverse \
	z-transform rule base InvZTransform."

L::usage =
	"L[t, s] is the forward Laplace transform operator. \
	Applying TheFunction to the L operator will invoke the bilateral \
	Laplace rule base."

LineImpulse::usage =
	"LineImpulse[nlist, coefflist] represents a line impulse where \
	nlist is a list of variables like {n1,n2,n3} and coefflist is \
	a corresponding list of coefficients like {1,2,3}. \
	For the previous lists, the line impulse is a set of impulse \
	along the line n1 = 2 n2 = 3 n3."

OperatorInOperExpr::usage =
	"OperatorInOperExpr[ operator_expression ] returns the full \
	operator in operator_expression. \
	For example, Shift[2,n] would be returned from \
	OperatorInOperExpr[ Shift[2,n][x[n]] ]."

OperatorName::usage =
	"OperatorName[ operator_expression ] returns the head of the \
	expression. \
	For example, OperatorName[ Shift[2,n][x[n]] ] will return Shift."

ParametersInOperExpr::usage =
	"ParametersInOperExpr[ operator_expression ] returns the parameters \
	of the operator in operator_expression. \
	For example, (2,n) would be returned from \
	ParametersInOperExpr[ Shift[2,n][x[n]] ]."

Periodic::usage =
	"Periodic[k,n][f] indicates that f is a function of n which is \
	periodic with period of k."

PolyphaseDownsample::usage =
	"PolyphaseDownsample[m, h, n][x] is equivalent to downsampling \
	x by m (w/r to n) and then filtering (w/r to n) by a polyphase \
	form of the FIR filter h. \
	Hence, the filtering is carried out at the lower sampling rate."

PolyphaseResample::usage =
	"PolyphaseResample[l, h, m, n][x] represents an efficient polyphase \
	structure to perform sampling rate changes by a rational factor. \
	The structure is equivalent to first upsampling by l, then filtering \
	by the FIR filer h, and finally downsampling by m. \
	In the efficient structure, the filtering is carried out at the \
	lower sampling rate. \
	The resampling factors (matrices) l and m must be relatively prime."

PolyphaseUpsample::usage =
	"PolyphaseUpsample[l, h, n][x] is equivalent to filtering (w/r to n) \
	x by a polyphase form of the FIR filter h and then upsampling the \
	result by l (w/r to n). \
	Hence, the filtering is carried out at the lower sampling rate."

Pulse::usage =
	"Pulse[len] and Pulse[len, n] define a pulse which begins at \
	n = 0 and ends at n = len - 1. \
	A discrete pulse centered at the origin is written as \
	Pulse[len, n + (len - 1)/2] or Shift[-(len - 1)/2, n][Pulse[len, n]]. \
	Sometime, a pulse will be automatically simplified; \
	e.g., Pulse[0, n] is zero and Pulse[1, n] is really Impulse[n]."

RationalGCD::usage =
	"RationalGCD[e1, e2, ...] returns the greatest common divisor of the \
	numerators of elements ei divided by the greatest common divisor of \
	the denominators of elements ei. \
	Every element must be a polynomial or a number. \
	Each real-valued and complex-valued element is rationalized \
	before GCD's are computed."

Rev::usage =
	"Rev[n][x] represents the operation of reversing the direction of x, \
	with respect to the variables(s) n."

RewriteSummations::usage =
	"RewriteSummations[f, t, tlower, tupper] rewrites Summation \
	and Periodic operators that appear in f so that the domain \
	of the new expression includes t in [tlower, tupper]. \
	It relies on the Extent1D function to compute the domain \
	of the arguments to the Periodic and Summation operators."

RotateAxes::usage =
	"RotateAxes[a, {x,y}][f] represents the rotation of the \
	multi-dimensional function f in the (x,y) plane about the \
	origin by a radians. \
	Applying the object TheFunction to the RotateAxes operator will \
	generate a mathematical expression for the rotated function."

ScaleAxis::usage =
	"ScaleAxis[l,w][x] represents the compression by a factor of l (an \
	integer) of the continuous w axis of x, which is usually a \
	continuous-frequency signal."

ScalingFactor::usage =
	"ScalingFactor[expr, variable] returns the greatest common divisor \
	of all coefficients of powers of z in expr. \
	The powers of z that are functions of z are not considered. \
	For example, ScalingFactor[ a z + a^2 z^2, z ] returns a. \
	This function enables the implementation of the similarity \
	property for transforms."

SequencePlot::usage =
	"SequencePlot[f, {n, start, end}] plots real-valued 1-D sequences \
	and has the same options as Show. \
	You will have to apply Re or Im to f to plot the real or \
	imaginary values of the sequence, respectively. \
	SequencePlot[f, {n1, start1, end1}, {n2, start2, end2}] plots \
	2-D sequences and supports the same options as Plot3D."

SequenceToFunction::usage =
	"SequenceToFunction[seq], SequenceToFunction[seq, n], and \
	SequenceToFunction[seq, n, noffset] return the sequence seq, \
	{x[noffset], ..., x[noffset + N - 1]}, in symbolic form as a sum of \
	impulse functions. \
	For example, SequenceToFunction[{1, 2, 3}] becomes \
	Impulse[n] + 2 Impulse[n - 1] + 3 Impulse[n - 2]. \
	For multidimensional sequences, the argument n must be a list of \
	variables of appropriate length. \
	For example, SequenceToFunction[{{1,2},{3,4}}, {n1, n2}] returns \
	Impulse[n1,n2] + 2 Impulse[n1,n2-1] + 3 Impulse[n1-1,n2] + \
	4 Impulse[n1-1, n2-2]."

Shift::usage =
	"Shift[m,v][x] represents the shifting of the expression x by m \
	samples to the right in the v direction, i.e. x[v] --> x[v - m]."

SignalCleanup::usage =
	"SignalCleanup[ expr, t, nstart, nend ] converts the signal \
	expression expr in variables t into a normal Mathematica expression \
	that can be plotted over the interval [nstart, nend]. \
	It returns a list of two expressions:  the first consists of the \
	Delta functions extracted from expr, and the second is the rest \
	of the expression."

SignalPlot::usage =
	"SignalPlot[f, {t, start, end}] will plot f(t) as an one-dimensional, \
	continuous-time function. \
	It will show the real part as solid lines, \
	and the imaginary part as dashed lines. \
	Delta functions are plotted as upward pointing arrows. \
	SignalPlot[f, {t1, start1, end1}, {t2, start2, end2}] treats f \
	as a function of two variables t1 and t2. \
	SignalPlot supports the same options as Plot for 1-D signals \
	(functions) and Plot3D for 2-D signals (functions)."

SignalsInOperExpr::usage =
	"SignalsInOperExpr[ operator_expression ] returns the signal(s) \
	of the operator in operator_expression. \
	For example, { x[n] } would be returned from \
	SignalsInOperExpr[ Shift[2,n][x[n]] ]. \
	Another version SignalsInOperExpr[ operator_expression, operator ] \
	behaves as the first version if operator is the head of \
	operator_expression; otherwise, operator_expression is returned."

Sinc::usage =
	"Sinc[t] is the unit sinc function which is the ratio of \
	Sin[t] over t.  Note that Sinc[0] = 1."

SPIntegrationRules::usage =
	"SPIntegrationRules is a collection of integral rules that augment \
	the abilities of Mathematica's Integrate command."

SPSimplify::usage =
	"SPSimplify[e] will simplify expression e according to Mathematica's \
	simplification rules and the rules in SPSimplificationRules. \
	At each iteration, Simplify is called and SPSimplificationRules \
	is applied to the result. \
	Unlike Simplify, SPSimplify supports a Dialogue option. \
	You can specify which symbols to treat as real variables by using \
	the Variables option."

Step::usage =
	"Step[n] is the unit step function which is 1 for n >= 0 \
	and 0 otherwise. \
	It is commonly used for discrete n. \
	This function differs from CStep only at the origin."

Summation::usage =
	"Summation[i][f], Summation[i, iend][f], and \
	Summation[i, istart, iend, inc][f] represents the summation of f \
	with respect to variable i:  i = (istart) to (iend) step (inc). \
	The Summation operator is an abstraction of Mathematica's Sum object. \
	Applying the object TheFunction to a Summation operator will invoke \
	the Sum object if istart, iend, and inc are numbers."

TheFunction::usage =
	"TheFunction[object] returns the mathematical function embedded \
	in object. \
	For list and transform objects, this function returns \
	the first element after the head of the object; otherwise, the \
	contents of the slot TheFunction are returned. \
	For signal processing operations, \
	the equivalent mathematical operation is returned."

ToContinuous::usage =
	"ToContinuous[expr] replaces any discrete operator or signal \
	in expr with its continuous equivalent."

ToDiscrete::usage =
	"ToDiscrete[expr] replaces any continuous operator or signal \
	in expr with its discrete equivalent."

Unit::usage =
	"Unit[n][t] is the nth order continuous unit function in t. \
	Unit[0][t] is the Dirac delta function (see Delta). \
	Unit[-1][t] is the continuous step function (see CStep)."

Upsample::usage =
	"Upsample[k,n][f] represents the upsampling operation on f, \
	which is either an expression of n or a sequence (list) of values. \
	Upsampling inserts k-1 zeroes between every two samples. \
	In N dimensions, n is a list of N indices and k is an N x N matrix."

UpsampledFunction::usage =
	"UpsampledFunction[m, fill, n, upf] represents the upsampling of f. \
	In one-dimension, the upsampled form of f, upf, \
	is obtained by evaluating n -> n / m."

UpsampleFactor::usage =
	"UpsampleFactor[f, n] returns the upsampling index which can be \
	a negative or positive number. \
	If this routine returns 1 or -1, then no upsampling has occurred; \
	if it returns 0, then f is constant with respect to n."

UpsampleSequence::usage =
	"UpsampleSequence[seq,m,fill] inserts m-1 fill values between \
	every two elements of seq."

Z::usage =
	"Z[n, z] is the forward z-transform operator. \
	Applying TheFunction to the Z operator will invoke the z-transform \
	rule base ZTransform."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin["`Private`"]


(*  C O N S T A N T S  *)

axesdefaultvalue = If [ TrueQ[$VersionNumber >= 2.0], True, Automatic]

$DeltaFunctionScaling = Scaled


(*  M E S S A G E S  *)

CircularShift::badlength =
    "The sequence length parameter of the CircularShift \
    operator must be non-zero."

DeltaPlot::unresolved = "These symbols were assigned a value of 1: ``."

Downsample::convert =
    "Converting downsampling vector into a diagonal downsampling matrix: ``."
Downsample::badmD =
    "Downsampling factor has different dimensions than downsampled variables."

Periodic::convert =
    "Converting periodicity vector into a diagonal periodicity matrix: ``."
Periodic::badmD =
    "Period has different dimensions than the periodic variables."

Pulse::notsym = "Last argument to Pulse is not a symbol."
Pulse::noder =
    "Pulse is an integer-valued function of an integer variable, so it \
    does not have continuous derivative."

SequencePlot::nolist =
    "It does not make sense to plot a list of 2-D sequences."

Shift::badmD =
    "Shift vector has different dimensions than the specified variables."

SignalPlot::nolist =
    "It does not make sense to plot a list of 2-D functions."

Step::noder =
    "Step is an integer-valued function of an integer variable, so it \
    does not have continuous derivative."

Summation::extent =
    "Could not compute which part of the summation should be plotted."

Upsample::convert =
    "Converting upsampling vector into a diagonal upsampling matrix: ``."
Upsample::badmD =
    "Upsampling factor has different dimensions than upsampled variables."


(*  A L T E R     E X I S T I N G    F U N C T I O N S  *)

Format[ integrate ] := "Integrate"

Format[ substitute[g_, x_, newx_] ] :=
	SequenceForm[ "(", g, ")", Subscript[x -> newx] ]

substitute[g_, x_, newx_] := ( g /. x -> newx ) /;
	FreeQ[g, integrate] && FreeQ[g, Integrate]

SPIntegrationRules = 			(* incrementally defined below *)
      { integrate[ f_. Exp[ a_. Abs[b_. x_ + c_.] + d_. ],
		   vars1___, {x_Symbol, -Infinity, Infinity}, vars2___ ] :>
	  ( 2 / Abs[-a b] ) integrate[f Exp[d], vars1, vars2] /;
	Negative[a] && FreeQ[{a,b,c,d,f}, x],

	integrate[ f_. Exp[ - Abs[x_] - Abs[s_. x_ + a_] + d_. ],
		   vars1___, {x_Symbol, -Infinity, Infinity}, vars2___ ] :>
	  ( 1 + Abs[a] ) Exp[-Abs[a]] integrate[f Exp[d], vars1, vars2] /;
	FreeQ[{a,d,f}, x] && ( (s == 1) || (s == -1) ) }

postProcessingRules =
	{ integrate[f_] :> f,
	  integrate[f_, rest__] :> Integrate[f, rest] }

Unprotect[ Integrate ]
If [ $VersionNumber >= 2.0,
     ClearAttributes[ Integrate, ReadProtected ];
     DownValues[Integrate] =
	 Prepend[ DownValues[Integrate], 
		  Literal[ Integrate[ f_, x___ ] ] :>
			( ( integrate[f,x] //. SPIntegrationRules ) //.
			  postProcessingRules ) /;
		    ! SameQ[ integrate[f, x],
			     Replace[ integrate[f, x], SPIntegrationRules] ] ];
     SetAttributes[ Integrate, ReadProtected ],

     Integrate[ f_, x___ ] :=
	( ( integrate[f,x] //. SPIntegrationRules ) //. postProcessingRules ) /;
	! SameQ[integrate[f,x], Replace[integrate[f,x], SPIntegrationRules] ] ]
Protect[ Integrate ]


(*  L O C A L     F U N C T I O N S  *)

(*  breakup  *)
breakup[a_, z_] := a /; breakupQ[z][a]
breakup[a_Times, z_] := Select[a, breakupQ[z]]
breakup[x_, z_] := 1

breakupQ[z_][a_] := FreeQ[a, z]

(*  discBreakup  *)
discBreakup[a_, z_] := a /; discBreakupQ[z][a]
discBreakup[a_Times, z_] := Select[a, discBreakupQ[z]]
discBreakup[x_, z_] := 1

discBreakupQ[z_][a_] := FreeQ[a, z] && Implies[ NumberQ[a], IntegerQ[a] ]

(*  findIt  *)
findIt[ contextStr_, headStr_, args___ ] :=
	Block [ {symbol, symbolStr},
		Needs[ contextStr ];
		symbolStr = StringJoin[ contextStr, headStr ];
		symbol = ToExpression[symbolStr];
		symbol[ args ] ]

(*  gcdfilter --  rationalize floating point numbers		*)
gcdfilter[x_Complex] := ToCollection[ gcdfilter[Re[x]], gcdfilter[Im[x]] ]
gcdfilter[x_Real] := Rationalize[x]
gcdfilter[x_] := x

(*  goesto  *)
goesto[u_, v_] := texFix[ ReplaceWith[u, v] ]

(*  informatargs and informat  *)
SetAttributes[informatargs, {Listable}]
informatargs[m_, n_] := ToString[ StringForm["``  in  ``", m, n] ]

informat[op_, m_, n_] :=
Format[ SequenceForm[op, Subscript[ ColumnForm[ToList[informatargs[m, n]]] ]] ]

(* intdivide --  the Quotient primitive is not really integer division *)
intdivide[n_, m_] := Sign[n] Sign[m] Quotient[Abs[n], Abs[m]]

(*  inTeXformat  *)
inTeXformat[op_, m_, n_] := Subscripted[ op[texFix[m], texFix[n]] ]

(*  kludgeQ  *)
kludgeQ[___] := True

(*  matharg and mathargs  *)
matharg[ a_List ] :=
    ToCollection[ Drop[ Flatten[ Transpose[ { a, Table[",", {Length[a]}] } ] ],
			-1 ] ]
matharg[ x_ ] := x

mathargs[ h_, l_, n_, open_, close_ ] :=
	SequenceForm[ h, "_{", matharg[l], "} ", open, matharg[n], close ]

(*  mymax  *)
mymax[x_, y_] := minmaxSimplify[ Max[Expand[x], Expand[y]] ]

(*  mymin  *)
mymin[x_, y_] := minmaxSimplify[ Min[Expand[x], Expand[y]] ]

(*  minmaxSimplify  *)
minmaxSimplify[x_] :=
	x //. SignalProcessing`Support`SupCode`Private`MinMaxRules

(*  operatorOutputFormat  *)
operatorOutputFormat[ opstr_, blankstr_, sup_, sub_ ] :=
	SequenceForm[ ColumnForm[ { SequenceForm[opstr, " ", Superscript[sup]],
				    SequenceForm[blankstr, " ", sub] } ] ]

(*  sumformat  *)
sumformat[ str_, i_, istart_, iend_, 1, OutputForm ] :=
	operatorOutputFormat[ "Summation", "         ", iend,
			      SequenceForm[ i, "=", istart ] ]

sumformat[ str_, i_, istart_, iend_, 1, form_ ] :=
	Subscripted[ str[ iend, SequenceForm[i, "=", istart] ], {2}, {1} ]

sumformat[ str_, i_, istart_, iend_, inc_, OutputForm ] :=
	operatorOutputFormat[ "Summation", "         ", iend,
			      SequenceForm[ i, "=", istart, ", ",
					    i, "=", i, "+", inc ] ]

sumformat[ str_, i_, istart_, iend_, inc_, TeXForm ] :=
	Subscripted[ str[ SequenceForm[ "\\stackrel{", i, "=", istart, "}{",
					i, "=", i, "+", inc, "}" ],
			  iend ],
		     {1}, {2} ]

(*  texFix  *)
texFix[a_?VectorQ] := MatrixForm[ Transpose[{a}] ]
texFix[a_?MatrixQ] := SequenceForm[ "\\left[ ", MatrixForm[a], " \\right]" ]
texFix[x_] := x


(*  D E T E R M I N A T I O N     O F      V A R I A B L E S	*)

DummyVariables[i_, v_] :=
	If [ i < 2,
	     v,
	     Table[GenerateSymbol[v, index], {index, 1, i}] ] /;
	IntegerQ[i] && ( i >= 0 )

DummyVariables[i_, v_, context_] :=
	If [ i < 2,
	     GenerateSymbol[v, " ", context],
	     Table[GenerateSymbol[v, index, context], {index, 1, i}] ] /;
	IntegerQ[i] && ( i >= 0 )


(*  F U N C T I O N S  *)

(*  Aliasby  *)
Aliasby/: Aliasby[m_, w_][f_] := 1/m Periodic[2 Pi / m, w][ f ]

(*  CConvolve --  continuous convolution  *)
cconv[ args___ ] :=
	findIt[ "SignalProcessing`Support`Convolution`",
		"CTPiecewiseConvolution", args ]

CConvolve/: CConvolve[{t_Symbol}] := CConvolve[t]

CConvolve/: Extent1D[ CConvolve[t_, options___][x__], t_ ] :=
	Apply[ Plus, Map[ Extent1D[t], {x} ] ]

CConvolve/: Format[CConvolve[t_, options___]] :=	(* output forms  *)
		Subscripted[ CConvolve[t] ]
CConvolve/: Format[CConvolve[t_, options___][x1_, x2_], TeXForm] :=
		SequenceForm[ x1, " \\star_{", matharg[t], "} ", x2 ]
CConvolve/: Format[CConvolve[t_, options___][x1_, xrest__, xlast_], TeXForm] :=
		Block [	{newexpr, operator, oplist, xlist},
			operator = SequenceForm[" \\star_{", matharg[t], "} "];
			xlist = {x1, xrest};
			oplist = Table[ operator, {Length[xlist]} ];
			newexpr = Flatten[ Transpose[ {xlist, oplist} ] ];
			Apply[ SequenceForm, Append[newexpr, xlast] ] ]

CConvolve/: TheFunction[ CConvolve[t_Symbol, options___][x_, y_] ] :=
	cconv[x, y, t, options]

CConvolve/: TheFunction[ CConvolve[t_Symbol, options___][x_, y_, rest__] ] :=
	TheFunction[ CConvolve[t][ conv[x, y, t, options], rest ] ]

(*  CircularShift  *)
CircularShift/: CircularShift[n0_, N_] := CircularShift[n0, N, Global`n]

CircularShift/: CircularShift[n0_, 0, n_, rest___][x_] :=
	MyMessage[CircularShift::badlength, x]

CircularShift/: Extent1D[ CircularShift[n0_, N_, n_, options___][f_], n_ ] :=
	{ 0, N-1 }

CircularShift/:
	TheFunction[
	  CircularShift[n0_Integer, N_Integer, n_, options___][f_List] ] :=
	RotateRight[ Take[f, N], n0 ]

CircularShift/: TheFunction[ CircularShift[n0_, N_, n_, options___][f_] ] :=
	TheFunction[f Pulse[N, n]] /. ( n -> Mod[n + n0, N] )

CircularShift/: N[ CircularShift[n0_, N_, n_, options___][f_] ] :=
	N[ TheFunction[ CircularShift[n0, N, n][f] ] ]

CircularShift/: Format[ CircularShift[n0_, N_, n_, options___] ] :=
	informat[CircularShift, Mod[n + n0, N], n]
CircularShift/: Format[ CircularShift[n0_, N_, n_, options___], TeXForm ] :=
	inTeXformat[CircularShift, Mod[n + n0, N], n]

CircularShift/:
	GetOperatorVariables[ CircularShift[n0_, N_, n_, options___] ] := n

CircularShift/:
	Extent1D[ CircularShift[n0_, N_, n_, options___][f_], n_ ] := {0, N-1}
CircularShift/:
	Extent1D[ CircularShift[n0_, N_, n_, options___][f_], n_ ] := {0, N-1}

(*  CPulse  *)
CPulse/: CPulse[l_List, t_List] :=			(* multidimensional *)
	Apply[ Times, Map[ CPulse, Transpose[ { l, t } ] ] ] /;
	SameQ[ Length[l], Length[t] ]			(* separability     *)

CPulse/: CPulse[0, t_] := 0				(* error checking   *)
CPulse/: CPulse[0., t_] := 0		(* lim II(t/L) as L->0 = II(oo) = 0 *)

CPulse/: CPulse[{l_, t_}] := CPulse[l, t]
CPulse/: CPulse[l_] := CPulse[l, Global`t]		(* default args     *)

CPulse/: CPulse[l_?Positive, t_] := 1 /; N[0 < t < l]	(* definition	    *)
CPulse/: CPulse[l_?Negative, t_] := 1 /; N[l < t < 0]		     (* JMW *)
CPulse/: CPulse[l_?Positive, t_] := 0 /; N[(t < 0) || (t > l)]
CPulse/: CPulse[l_?Negative, t_] := 0 /; N[(t < l) || (t > 0)]	     (* JMW *)
CPulse/: CPulse[l_, t_] := 1/2   /; N[(t == 0) || (t == l)]
CPulse/: CPulse[Infinity, t_] := CStep[t]
CPulse/: CPulse[-Infinity, t_] := CStep[-t]
CPulse/: Extent1D[ CPulse[l_, a_. t_ + b_.], t_ ] :=
		Extent1DOrder[-b/a, (l-b)/a] /;
		FreeQ[{a,b,l}, t]

CPulse/: Format[ CPulse[l_, n_] ] :=			(* output forms     *)
	 Subscripted[ CPulse[l, n], {1} ]

CPulse/: Format[ CPulse, TeXForm ] := "\\sqcap"	
CPulse/: Format[ CPulse[l_, n_], TeXForm ] :=
	mathargs[ "CPulse", l, n, "(", ")" ]

Derivative[0,1][CPulse] :=				(* derivative rule *)
	( Sign[#1] (Delta[#2] - Delta[#2 - #1]) )&		    (* JMW *)

SPIntegrationRules =				      (* integration rules *)
Join[ SPIntegrationRules,
      { integrate[ f_. CPulse[L_, a_. x_ + b_.] + c_.,
		   vars1___, x_Symbol, vars2___ ] :>
		( 1/a CPulse[L, a x + b] *
		    substitute[integrate[f, vars1, x, vars2], x, a x + b] +
		  1/a CStep[x - (L-b)/a] *
		    integrate[f, vars1, {x, -b/a, (L-b)/a}, vars2] +
		  integrate[c, vars1, x, vars2] ) /;
	FreeQ[{a, b, L}, x],

	integrate[ expr_. CPulse[L_, a_. x_ + b_.] + c_.,
		   vars1___, {x_Symbol, x1_, x2_}, vars2___ ] :>
	      ( Sign[x2 - x1] *
		cpulseintersect[ Min[x1, x2], Max[x1, x2],
				 Min[-b/a, (L - b)/a], Max[-b/a, (L - b)/a] ] *
		integrate[expr,
			  vars1,
			  {x, Max[Min[x1,x2], Min[-b/a, (L - b)/a]],
			      Min[Max[x1,x2], Max[-b/a, (L - b)/a]]},
			  vars2] +
		integrate[c, vars1, {x, x1, x2}, vars2] ) /;
	FreeQ[{a, b, L}, x]
      } ]

cpulseintersect[x1_, x2_, x3_, x4_] := CStep[x4 - x1] CStep[x2 - x3]

(* CStep  *)

CStep/: CStep[t_, args__] := CStep[t] CStep[args]	(* multidimensional *)
CStep/: CStep[List[args__]] := CStep[args]		(* separability     *)

If [ TrueQ[ $VersionNumber >= 2.1 ],

     CStep[t_] = UnitStep[t];
     Unprotect[UnitStep];
     SetOptions[ UnitStep, ZeroValue -> 1/2 ];
     UnitStep/: UnitStep[n_Symbol + m_.] UnitStep[n_Symbol  + k_.] :=
	UnitStep[n + Min[m,k]] /;
	NumberQ[m] && NumberQ[k];
     UnitStep/: UnitStep[n_Symbol + m_.] UnitStep[-n_Symbol + k_.] :=
	CPulse[k + m, n + m] /;
	N[ -m <= k ];
     UnitStep/: UnitStep[n_Symbol + m_.] UnitStep[-n_Symbol + k_.] := 0 /;
	N[ -m > k ];
     UnitStep/: Format[ UnitStep, TeXForm ] :=		(* output form     *)
       "u_{-1}";
     UnitStep/: Extent1D[ UnitStep[a_. t_ + b_.], t_ ] :=
		Extent1DOrder[-b/a, Sign[a] Infinity] /;
		FreeQ[{a,b}, t];
     Format[ UnitStep ] := "CStep";
     Protect[UnitStep],

     CStep/: CStep[t_?Positive] := 1;			(* definition	   *)
     CStep/: CStep[t_?Negative] := 0;
     CStep/: CStep[0] := 1/2;
     CStep/: CStep[0.] := 0.5;
					 (* automatic simplification rules *)
     CStep/: CStep[n_Symbol + m_.] CStep[n_Symbol  + k_.] :=
	CStep[n + Min[m,k]] /;
	NumberQ[m] && NumberQ[k];
     CStep/: CStep[n_Symbol + m_.] CStep[-n_Symbol + k_.] :=
	CPulse[k + m, n + m] /;
	N[ -m <= k ];
     CStep/: CStep[n_Symbol + m_.] CStep[-n_Symbol + k_.] := 0 /;
	N[ -m > k ];

     CStep/: Format[ CStep, TeXForm ] := "u_{-1}";	(* output form     *)

     CStep/: Extent1D[ CStep[a_. t_ + b_.], t_ ] :=
		Extent1DOrder[-b/a, Sign[a] Infinity] /;
		FreeQ[{a,b}, t];

     Derivative[1][CStep] := Delta			(* derivative rule *)
]

SPIntegrationRules =
Join[ SPIntegrationRules, 
      {	integrate[ f_. CStep[a_. x_ + b_.] + c_.,
		   vars1___, x_Symbol, vars2___ ] :>
		1/a CStep[a x + b] *
		  substitute[integrate[f, vars1, x, vars2], x, a x + b] +
		  integrate[c, vars1, x, vars2] /;
	FreeQ[{a, b}, x],

	integrate[ expr_. CStep[a_. x_ + b_.] + c_.,
		   vars1___, {x_Symbol, x1_, x2_}, vars2___ ] :>
	Block [	{domain, xmax, xmin},
		domain = Extent1D[CStep[a x + b] CPulse[x2 - x1, x + x1], x];
		If [ SameQ[domain, None],
		     0,
		     {xmin, xmax} = domain;
		     Sign[x2 - x1] *
		     integrate[expr / Abs[a], vars1, {x, xmin, xmax}, vars2] +
		     integrate[c, vars1, {x, x1, x2}, vars2] ] ] /;
	FreeQ[{a, b}, x]
      } ]

(*  Convolve --  discrete convolution  *)
dconv[ args___ ] :=
	findIt[ "SignalProcessing`Support`Convolution`",
		"DTPiecewiseConvolution", args ]

Convolve/: Convolve[{n_Symbol}] := Convolve[n]

Convolve/: Extent1D[ Convolve[n_, rest___][x__], n_ ] :=
	Apply[ Plus, Map[ Extent1D[n], {x} ] ]

Convolve/: Format[Convolve[n_]] :=			(* output form  *)
		Subscripted[Convolve[n]]
Convolve/: Format[Convolve[n_][x1_, x2_], TeXForm] :=
		SequenceForm[ x1, " \\star_{", matharg[n], "} ", x2 ]
Convolve/: Format[Convolve[n_][x1_, xrest__, xlast_], TeXForm] :=
		Block [	{newexpr, operator, oplist, xlist},
			operator = SequenceForm[" \\star_{", matharg[n], "} "];
			xlist = {x1, xrest};
			oplist = Table[ operator, {Length[xlist]} ];
			newexpr = Flatten[ Transpose[ {xlist, oplist} ] ];
			Apply[ SequenceForm, Append[newexpr, xlast] ] ]

Convolve/: TheFunction[ Convolve[n_Symbol, options___][x_, y_] ] :=
	dconv[x, y, n, options]

Convolve/: TheFunction[ Convolve[n_Symbol, options___][x_, y_, rest__] ] :=
	TheFunction[ Convolve[n][ dconv[x, y, n, options], rest ] ]

(*  Delta  *)

If [ TrueQ[ $VersionNumber >= 2.1 ],

     (* Remove undesirable automatic simplification rule in "DiracDelta.m" *)
     If [ 2.2 <= $VersionNumber < 2.3,
	  TagUnset[ DiracDelta,
		    (Calculus`DiracDelta`Private`y_)^
	 	    (Calculus`DiracDelta`Private`n_.)*
		    DiracDelta[___, Calculus`DiracDelta`Private`y_] ] ];

     Delta[t_] := DiracDelta[t];
     DiracDelta/: DiracDelta[t_] := 0 /; N[ t != 0 ];
     DiracDelta/: DiracDelta[a_. x_Symbol + b_.] DiracDelta[c_. x_ + d_.] :=
	0 /;
	( b c != a d );
     DiracDelta/: Extent1D[ DiracDelta[a_. t_ + b_.], t_ ] := { -b/a, -b/a } /;
	FreeQ[{a,b}, t];
     DiracDelta/: DiracDelta[List[args__]] := DiracDelta[args];
     DiracDelta/: Format[ DiracDelta, TeXForm ] :=	(* output form       *)
	"\\delta";
     Derivative[1][DiracDelta] := Unit[1];		(* derivative rule   *)
     Format[ DiracDelta ] := "Delta",

     Delta/: Delta[Infinity] := 0;			(* definition	     *)
     Delta/: Delta[t_] := 0 /; N[ t != 0 ];
     Delta/: Delta[-Infinity] := 0;
     Delta/: Delta[a_. x_Symbol + b_.] Delta[c_. x_ + d_.] := 0 /;
       ( b c != a d );
     Delta/: Extent1D[ Delta[a_. t_ + b_.], t_ ] := { -b/a, -b/a } /;
	FreeQ[{a,b}, t];
     Delta/: Format[ Delta, TeXForm ] := "\\delta";	(* output form       *)
     Derivative[1][Delta] := Unit[1]			(* derivative rule   *)
]

Delta/: Delta[ t_List ] := Apply[ Times, Map[ Delta, t ] ]

	(* The following Delta rules were written by John Mitchell *)
	(* and modified by Brian Evans.				   *)
getivars[ x_Symbol ] := x
getivars[ {x_Symbol, xl_, xu_} ] := x

SPIntegrationRules =				(* integration rules *)
Join[ SPIntegrationRules,
      { integrate[a_. + expr_. Delta[term_], vars_] :>
	  integrate[a, vars] + DeltaIntegrate[expr Delta[term], vars] /;
	! FreeQ[ term, getivars[vars] ],

	integrate[a_. + expr_. Delta[term_], vars1___, vars_, vars2___] :>
	  integrate[a, vars1, vars, vars2] + 
	    integrate[ DeltaIntegrate[expr Delta[term], vars], vars1, vars2 ] /;
	! FreeQ[ term, getivars[vars] ]
      } ]

(*  DeltaIntegrate  *)
DeltaIntegrate[expr_. Delta[a_. x_ + b_.], x_Symbol] :=
	Limit[expr, x -> -b/a] CStep[x + b/a] / Abs[a] /;
	FreeQ[{a,b}, x] && FreeQ[expr, Delta]
DeltaIntegrate[expr_. Delta[a_. x_ + b_.], {x_Symbol, x1_, x1_}] := 0
DeltaIntegrate[expr_. Delta[a_ x_ + b_.], {x_Symbol, x1_, x2_}] :=
	If [ N[x1 > x2],
	     -1 DeltaIntegrate[expr Delta[a x + b], {x, x2, x1}],
	     Integrate[expr Delta[x + b/a] / Abs[a], {x, x1, x2}],
	     Integrate[expr Delta[x + b/a] / Abs[a], {x, x1, x2}] ] /;
	FreeQ[{a,b}, x] && FreeQ[expr, Delta]
DeltaIntegrate[expr_. Delta[x_ + c_.], {x_Symbol, x1_, x2_}] :=
	If [ N[x1 > x2],
	     -1 DeltaIntegrate[expr Delta[x + c], {x, x2, x1}],
	     Limit[expr, x -> -c] CStep[-c - x1] CStep[c + x2],
	     Limit[expr, x -> -c] CStep[-c - x1] CStep[c + x2] ] /;
	FreeQ[c, x] && FreeQ[expr, Delta]
		(* CStep[-c - x1] CStep[c + x2] <--> x1 <= -c <= x2 *)
DeltaIntegrate[Delta[a_. x_ + b_.] Delta[c_.x_ + d_.],
		      {x_Symbol, x1_, x2_}] :=
	If [ N[x1 > x2],
	     -1 DeltaIntegrate[Delta[a x + b] Delta[c x + d], {x, x2, x1}],
	     (1/Abs[a c]) Delta[-b/a + d/c] *
		CStep[-(b/a) - x1] CStep[(b/a) + x2],
	     (1/Abs[a c]) Delta[-b/a + d/c] *
		CStep[-(b/a) - x1] CStep[(b/a) + x2] ] /;
	FreeQ[{a,b,c,d}, x]
DeltaIntegrate[Delta[a_. x_ + b_.]^2, {x_Symbol, x1_, x2_}] :=
	If [ N[x1 > x2],
	     -1 DeltaIntegrate[Delta[a x + b]^2, {x, x2, x1}],
	     (1/a^2) Delta[0] CStep[-(b/a) - x1] CStep[(b/a) + x2],
	     (1/a^2) Delta[0] CStep[-(b/a) - x1] CStep[(b/a) + x2] ] /;
	FreeQ[{a,b}, x]

(*  DeltaPlot--  plot Delta functions as special case of continuous plotting  *)

rewriteSymbolicSummation[index_, istart_, iend_, inc_,
			 c_, b_, a_, {t_, tmin_, tmax_} ] :=
  	  Block [ {aa = a, adjfun, bb = b, cc = c, defaults, f1, f2,
		   i, indexfunoft, minindex, maxindex, rules, varlist},
		  varlist = Select[GetVariables[{a, b, c}],
			   	   ((! SameQ[#1, t]) &&
				    (! SameQ[#1, index]))&];
		  If [ ! EmptyQ[varlist],
		       defaults = Map[(#1 -> 1)&, varlist];
		       aa = a /. defaults;
		       bb = b /. defaults;
		       cc = c /. defaults ];
		  rules = Solve[ bb t + aa == 0, index ];
		  indexfunoft = Replace[index, First[rules] ];
		  f1 = N[ indexfunoft /. t -> tmin ];
		  f2 = N[ indexfunoft /. t -> tmax ];
		  minindex = Max[ istart, Ceiling[Min[f1, f2]] ];
		  maxindex = Min[ iend, Max[f1, f2] ];
		  adjfun = ((cc /. t -> - aa / bb) Delta[bb t + aa]) /.
			   index -> i;
		  Apply[ Plus,
			 Table[adjfun, {i, minindex, maxindex, inc}] ] ]

dpcleanup[deltafuns_, t_, tmin_, tmax_] :=
	Block [	{delfuns, itemp, varlist = {}},
		delfuns = deltafuns /.
			    ( d_ Summation[index_, istart_, iend_, inc_][c_] :>
			      Summation[index, istart, iend, inc][c d] );
		delfuns = delfuns /.
		{ Summation[index_, istart_, iend_, inc_]
			   [c_. Delta[b_. t + a_.]] :>
		    rewriteSymbolicSummation[ index, istart, iend, inc, c,
					      b, a, {t, tmin, tmax} ],
		  Periodic[k_, t][c_. Delta[b_. t + a_.]] :>
		    rewriteSymbolicSummation[ itemp, -Infinity, Infinity, 1,
					      c, b, a + itemp k b,
					      {t, tmin, tmax} ] };
		varlist = Sort[ varlist ~Join~
				Select[ GetVariables[delfuns],
					(! SameQ[#1, t])& ] ];
		If [ ! EmptyQ[varlist],
		     Message[ DeltaPlot::unresolved, varlist ];
		     delfuns = delfuns /. Map[(#1 -> 1)&, varlist] ];

		delfuns ]

getheight[ c_. Delta[a_. t_ + b_.], t_ ] := N[ (c /. t -> -b/a) / Abs[a] ]

getlocation[ c_. Delta[a_. t_ + b_.], t_ ] := N[ - b / a ]

DeltaPlot[deltafuns_, t_, start_, end_] :=
	DeltaPlot[deltafuns, t, start, end, 0, 1]

DeltaPlot[deltafuns_, t_, start_, end_, plot_] := plot /;
	FreeQ[deltafuns, t]

    (* Use PlotRange as function to find range of plot [see MJ v1 #4 p47] *)
DeltaPlot[deltafuns_, t_, start_, end_, plot_] :=
	Block [	{xmax, xmin, ylist, ymax, ymin},
		{{xmin, xmax}, {ymin, ymax}} = PlotRange[plot];
		DeltaPlot[deltafuns, t, start, end, plot, ymin, ymax] ]

DeltaPlot[deltafuns_, t_, start_, end_, min_, max_] := NullPlot /;
	FreeQ[deltafuns, t]

	(* every version of DeltaPlot calls this one *)
DeltaPlot[deltafuns_, t_, start_, end_, min_, max_] :=
	Block [	{bottom, delfuns, height, heightList, length,
		 locations, nend, nstart, width},
		nstart = N[start];
		nend = N[end];
		delfuns = Expand[ dpcleanup[deltafuns, t, nstart, nend] ];
		delfuns = If [ SameQ[Head[delfuns], Plus],
			       Apply[List, delfuns],
			       { delfuns } ];
		delfuns = Select[ delfuns,
			      N[LessEqual[nstart, getlocation[#, t], nend]]& ];
		height = N[max - min];
		width = nend - nstart;
		locations = Map[ getlocation[#1, t]&, delfuns];
		If [ min <= 0,
		     bottom = 0; length = max,
		     bottom = min; length = height ];
		length = Min[height, max];
		Which [ SameQ[locations, {}],
			  NullPlot,
			SameQ[$DeltaFunctionScaling, None],
			  Map[Arrow2D[{#1, bottom}, width, height, length]&,
			      locations ],
			True,
			  heightList = Map[ getheight[#1, t]&, delfuns ];
			  Map[ Arrow2D[{#[[1]],bottom}, width, height, #[[2]]]&,
			       Transpose[ {locations, heightList} ] ] ] ]

DeltaPlot[deltafuns_, t_, start_, end_, plot_, min_, max_] :=
	Block [	{deltaplot, ymax = max, ymin = min},
		If [ ymax == ymin,
		     Which [ ymin > 0,  ymin = -ymax,
			     ymin == 0, ymax = 1,
			     ymin < 0,  ymax = -ymin ] ];
		deltaplot = DeltaPlot[deltafuns, t, start, end, ymin, ymax];
		If [ SameQ[deltaplot, NullPlot],
		     plot,
		     Prepend[ deltaplot, plot ] ] ]

(*  Difference  *)
Difference/: TheFunction[Difference[k_, n_, options___][f_]] :=
	Block [	{jj, newf},
		newf = TheFunction[f] /. ( n ~ReplaceWith~ (n - jj) );
		Apply[ Plus,
		       Table[(-1)^jj Binomial[k,jj] newf, {jj, 0, k}] ] ] /;
	IntegerQ[k] && k > 0

Difference/:
	Extent1D[ Difference[k_, n_, options___][f_], n_ ] := Extent1D[f, n]

Difference/: N[Difference[1, n_, options___][f_]] :=
		f - ( f /. ( n ~ReplaceWith~ (n - 1) ) )
Difference/: N[Difference[k_, n_, options___][f_]] :=
	Apply[Plus,
	      Table[(-1)^jj Binomial[k,jj] f /. ( n ~ReplaceWith~ (n - jj) ),
		    {jj, 0, k}] ] /;
	IntegerQ[k] && k > 0

Difference/:
	Format[ Difference[k_, n_, options___] ] := informat[Difference, k, n]
Difference/: Format[ Difference, TeXForm ] := "\\Delta"
Difference/: Format[ Difference[k_, n_, options___], TeXForm ] :=
		inTeXformat[ Difference, k, n ]

Difference/: GetOperatorVariables[ Difference[k_, n_, options___] ] := n

(*  Dirichlet  *)
Dirichlet/: Dirichlet[len_, 0] := 1			(* definition	*)
Dirichlet/: Dirichlet[len_, k_?EvenQ Pi] := len (-1)^(len - 1)
Dirichlet/: Dirichlet[len_Integer, w_?NumberQ] :=
		Sin[len w / 2] / ( len Sin[w / 2] )

Dirichlet/: TheFunction[Dirichlet[len_, w_]] :=
		Sin[len w / 2] / ( len Sin[w / 2] )

Dirichlet/: N[ Dirichlet[len_, w_] ] := N [ TheFunction[Dirichlet[len, w]] ]

Dirichlet/: Format[ Dirichlet[l_, n_] ] :=		(* output forms     *)
		Format[ Dirichlet Subscript[l] [n] ]

Derivative[0,1][Dirichlet] :=
	( Cos[(#1) (#2) / 2] / (2 Sin[(#2) / 2]) -
	  ( Cos[(#2) / 2] / Sin[(#2) / 2] ) Dirichlet[#1, #2] / 2 ) &

(*  DiscreteGraphics  *)
oneline[xycoord_] :=
	ToCollection[ Line[{{xycoord[[1]], 0}, xycoord}], Point[xycoord] ]
DiscreteGraphics[ coordlist_List ] :=
	DiscreteGraphics[ coordlist, Thickness[0.008] ]
DiscreteGraphics[ coordlist_List, lthick_Real, rest___ ] :=
	DiscreteGraphics[ coordlist, Thickness[lthick], rest ]
DiscreteGraphics[ coordlist_List, lstyle_, pthick_:0.024 ] :=
	Graphics[ Join[ { lstyle, PointSize[pthick] },
			N [ Map[oneline, coordlist] ] ] ]

(*  Downsample [Myers, 219] [Bamberger, ch. 2]  *)

	(* Error checking of downsample parameters  *)

Downsample/: Downsample[m_List, n_List, options___][f_] :=
	MyMessage[ Downsample::convert,
		   Downsample[DiagonalMatrix[m], n][f],
		   m ] /;
	VectorQ[m] && SameQ[Length[m], Length[n]] && ! PatternQ

Downsample/: Downsample[m_List, n_List, options___][f_] :=
	MyMessage[Downsample::badmD, f] /;
	VectorQ[m] && ! SameQ[Length[m], Length[n]]

Downsample/: Downsample[m_List, n_List, options___][f_] :=
	MyMessage[Downsample::badmD, f] /;
	MatrixQ[m] && ! Apply[SameQ, Append[Dimensions[m], Length[n]]]

Downsample/: Downsample[m_List, n_Symbol, options___][f_] :=
	MyMessage[Downsample::badmD, f]

		(*  degenerate cases  *)

Downsample/: Downsample[1, n_Symbol, options___][c_] := c
Downsample/: Downsample[-1, n_Symbol, options___][c_] := Rev[n][c]

	(* convert upsample into a functional form  *)

Downsample/: TheFunction[Downsample[m_, n_Symbol, options___][f_]] :=
	TheFunction[f] /. ( n -> m n )
Downsample/: TheFunction[Downsample[m_, n_List, options___][f_]] :=
	( TheFunction[f] /. ( n ~ReplaceWith~ (m . n) ) ) /;
	MatrixQ[m] && VectorQ[n]

Downsample/: N [ Downsample[m_, n_, options___][f_] ] :=
	N [ TheFunction[Downsample[m,n][f]] ]

	(*  extent of downsample operation  *)

Downsample/: Extent1D[ Downsample[m_, n_, options___][f_], n_ ] :=
	intdivide[ Extent1D[f, n], m ]

	(* formatting of the downsample operator *)

Downsample/: Format[ Downsample[m_, n_List, options___] ] :=
	Block [	{bar},
		bar = TableForm[ Apply[ Table,
					{ "  |  ", Dimensions[n] } ] ];
		Format[ SequenceForm[ Downsample,
				      Subscript[ TableForm[m] ],
				      Subscript[ bar ],
				      Subscript[ TableForm[n] ] ] ] ]
Downsample/: Format[ Downsample[m_, n_Symbol, options___] ] :=
	Subscripted[ Downsample[m,n] ]

Downsample/: Format[ Downsample, TeXForm ] := "\\downarrow"
Downsample/: Format[ Downsample[m_, n_, options___], TeXForm ] :=
	inTeXformat[ Downsample, m, n ]

Downsample/: GetOperatorVariables[ Downsample[m_, n_, options___] ] := n

(*  DFT  [Myers, 221]  *)
DFT/: DFT[L_, n_] := DFT[L, n, DummyVariables[Length[n], Global`k]]
DFT/: DFT[L_, n_, k_, options___] [ InvDFT[L_, ki_, n_, ioptions___] [f_] ] := 
	If [ SameQ[k, ki], f, f /. ReplaceWith[ki, k] ] /;
	Length[k] == Length[ki]

DFT/: DFT^-1[a__] := InvDFT[a]

DFT/: Format[ DFT[L_, n_, k_, options___] ] :=
	Subscripted["DFT"[n ~goesto~ k, SequenceForm[" L = ", L]]]
DFT/: Format[ DFT, TeXForm ] := "{\\cal DFT}"
DFT/: Format[ DFT[L_, n_, k_, options___], TeXForm ] :=
	Subscripted["{\\cal DFT}"[n ~goesto~ k, SequenceForm[" {\rm L} = ", L]]]

DFT/: GetOperatorVariables[ DFT[L_, n_, k_, options___] ] := n

DFT/: Extent1D[ DFT[L_, n_, k_, options___][f_], n_ ] := {0, L-1}
DFT/: Extent1D[ DFT[L_, n_, k_, options___][f_], k_ ] := {0, L-1}

DFT/: TheFunction[ DFT[L_, n_, k_, options____][f_] ] :=
	findIt[ "SignalProcessing`Digital`DFT`",
		"DFTransform", f, L, n, k, options ]

(*  DTFT  *)
DTFT/: DTFT[n_] := DTFT[n, DummyVariables[Length[n], Global`w]]
DTFT/: DTFT[n_, w_, options___] [ InvDTFT[wi_, n_, ioptions___] [f_] ] :=
	If [ SameQ[w, wi], f, f /. ReplaceWith[wi, w] ] /;
	Length[w] == Length[wi]

DTFT/: DTFT^-1[a__] := InvDTFT[a]

DTFT/: Format[ DTFT[n_, w_, options___] ] :=
	Subscripted[ "DTFT"[ n ~goesto~ w ] ]
DTFT/: Format[ DTFT, TeXForm ] := "{\\cal DTFT}"
DTFT/: Format[ DTFT[n_, w_, options___], TeXForm ] :=
	Subscripted[ "{\\cal DTFT}"[ n ~goesto~ w ] ]

DTFT/: TheFunction[ DTFT[n_, w_, options____][f_] ] :=
	findIt[ "SignalProcessing`Digital`DTFT`",
		"DTFTransform", f, n, w, options ]

(*  Extent1D  *)
lastExpression = Null

modifiedQ[x_, n_] :=
	Block [	{},
		lastExpression = x /.
		  ( f_ :> Collect[f, n] /;
		    PolynomialQ[f, n] || MixedPolynomialQ[Expand[f], n] );
		! SameQ[ x, lastExpression ] ]

Extent1DUnion[ {x1_, y1_}, {x2_, y2_} ] :=
	{ mymin[x1, x2], mymax[y1, y2] }
Extent1DUnion[ ___ ] := None

Extent1DIntersection[ {x1_, y1_}, {x2_, y2_} ] :=
	Extent1DCheckOrder[ mymax[x1, x2], mymin[y1, y2] ]
Extent1DIntersection[ ___ ] := None

Extent1DOrder[ x1_, y1_ ] :=
	{ mymin[x1, y1], mymax[x1, y1] }
Extent1DOrder[ ___ ] := None

Extent1DCheckOrder[ x_, y_ ] := {x, y} /; N[x <= y] || SameQ[mymin[x, y], x]
Extent1DCheckOrder[ x_, y_ ] := None /; N[x > y] || SameQ[mymax[x, y], x]
Extent1DCheckOrder[ None ] := None 

Extent1D[t_][x_] := Extent1D[x, t]

Extent1D[x_ + y_, n_] := Extent1DUnion[ Extent1D[x, n], Extent1D[y, n] ]
Extent1D[x_ y_, n_] := Extent1DIntersection[ Extent1D[x, n], Extent1D[y, n] ]
Extent1D[0, n_] := None
Extent1D[x_, n_] := Extent1D[lastExpression, n] /; modifiedQ[x, n]
Extent1D[x_, n_] := {-Infinity, Infinity}

(*  FT  *)
FT/: FT[t_] := FT[t, DummyVariables[Length[t], Global`w]]
FT/: FT[t_, w_, options___] [ InvFT[wi_, t_, ioptions___] [f_] ] :=
	If [ SameQ[w, wi], f, f /. ReplaceWith[wi, w] ] /;
	Length[w] == Length[wi]

FT/: FT^-1[a__] := InvFT[a]

FT/: Format[ FT[t_, w_, options___] ] :=
	Subscripted[ "F"[ t ~goesto~ w ] ]
FT/: Format[ FT, TeXForm ] := "{\\cal F}"
FT/: Format[ FT[t_, w_, options___], TeXForm ] :=
	Subscripted[ "{\\cal F}"[ t ~goesto~ w ] ]

FT/: TheFunction[ FT[t_, w_, options____][f_] ] :=
	findIt[ "SignalProcessing`Analog`Fourier`",
		"CTFTransform", f, t, w, options ]

(*  GetDeltaFunctions  *)
choose[{a_, aterm_}] := If [ SameQ[aterm, 0], a, aterm ]

	(* rules specific for 1-D functions of t  *)
GetDeltaFunctions[ c_. Delta[a_. t_Symbol + b_.], t_Symbol ] :=
	c Delta[a t + b]

	(* rules specific for m-D functions of t1, t2, ...  *)
GetDeltaFunctions[ c_. Delta[a_. t_Symbol + b_.], tlist_List ] :=
	c Delta[a t + b] /;
	MemberQ[tlist, t]

	(* rules for all dimensions  *)
GetDeltaFunctions[ a_ + b_, rest__ ] :=
	GetDeltaFunctions[a, rest] + GetDeltaFunctions[b, rest]
GetDeltaFunctions[ h_[b__], rest__ ] :=
	Block [ {arglist, deltalist},
		arglist = {b};
		deltalist = Map[ GetDeltaFunctions[#1, rest]&, arglist ];
		If [ SameQ[Apply[Plus, deltalist], 0],
		     0,
		     Apply[h, Map[choose, Transpose[{arglist, deltalist}]]] ] ]
GetDeltaFunctions[ x_, rest__ ] := 0

(*  Impulse  *)
Impulse/: Impulse[Infinity] := 0		 	  (* definition       *)
Impulse/: Impulse[n_] := 0	/; N[ n != 0 ]
Impulse/: Impulse[n_] := 1	/; N[ n == 0 ]
Impulse/: Impulse[-Infinity] := 0

Impulse/: Extent1D[ Impulse[a_. n_ + b_.], n_ ] := { -b/a, -b/a } /;
	FreeQ[{a,b}, n]

Impulse/: Impulse[n_, args__] := Impulse[n] Impulse[args] (* multidimensional *)
Impulse/: Impulse[List[args__]] := Impulse[args]	  (* separability     *)

Impulse/: Format[ Impulse[n_], TeXForm ] :=		  (* output form rule *)
	  SequenceForm["\\delta[", n, "]" ]

	(* the derivative of the Kronecker impulse is another *)
	(* singularity function, the doublet, not coded here. *)

(*  Interleave [Myers, 219-220]  *)
Interleave/: Interleave[n_, options___][x_] := x 	  (* definition	      *)
Interleave/: TheFunction[Interleave[n_, options___][x1_, x__]] :=
	Block [	{index = Unique["k"], L, xlist},
		xlist = List[x1, x];
		L = Length[xlist];
		Summation[index, 0, L-1, 1]
			 [ Shift[index, n] [Upsample[L, n]
					     [ Element[xlist, index + 1] ] ] ] ]
Interleave/: N [ Interleave[n_, options___][x1_, x__] ] :=
	N [ TheFunction[Interleave[n][x1,x]] ]

Interleave/: Format[ Interleave[n_, options___] ] :=	 (* output form rules *)
		Subscripted[ Interleave[n] ]

Interleave/: Format[ Interleave[n_, options___][x1_, xrest__], TeXForm ] :=
		Subscripted[ "{\it Interleave}"[n, x1, xrest], {1} ]

(*  IntervalsToFunction  *)
IntervalsToFunction[interlist_, n_:Global`n, step_:Step, pulse_:Pulse] :=
	Block [	{int, len, signal},

		MakeOne[interval_] :=
		  Block [
		    {f, nminus, nplus, pulselen},
		    f = interval[[1]];
		    nminus = interval[[2]];
		    nplus = interval[[3]];
		    pulselen = nplus - nminus + 1;
		    Which [ SameQ[nminus, -Infinity],
			      f step[nplus - n],
			    SameQ[nplus, Infinity],
			      f step[n - nminus],
			    True,
			      f pulse[pulselen, n - nminus] ] ];

		len = Length[interlist];
		signal = MakeOne[ interlist[[1]] ];
		For [ int = 2, int <= len, int++,
		      signal += MakeOne[ interlist[[int]] ] ];

		signal ]

(*  InvDFT  [Myers, 221]  *)
InvDFT/: InvDFT[L_, k_] :=
	InvDFT[L, k, DummyVariables[Length[k], Global`n]]
InvDFT/: InvDFT[L_, k_, ni_, ioptions___][ DFT[L_, n_, k_, options___] [f_] ] :=
	If [ SameQ[ni, n], f, f /. ReplaceWith[n, ni] ] /;
	Length[ni] == Length[n]

InvDFT/: Format[ InvDFT[L_, k_, n_, options___] ] :=
	 Subscripted[ "DFT"[ -1, k ~goesto~ n, SequenceForm["L = ", L] ],
	 	      {2,3}, {1} ]
InvDFT/: Format[ InvDFT, TeXForm ] := "{\\cal DFT}^{-1}"
InvDFT/: Format[ InvDFT[L_, k_, n_, options___], TeXForm ] :=
	 Subscripted[ "{\\cal DFT}"[ -1, k ~goesto~ n,
				     SequenceForm["{\rm L} = ", L] ],
	 	      {2,3}, {1} ]

InvDFT/: GetOperatorVariables[ InvDFT[L_, k_, n_, options___] ] := k

InvDFT/: Extent1D[ InvDFT[L_, k_, n_, options___][f_], n_ ] := {0, L-1}
InvDFT/: Extent1D[ InvDFT[L_, k_, n_, options___][f_], k_ ] := {0, L-1}

InvDFT/: TheFunction[ InvDFT[L_, k_, n_, options____][f_] ] :=
	findIt[ "SignalProcessing`Digital`DFT`",
		"InvDFTransform", f, w, n, options ]

(*  InvDTFT  *)
InvDTFT/: InvDTFT[w_] := InvDTFT[w, DummyVariables[Length[w], Global`n]]
InvDTFT/: InvDTFT[w_, ni_, ioptions___] [ DTFT[n_, w_, options___] [f_] ] :=
	If [ SameQ[ni, n], f, f /. ReplaceWith[n, ni] ] /;
	Length[ni] == Length[n]

InvDTFT/: Format[ InvDTFT[w_, n_, options___] ] :=
	  Subscripted[ "DTFT"[-1, w ~goesto~ n], {2}, {1} ]
InvDTFT/: Format[ InvDTFT, TeXForm ] := "{\\cal DTFT}^{-1}"
InvDTFT/: Format[ InvDTFT[w_, n_, options___], TeXForm ] :=
	  Subscripted[ "{\\cal DTFT}"[-1, w ~goesto~ n], {2}, {1} ]

InvDTFT/: TheFunction[ InvDTFT[w_, n_, options____][f_] ] :=
	findIt[ "SignalProcessing`Digital`DTFT`",
		"InvDTFTransform", f, w, n, options ]


(*  InvFT  *)
InvFT/: InvFT[w_] := InvFT[w, DummyVariables[Length[w], Global`t]]
InvFT/: InvFT[w_, ti_, ioptions___] [ FT[t_, w_, options___] [f_] ] :=
	If [ SameQ[ti, t], f, f /. ReplaceWith[t, ti] ] /;
	Length[ti] == Length[t]

InvFT/: Format[ InvFT[w_, t_, options___] ] :=
	Subscripted[ "F"[-1, w ~goesto~ t], {2}, {1} ]
InvFT/: Format[ InvFT, TeXForm ] := "{\\cal F}^{-1}"
InvFT/: Format[ InvFT[w_, t_, options___], TeXForm ] :=
	Subscripted[ "{\\cal F}"[-1, w ~goesto~ t], {2}, {1} ]

InvFT/: TheFunction[ InvFT[w_, t_, options____][f_] ] :=
	findIt[ "SignalProcessing`Analog`Fourier`",
		"InvCTFTransform", f, w, t, options ]

(*  InvL  *)
InvL/: InvL[s_] := InvL[s, DummyVariables[Length[s], Global`t]]
InvL/: InvL[s_, ti_, ioptions___] [ L[t_, s_, options___] [f_] ] :=
	If [ SameQ[ti, t], f, f /. ReplaceWith[t, ti] ] /;
	Length[t] == Length[ti]

InvL/: Format[ InvL[s_, t_, options___] ] :=
	Subscripted[ "L"[-1, s ~goesto~ t], {2}, {1} ]
InvL/: Format[ InvL, TeXForm ] := "{\\cal L}^{-1}"
InvL/: Format[ InvL[s_, t_, options___], TeXForm ] :=
	Subscripted[ "{\\cal L}"[-1, s ~goesto~ t], {2}, {1} ]

InvL/: TheFunction[ InvL[s_, args__][x_] ] :=
	findIt[ "SignalProcessing`Analog`InvLaPlace`",
		"InvLaPlace", x, s, args ]

(*  InvZ  [Myers, 221]  *)
InvZ/: InvZ[zi_] := InvZ[zi, DummyVariables[Length[zi], Global`n]]
InvZ/: InvZ[z_, ni_, ioptions___] [ Z[n_, z_, options___] [f_] ] :=
	If [ SameQ[ni, n], f, f /. ReplaceWith[n, ni] ] /;
	Length[n] == Length[ni]

InvZ/: Format[ InvZ[z_, n_, options___] ] :=
	Subscripted[ "Z"[-1, z ~goesto~ n], {2}, {1} ]
InvZ/: Format[ InvZ[z_, n_], TeXForm ] := "{\\cal Z}^{-1}"
InvZ/: Format[ InvZ[z_, n_, options___], TeXForm ] :=
	Subscripted[ "{\\cal Z}"[-1, z ~goesto~ n], {2}, {1} ]

InvZ/: TheFunction[ InvZ[z_, args__][x_] ] :=
	findIt[ "SignalProcessing`Digital`InvZTransform`",
		"InvZTransform", x, z, args ]


(*  L  *)
L/: L[t_] := L[t, DummyVariables[Length[t], Global`s]]
L/: L[t_, s_, options___] [ InvL[si_, t_, ioptions___] [f_] ] :=
	If [ SameQ[s, si], f, f /. ReplaceWith[si, s] ] /;
	Length[s] == Length[si]

L/: L^-1[a__] := InvL[a]

L/: Format[ L[t_, s_, options___] ] :=
	Subscripted[ "L"[ t ~goesto~ s ] ]
L/: Format[ L[t_, s_, options___], TeXForm ] :=
	Subscripted[ "{\\cal L}"[ t ~goesto~ s ] ]

L/: TheFunction[ L[t_, args__][x_] ] :=
	findIt[ "SignalProcessing`Analog`LaPlace`",
		"LaPlace", x, t, args ]

(*  LineImpulse  *)
Unprotect[samecoefficient];
Unprotect[mergecoefficient];

SetAttributes[samecoefficient, Listable];
samecoefficient[c1_, c2_] :=
	If [ SameQ[c1,c2],
	     ! SameQ[c1, Null],
	     SameQ[c1, Null] || SameQ[c2, Null] ]

SetAttributes[mergecoefficient, Listable];
mergecoefficient[c1_, c2_] := 
	Which [ SameQ[c1, Null], c2,
		SameQ[c2, Null], c1,
		True,		 c1 ]

Protect[samecoefficient];
Protect[mergecoefficient];

LineImpulseMerge[nlist_, ncoeffs_, klist_, kcoeffs_] :=
	Block [	{commonvars, kscale, nscale, scaledkcoeffs, scaledncoeffs},
		scaledncoeffs = ncoeffs;
		scaledkcoeffs = kcoeffs;
		commonvars = Intersection[nlist, klist];
		If [ ! EmptyQ[commonvars],
		     normvar = First[commonvars];
		     nscale = AssociateItem[normvar, nlist, ncoeffs];
		     scaledncoeffs = ncoeffs / nscale;
		     kscale = AssociateItem[normvar, klist, kcoeffs];
		     scaledkcoeffs = kcoeffs / kscale ];
		allvars = Union[nlist, klist];
		coeffs1 = AssociateItem[allvars, nlist, scaledncoeffs];
		coeffs2 = AssociateItem[allvars, klist, scaledkcoeffs];
		{ Apply[And, samecoefficient[coeffs1, coeffs2]],
		  LineImpulse[allvars, mergecoefficient[coeffs1, coeffs2]] } ]

LineImpulse/: LineImpulse[n_?AtomQ, c_] := Impulse[c n]

LineImpulse/: LineImpulse[nlist_, ncoeffs_] LineImpulse[klist_, kcoeffs_] :=
	LineImpulseMerge[nlist, ncoeffs, klist, kcoeffs] [[2]] /;
	! EmptyQ[Intersection[nlist,klist]] && LineImpulseMerge[nlist, ncoeffs, klist, kcoeffs] [[1]]

LineImpulse/: TheFunction[ LineImpulse[nlist_, ncoeffs_] ] :=
	Impulse[ Apply[Plus, nlist, ncoeffs] ]

LineImpulse/: N [ LineImpulse[nlist_, ncoeffs_] ] :=
	N [ TheFunction[ LineImpulse[nlist, ncoeffs] ] ]

LineImpulse/: Format[ LineImpulse[nlist_List, ncoeffs_List] ] :=
	Format[Impulse[Apply[Plus, nlist ncoeffs]]] /;
	Length[nlist] == Length[ncoeffs]

(*  OperatorInOperExpr  *)
OperatorInOperExpr[ h_[a__][s__] ] := h[a]
OperatorInOperExpr[ h_[s__] ] := h

(*  OperatorName  *)
OperatorName[ h_[a__][s__] ] := h
OperatorName[ h_[a__] ] := h
OperatorName[ x_ ] := x

(*  ParametersInOperExpr  *)
ParametersInOperExpr[ h_[a__][s__] ] := a
ParametersInOperExpr[ h_[s__] ] := Null

(*  Periodic  *)

	(* Degenerate Cases *)

Periodic/: Periodic[ {{m_}}, {n_Symbol} ] := Periodic[m, n]
Periodic/: Periodic[ {m_}, {n_Symbol} ] := Periodic[m, n]

	(* Error checking of periodic parameters  *)

Periodic/: Periodic[m_List, n_List, options___][f_] :=
	MyMessage[Periodic::convert, Periodic[DiagonalMatrix[m], n][f], m] /;
	VectorQ[m] && SameQ[Length[m], Length[n]]

Periodic/: Periodic[m_List, n_List, options___][f_] :=
	MyMessage[Periodic::badmD, f] /;
	VectorQ[m] && ! SameQ[Length[m], Length[n]]

Periodic/: Periodic[m_List, n_List, options___][f_] :=
	MyMessage[Periodic::badmD, f] /;
	MatrixQ[m] && ! Apply[SameQ, Append[Dimensions[m], Length[n]]]

Periodic/: Periodic[m_List, n_Symbol, options___][f_] :=
	MyMessage[Periodic::badmD, f]

	(* Definition  *)

Periodic/: Periodic[p1_, w_Symbol, o1___][x1_] *
	   Periodic[p2_, w_Symbol, o2___][x2_] :=
		Periodic[ p1 p2 / PolynomialGCD[p1, p2], w, o1, o2][ x1 x2 ]

Periodic/: Format[ Periodic[m_, n_List, options___] ] :=
	Block [	{bar},
		bar = TableForm[ Apply[ Table,
					{ "  |  ", Dimensions[n] } ] ];
		Format[ SequenceForm[ Periodic,
				      Subscript[ TableForm[m] ],
				      Subscript[ bar ],
				      Subscript[ TableForm[n] ] ] ] ]
Periodic/: Format[ Periodic[m_, n_Symbol, options___] ] :=
	Subscripted[ Periodic[m,n] ]

Periodic/: Format[ Periodic[k_, n_, options___], TeXForm ] :=
		inTeXformat[Perodic, k, n]

Periodic/: GetOperatorVariables[ Periodic[k_, n_, options___] ] := n

(*  PolyphaseDownsample  *)
PolyphaseDownsample/:
	GetOperatorVariables[ PolyphaseDownsample[m_, h_, n_, ___] ] := n

(*  PolyphaseResample  *)
PolyphaseResample/:
	GetOperatorVariables[ PolyphaseResample[m_, h_, l_, n_, ___] ] := n

(*  PolyphaseUpsample  *)
PolyphaseUpsample/:
	GetOperatorVariables[ PolyphaseUpsample[l_, h_, n_, ___] ] := n

(*  Pulse  *)
Pulse/: Pulse[l_List, t_List] :=			(* multidimensional *)
	Apply[ Times, Map[ Pulse, Transpose[ { l, t } ] ] ] /;
	SameQ[ Length[l], Length[t] ]			(* separability     *)

Pulse/: Pulse[{l_, t_}] := Pulse[l, t]
Pulse[l_] := Pulse[l, Global`n]				(* default args    *)

Pulse[0, n_] := 0 					(* definition      *)
Pulse[0., n_] := 0
Pulse[L_, 0] := 1
Pulse[1, n_] := Impulse[n]
Pulse[Infinity, n_] := Step[n]
Pulse[-Infinity, n_] := Step[-n]
Pulse/: Pulse[l_?Positive, n_] := 1 /; N[0 <= n < l]
Pulse/: Pulse[l_?Negative, n_] := 1 /; N[l < n <= 0]
Pulse/: Pulse[l_?Positive, n_] := 0 /; Negative[n] || N[n >= l]
Pulse/: Pulse[l_?Negative, n_] := 0 /; Positive[n] || N[n <= l]

Pulse/: TheFunction[Pulse[l_, n_]] := Step[n] - Step[n - l]

Pulse/: Extent1D[ Pulse[l_, a_. n_ + b_.], n_ ] :=
		Extent1DOrder[-b/a, (l-1-b)/a] /;
		FreeQ[{a,b,l}, n]

Pulse/: Format[ Pulse[l_, n_] ] :=			(* output form     *)
	Subscripted[ Pulse[l, n], {1} ]
Pulse/: Format[ Pulse, TeXForm ] := "\\sqcap"
Pulse/: Format[ Pulse[l_, n_], TeXForm ] :=
	mathargs[ "Pulse", l, n, "[", "]" ]

Derivative[0,1][Pulse] :=				(* derivative rule *)
	MyMessage[ Pulse::noder, (Sign[#1]*(Impulse[#2] + Impulse[#2 - #1]))& ]

(*  RationalGCD  *)
RationalGCD[ ] := 0
RationalGCD[ x_ ] := rationalGCD[{x}]
RationalGCD[ arg_, rest__ ] := rationalGCD[ {arg, rest} ]

rationalGCD[ numlist_List ] :=
	Block [	{denomgcd, gcdfun, gcdlist, newlist, numergcd},
		newlist = Flatten[numlist];
		gcdlist = Map[ gcdfilter, newlist ];
		gcdfun = If [ TrueQ[$VersionNumber >= 2.0],
			      PolynomialGCD, GCD ];
		numergcd = Apply[gcdfun, Map[Numerator, gcdlist]];
		denomgcd = Apply[gcdfun, Map[Denominator, gcdlist]];
		numergcd / denomgcd ]

(*  Rev  *)
Rev/: Rev[n_, options1___][Rev[n_, options2___][x_]] := x
Rev/: Rev[n_, options1___][Rev[m_, options2___][x_]] :=
	Block [	{remaining},
		remaining = SetExclusion[ToList[n], ToList[m]];
		Which [ EmptyQ[remaining],
			  x,
			SameQ[Length[remaining], 1],
			  Rev[ remaining[[1]] ] [x],
			True,
			  Rev[remaining][x] ] ] /;
	! EmptyQ[Intersection[ToList[n], ToList[m]]]

Rev/: Extent1D[ Rev[n_, options___][f_], n_ ] :=
	Block [	{extent},
		extent = Extent1D[f, n];
		{ -Second[extent], -First[extent] } ]

Rev/: TheFunction[Rev[n_, options___][x_]] :=
	( TheFunction[x] /. ( n ~ReplaceWith~ (-n) ) )
Rev/: N[ Rev[n_, options___][x_] ] := N [ TheFunction[Rev[n, options][x]] ]

Rev/: Format[ Rev[n_, options___] ] := Subscripted[Rev[n]]
Rev/: Format[ Rev[n_, options___], TeXForm ] := Subscripted[Rev[n]]

(*  RewriteSummations  *)
donotrewrite[f_, t_] := FreeQ[f, Summation] && FreeQ[f, Periodic[k_,t] ]

rewriteFindLimits[ x_, t_, tlower_, tupper_ ] :=
	Block [	{ext, tl, tu},
		ext = Extent1D[x, t];
		tl = mymax[ext[[1]], tlower];
		tu = mymin[ext[[2]], tupper];
		{tl, tu} ]

RewriteSummations[ f_, t_, tl_, tu_ ] := f /; donotrewrite[f, t]

RewriteSummations[ f_, t_, tlower_, tupper_ ] := 
	Block [	{ext, ii, tl, tu},
		f /.

		  { Periodic[k_, v_][g_] :> g /;
		    SameQ[v, t] && FreeQ[g, t] } /.

		    { x_ Periodic[k_, v_][g_] :>	   (* use v_ not t *)
			Block [ {},
			  {tl, tu} = rewriteFindLimits[x, t, tlower, tupper];
			  x rewriteonesummation[ Simplify[g /. (t -> t + ii k)],
						 True, t, tl, tu, ii,
						 -Infinity, Infinity, 1 ] ] /;
			SameQ[v, t] && donotrewrite[g, t],
		      Periodic[k_, v_][g_] :>	 	   (* use v_ not t *)
			Block [ {},
			  rewriteonesummation[ Simplify[g /. (t -> t + ii k)],
					       True, t, tlower, tupper, ii,
					       -Infinity, Infinity, 1 ] ] /;
			SameQ[v, t] && donotrewrite[g, t],
		      x_ Summation[i_, il_, iu_, inc_][g_] :>
			Block [ {},
			  {tl, tu} = rewriteFindLimits[x, t, tlower, tupper];
			  x rewriteonesummation[ Simplify[g], False, t, tl, tu,
						 i, il, iu, inc ] ] /;
			donotrewrite[g, t],
		      Summation[i_, il_, iu_, inc_][g_] :>
			Block [ {},
			  rewriteonesummation[ Simplify[g], False,
					       t, tlower, tupper,
					       i, il, iu, inc ] ] /;
			donotrewrite[g, t] } ]

rewriteonesummation[ a_ + b_, flag_, t_, tl_, tu_, i_, il_, iu_, inc_ ] :=
	rewriteonesummation[a, flag, t, tl, tu, i, il, iu, inc] +
	rewriteonesummation[b, flag, t, tl, tu, i, il, iu, inc]

rewriteonesummation[ g_, periodicflag_, t_, tl_, tu_, i_, il_, iu_, inc_ ] :=
	Block [ {ext, il0, ilact, intflag, iu0, iuact,
		 lext, lsol, retval, solutions, uext, usol, valid},
		intflag = IntegerQ[inc] &&	(* index var i is an integer *)
			  ( IntegerQ[il] || SameQ[il, -Infinity] ) &&
			  ( IntegerQ[iu] || SameQ[iu, Infinity] );
		ext = Simplify[ Extent1D[g, t] ] // minmaxSimplify;
		valid = ListQ[ext] && SameQ[Length[ext], 2];
		If [ ! valid, ext = { -Infinity, Infinity } ];

		{lext, uext} = ext;
		If [ FreeQ[lext, i] && ! FreeQ[uext, i], lext = uext ];
		If [ FreeQ[uext, i] && ! FreeQ[lext, i], uext = lext ];

		If [ FreeQ[lext, i] && FreeQ[uext, i],
		     Message[ Summation::extent ];
		       retval = If [ periodicflag,
				     g /. i -> 0,
				     Summation[i, il, iu, inc][g] ],

		     lsol = Solve[ lext == tu, i ];
		     usol = Solve[ uext == tl, i ];
		     Which [ SameQ[Head[lsol], Solve] ||
			     SameQ[Head[usol], Solve],
			       Message[ Summation::extent ];
		     		retval = If [ periodicflag,
					      g /. i -> 0,
					      Summation[i, il, iu, inc][g] ],

			     intflag,
			       solutions = N[ Join[ i /. lsol, i /. usol ] ];
			       il0 = Min[solutions];
			       iu0 = Max[solutions];
			       ilact = Max[il, Ceiling[Min[il0, iu0]]];
			       iuact = Min[iu, Max[il0, iu0]];
			       retval = Sum[ g, {i, ilact, iuact, inc} ],

			     True,
			       solutions = Join[ i /. lsol, i /. usol ];
			       il0 = Apply[Min, solutions] // minmaxSimplify;
			       iu0 = Apply[Max, solutions] // minmaxSimplify;
			       ilact = Max[il, Min[il0, iu0]] // minmaxSimplify;
			       iuact = Min[iu, Max[il0, iu0]] // minmaxSimplify;
			       If [ InfinityQ[il],
				    ilact = ilact - Mod[ilact, inc];
				      iuact = iuact - Mod[iuact, inc],
				    ilact = ilact - Mod[ilact - il, inc];
				      iuact = iuact - Mod[iuact - il, inc] ];
			       retval = Sum[ g, {i, ilact, iuact, inc} ] ] ];
	retval ]

(*  RotateAxes (JMW) *)
RotateAxes/: RotateAxes[ 0, x_, options___ ][f_] := f
RotateAxes/: RotateAxes[ a_Symbol, x_Symbol, options___ ][f_] := f
RotateAxes/: TheFunction[ RotateAxes[ a_, {x_, y_}, options___ ][ f_ ] ] :=
	TheFunction[ f ] /.
		{ x -> (x Cos[a] - y Sin[a]), y -> (x Sin[a] + y Cos[a]) }
RotateAxes/: N[ RotateAxes[ a_, axes_List, options___ ][ f_ ] ] :=
	N[ TheFunction[ RotateAxes[ a, axes ][ f ] ] ]
RotateAxes/: Format[ RotateAxes[ a_, axes_, options___ ] ] :=
	Subscripted[RotateAxes[ a, axes ]]
RotateAxes/: Format[ RotateAxes[ a_, axes_, options___ ], TeXForm ] :=
	inTeXformat[RotateAxes, a, axes]
RotateAxes/: GetOperatorVariables[ RotateAxes[ a_, v_, ___ ][ f_ ] ] := v

(*  ScaleAxis [Myers, 220]  *)
ScaleAxis/: TheFunction[ScaleAxis[l_, w_Symbol, options___][x_]] :=
		TheFunction[x] /. w -> l w
ScaleAxis/: TheFunction[ScaleAxis[{l__}, {w__}, options___][x_]] :=
		( TheFunction[x] /. ( {w} ~ReplaceWith~ ({l} * {w}) ) )

ScaleAxis/: N[ ScaleAxis[l_, w_, options___][x_] ] :=
		N[ TheFunction[ ScaleAxis[l,w][x] ] ]

ScaleAxis/: Format[ ScaleAxis[l_, w_, options___] ] :=
		informat[ScaleAxis, l, w]
ScaleAxis/: Format[ ScaleAxis[l_, w_, options___], TeXForm ] :=
		inTeXformat[ScaleAxis, l, w]

ScaleAxis/: GetOperatorVariables[ ScaleAxis[l_, w_, ___] ] := w

(*  ScalingFactor --  must be independent of variable z  *)
ScalingFactor[f_, z_] :=
	breakup[ Apply[ RationalGCD, GetAllFactors[f, z] ], z ]

(*  SequencePlot  *)
Options[ SequencePlot ] :=
	{ Axes -> axesdefaultvalue,
	  DisplayFunction :> $DisplayFunction,
	  PlotLabel -> "Sequence Plot",
	  PlotRange -> All }

baseticks = { 1, 2, 2, 5, 5, 5, 5, 5, 10, 10 };
getticks[nstart_, nend_] :=
	Block [	{factor, i, ninc, ns, nterm},
		ninc = Round[(nend - nstart) / 5];
		If [ ninc == 0, ninc = 1 ];
		If [ ninc > 2,
		     factor = 10;
		     While [ ninc > factor, factor *= 10 ];
		     factor /= 10;
		     nterm = intdivide[ninc, factor];
		     ninc = factor * baseticks[[nterm]] ];
		ns = Sign[nstart] ninc intdivide[Abs[nstart], ninc];
		Table[i, {i, ns, nend, ninc}] ]

dgetvalue[f_, n_Symbol, n0_Integer] := GetValue[f, n, n0]
dgetvalue[f_, n_Symbol, n0_Real] := GetValue[f, n, Round[n0]]
dgetvalue[f_, {n1_Symbol, n2_Symbol}, {n01_?NumberQ, n02_?NumberQ}] :=
	GetValue[f, {n1, n2}, {Round[n01], Round[n02]}]

SequencePlot[f_List, {n1_Symbol, st1_, end1_},
		     {n2_Symbol, st2_, end2_}, op___] :=
	MyMessage[ SequencePlot::nolist, NullPlot ]

SequencePlot[f_, {n1_Symbol, st1_, end1_}, {n2_Symbol, st2_, end2_}, op___] :=
	Block [	{funct, ftemp, funs, n1length, n2length, n1str, n2str,
		 n1temp, n2temp, nstart, nend, oplist, plot},

		nstart = Floor[N[{st1, st2}]];
		nend   = Ceiling[N[{end1, end2}]];
		n1length = nend[[1]] - nstart[[1]];
		n2length = nend[[2]] - nstart[[2]];
		n1str = StripPackage[n1];
		n2str = StripPackage[n2];
		oplist = Join [	ToList[op],
				Options[SequencePlot],
				{ PlotPoints -> {n1length + 1, n2length + 1},
				  AxesLabel -> {n1str, n2str, " "},
				  Ticks -> Automatic } ];

		funct = ToDiscrete[f];
		ftemp = N[ TheFunction[ funct ] ];

		(*  Generate three-dimensional coordinates for plot  *)
		Off[Power::infy, Infinity::indt];
		plot = Apply[ Plot3D,
			      Join[ { dgetvalue[ ftemp, {n1, n2},
						 {n1temp, n2temp} ],
				      { n1temp, nstart[[1]], nend[[1]] },
				      { n2temp, nstart[[2]], nend[[2]] } },
				    oplist ] ];
		On[Power::infy, Infinity::indt];

		plot ]


SequencePlot[f_, {n_Symbol, start_, end_}, op___] :=
	Block [	{factor, flength, ftemp, funct, funs, llength, length,
		 linethickness, nend, nstart, nstr, ntemp, nticks,
		 oplist, plot, pointthickness, showoptions},

		nstart = Floor[N[start]];
		nend   = Ceiling[N[end]];

		(*  Set up the options  *)
		nticks = getticks[nstart, nend];
		nstr = StripPackage[n];
		oplist = Join [	Flatten[ToList[op]],
				Options[SequencePlot],
				{ AxesLabel -> { nstr, " " },
				  Ticks -> { nticks, Automatic } } ];

		factor = N[ Log[10, nend - nstart + 3 ] ];
		pointthickness = 0.024 / factor;
		If [ SameQ[PlotStyle, Replace[PlotStyle, oplist]],
		     linethickness = 0.008 / factor,
		     linethickness = Replace[PlotStyle, oplist] ];

		(*  Extract the functional form of the expression  *)
		funct = ToDiscrete[f];
		ftemp = N[ TheFunction[ funct ] ];

		(*  Handle any infinite summations  *)
		ftemp = RewriteSummations[ftemp, n, nstart, nend];

		If [ ListQ[ftemp] && ListQ[linethickness],
		     flength = Length[ftemp];
		     llength = Length[linethickness];
		     If [ llength > flength,
			  linethickness = Take[linethickness, flength] ];
		     If [ llength < flength,
			  linethickness =
			      Table[ linethickess, Round[flength/llength] ];
			  linethickness =
			      Take[ Flatten[linethickness], flength ] ] ];

		(*  Generate form for 1-D function  *)
		SetAttributes[seqFun1D, Listable];    (* avoid fun pointers *)
		seqFun1D[g_, lstyle_] :=
			DiscreteGraphics [
				Table [ {ntemp, dgetvalue[g, n, ntemp]},
					{ntemp, nstart, nend}],
				lstyle,
				pointthickness ];

		(*  Plot "time"-domain	*)
		Off[Power::infy, Infinity::indt];
		showoptions = RemoveOptions[oplist, specialPlotOptions];
		plot = Show [ seqFun1D[ftemp, linethickness], showoptions ];
		On[Power::infy, Infinity::indt];

		plot ]

(*  SequenceToFunction  *)
SequenceToFunction[seq_, n_:Global`n, noffset_:0] :=
	Block [ {},
		(* Variable no. of arguments -- could not use Function *)
		impulsegen[l__] := seq[[l]] Apply[Impulse,
						  n - {l} - noffset + 1];
		Apply[ Plus, Flatten [ Array[impulsegen, Dimensions[seq]] ] ] ]

(*  SignalCleanup  *)
SignalCleanup[funct_, t_Symbol, nstart_, nend_] :=
	Block [	{deltafuns, ftemp},

		ftemp = SPSimplify[ TheFunction[funct], Variables -> t ];

		(*  Separate delta functions from rest of funct     *)
		(*  DeltaPlot handles Summation operators by itself *)
		deltafuns = GetDeltaFunctions[ftemp, t];
		ftemp = ftemp //.
			{ h_[ c_. Delta[a_. t + b_.] ] :> 0,
			  h_[ c_. Delta[a_. t + b_.] + x_ ] :> h[x],
			  Delta[a_. t + b_.] :> 0 };

		(*  Rewrite infinite summations and periodic operators  *)
		deltafuns = RewriteSummations[deltafuns, t, nstart, nend];
		ftemp = RewriteSummations[ftemp, t, nstart, nend];

		{ N[deltafuns], N[ftemp] } ]


SignalCleanup[funct_, {t1_Symbol, t2_Symbol}, nstart_, nend_] :=
	Block [	{deltafuns, ftemp},

		ftemp = SPSimplify[ TheFunction[funct],
				    Variables -> {t1, t2} ];	(* JMW *)

		(*  Separate delta functions from rest of funct  *)
		deltafuns = GetDeltaFunctions[ftemp, {t1, t2}];
		ftemp = ftemp //.
			{ h_[ c_. Delta[a_. t1 + b_.] ] :> 0,
			  h_[ c_. Delta[a_. t1 + b_.] + x_ ] :> h[x],
			  Delta[a_. t1 + b_.] :> 0,
			  h_[ c_. Delta[a_. t2 + b_.] ] :> 0,
			  h_[ c_. Delta[a_. t2 + b_.] + x_ ] :> h[x],
			  Delta[a_. t2 + b_.] :> 0 };

		{ N[deltafuns], N[ftemp] } ]

(*  SignalPlot  *)
specialPlotOptions =
	Complement[ Sort[ Map[First, Options[Plot]] ],
		    Sort[ Map[First, Options[Graphics]] ] ]

sigPlot1D[args__][g_] := SignalPlot[g, args]

Options[ SignalPlot ] :=
	{ Axes -> axesdefaultvalue,
	  DisplayFunction :> $DisplayFunction,
	  PlotLabel -> "Signal Plot",
	  PlotRange -> All,
	  Ticks -> Automatic }

SignalPlot[ f_List, {t1_Symbol, start1_, end1_},
		    {t2_Symbol, start2_, end2_}, op___ ] :=
	MyMessage[ SignalPlot::nolist, NullPlot ]

SignalPlot[ f_,	{t1_Symbol, start1_, end1_},
		{t2_Symbol, start2_, end2_}, op___ ] :=
	Block [	{deltafuns, freqresp, ftemp, funct, oplist,
		 t1str, t2str, t1temp, t2temp},

		(*  Set up for plotting  *)
		t1str = StripPackage[t1];
		t2str = StripPackage[t2];
		oplist = Join[ ToList[op],
			       { AxesLabel -> { t1str, t2str, " " } },
			       Options[ SignalPlot ] ];

		(*  Make function continuous and replace non-sep delta's *)
		funct = ToContinuous[f] /.
		     Delta[a_. t1 + b_. t2 + c_.] :> Impulse[a t1 + b t2 + c];

		(*  Separate delta functions from rest of funct and      *)
		(* convert the non-delta expressions into plottable form *)
		{ deltafuns, ftemp } =
			SignalCleanup[ funct, {t1,t2},
				       N[{start1, start2}], N[{end1, end2}] ];

		(*  Plot delta functions separately from rest of function  *)

		Plot3D[ GetValue[ftemp, {t1, t2}, {t1temp, t2temp}],
			{ t1temp, start1, end1 },
			{ t2temp, start2, end2 },
			Release[oplist] ] ]

SignalPlot[f_List, {t_Symbol, start_, end_}, op___] :=
	Block [	{fun, oplist, showoptions, theplots},
		fun = sigPlot1D[{t,start,end}, DisplayFunction->Identity, op];
		theplots = Map[ fun, f ];
		oplist = Join[ToList[op], Options[SignalPlot]];
		showoptions = RemoveOptions[oplist, specialPlotOptions];
		Show[ theplots, ToCollection[showoptions] ] ]

SignalPlot[f_, {t_Symbol, start_, end_}, op___] :=
	Block [	{deltafuns, ftemp, i, imstyle,
		 nend = N[end], nstart = N[start], oplist,
		 plot, showoptions, restyle, tstr, ttemp },

		(*  Set up for plotting  *)

		tstr = StripPackage[t];
		restyle = Thickness[0.006];
		imstyle = Dashing[{0.05,0.05}]; 
		oplist = Join[ ToList[op],
			       { AxesLabel -> { tstr, " " },
				 DisplayFunction -> Identity,
				 PlotStyle -> { restyle, imstyle } },
			       Options[ SignalPlot ] ];

		(*  Separate delta functions from rest of funct and  *)
		(*  convert function into plottable functional form  *)

		{ deltafuns, ftemp } =
			SignalCleanup[ ToContinuous[f], t, nstart, end ];

		(*  Plot delta functions separately from ftemp  *)

		Off[Power::infy, Infinity::indt];

		(*  1.  Generate the graphics for the entire signal  *)

		If [ ZeroQ[ftemp],
		     plot = If [ ! AtomQ[deltafuns],   (* delta funs found *)
				 DeltaPlot[deltafuns, t, nstart, nend],
				 NullPlot ];
		     plot = { Plot[ 0,
				    { ttemp, nstart, nend},
				    Release[oplist] ],
			      plot },

		     plot = Plot[ { Re[GetValue[ftemp, t, ttemp]],
				    Im[GetValue[ftemp, t, ttemp]] },
				  { ttemp, nstart, nend },
				  Release[oplist] ];
		     If [ ! AtomQ[deltafuns],      (* delta funs found *)
			  plot = DeltaPlot[deltafuns, t, nstart, nend, plot]] ];

		(*  2.  Plot the entire signal  *)

		oplist = Join[ ToList[op], Options[SignalPlot] ];
		showoptions = RemoveOptions[oplist, specialPlotOptions];

		plot = Show[ plot, ToCollection[showoptions] ];

		On[Power::infy, Infinity::indt];

		plot ]


(*  SignalsInOperExpr  *)
SignalsInOperExpr[ h_[a__][s__] ] := s
SignalsInOperExpr[ h_[s__] ] := s

SignalsInOperExpr[ h_[a__][s__], h_ ] := s
SignalsInOperExpr[ h_[a__][s__], op_ ] := h[a][s]	/; ! SameQ[h, op]

SignalsInOperExpr[ h_[s__], h_ ] := s
SignalsInOperExpr[ h_[s__], op_ ] := h[s]		/; ! SameQ[h, op]

(*  Shift [Myers, 217]  *)
Shift/: Shift[m_, n_List, options___] :=
	Shift[ Table[m, Length[n]], n ] /; ! ListQ[m]

Shift/: Shift[m_List, n_List, options___][x_] :=
	MyMessage[ Shift::badmD, x ] /;
	Length[m] != Length[n]

Shift/: Shift[0, n_, options___][x_] := x

Shift/: Extent1D[ Shift[m_, n_, options___][f_], n_ ] := Extent1D[f, n] + m

Shift/:	TheFunction[Shift[k_, n_, options___][x_]] :=
	( TheFunction[x] /. ( n ~ReplaceWith~ (n - k) ) )
Shift/: N[ Shift[k_,n_][x_] ] := N [ TheFunction[ Shift[k,n][x] ] ]

Shift/: Format[ Shift[m_, n_, options___] ] := informat[Shift, m, n]
Shift/: Format[ Shift[m_, n_, options___], TeXForm ] := inTeXformat[Shift, m, n]

Shift/: GetOperatorVariables[ Shift[l_, n_, options___] ] := n


(*  Sinc  *)
Sinc/: Sinc[-Infinity] := 0
Sinc/: Sinc[0] := 1
Sinc/: Sinc[0.] := 1.
Sinc/: Sinc[Infinity] := 0

Sinc/: Sinc[x_?NumberQ] := Sin[x] / x

Sinc/: Sinc[x_, y__] := Sinc[x] Sinc[y]

Sinc/: Format[Sinc, TeXForm] := "{\\rm sinc}"

Sinc/: Derivative[1][Sinc] := (( Cos[#1] / #1 ) - ( Sinc[#1] / #1 ))&


(*  SPSimplify  *)
Options[SPSimplify] :=
	Join[{ Dialogue -> False, Variables -> {} }, Options[Simplify]]

kludgeRules =   (* prevents infinite loop in Mathematica 2.0 and lower *)
      {	a1_. x_. Exp[Complex[0, i1_] th1_.] +
	a2_. x_. Exp[Complex[0, i2_] th2_.] :>
		2 x Abs[a1] Cos[Arg[a1] + i1 th1] /;
		( SameQ[a1, Conjugate[a2]] && SameQ[i1 th1, -i2 th2] ),
	a1_. x_. Exp[Complex[0, i1_] th1_.] +
	a2_. x_. Exp[Complex[0, i2_] th2_.] :>
		2 I x a1 Sin[i1 th1] /;
		( SameQ[a1, -a2] && SameQ[i1 th1, -i2 th2] ) }

Clear[delta];
Protect[delta];
preprocessRules = {};
postprocessRules = {};
If [ 2.2 <= $VersionNumber < 2.3,
     preprocessRules = ( DiracDelta -> delta );
     postprocessRules = ( delta -> DiracDelta ) ]

SPSimplify[ x_, op___ ] :=
	Block [	{dialflag, different, firstTimeFlag, laste, newe, oplist,
		 trig, varrules = {}, varflag, variables},
		oplist = ToList[op] ~Join~ Options[SPSimplify];
		trig = Replace[Trig, oplist];
		dialflag = InformUserQ[oplist];
		variables = ToList[ Replace[Variables, oplist] ];
		varflag = ! EmptyQ[variables];
		If [ varflag, varrules = makesimplifyrules[variables] ];

		(* Let our simplification rules take the first crack *)
		newe = x /. SPSimplificationRules;
		If [ varflag, newe = newe /. varrules ];
		different = ! SameQ[newe, x];
		If [ dialflag && different,
		     Print[x]; Print["which simplifies to"]; Print[newe] ];

		(* Run simplification loop until expression stops changing *)
		different = True;
		While [ different,
			laste = newe;
			If [ ! EmptyQ[ kludgeRules ],
			     newe = newe //. kludgeRules ];
                        If [ ! EmptyQ[ preprocessRules ],
			     newe = newe /. preprocessRules ];
			newe = If [ TrueQ[ $VersionNumber >= 2.0 ],
				    Simplify[newe, Trig -> trig],
				    Simplify[newe] ];
			If [ ! EmptyQ[ postprocessRules ],
			     newe = newe /. postprocessRules ];
			newe = newe /. SPSimplificationRules;
			If [ varflag, newe = newe /. varrules ];
			different = ! SameQ[laste, newe];
			If [ dialflag && different,
			     Print["which simplifies to"]; Print[newe] ] ];

		newe ]

makesimplifyrules[{x_}] := makesimplifyrules[x, {}]

makesimplifyrules[varlist_List] :=
Block [ {},
	Append[ Apply[ Join,
		       Map[ makesimplifyrules[#,
					      Complement[varlist, ToList[#]]]&,
			    varlist ] ],
		(x1_)^(e_) (x2_)^(e_) :> (x1 x2)^e /;
		  FreeQ[x1, Complex[a_, b_]] && FreeQ[x2, Complex[c_, d_]] &&
		  SubsetQ[Union[GetVariables[x1], GetVariables[x2]], varlist] ] ]

makesimplifyrules[t_, restvarlist_] :=
Block [	{simpRules},

	simpRules = {
	  (* Abs[a] adjusts area under Delta function. First, Delta is an   *)
	  (* even function so Delta[-a t] = Delta[a t] for a > 0. Next,     *)
	  (* define Delta at t=0 as a triangle from -a T to a T with height *)
	  (* 1/T; take the limit of the area of the triangle as T goes to 0 *)
	  Delta[a_ t + b_.] :>
	    Delta[t + b/a] / Abs[a] /;
	    FreeQ[a, t] && MyFreeQ[b, restvarlist],
	  f_ Delta[ t + a_. ] :>
	    Limit[f, t -> -a] Delta[ t + a ] /;
	    FreeQ[ a, t ] && (! FreeQ[ f, t ]),		(* JMW *)
	  d_. (y_. + amp1_. Delta[t + t0_.]) :>
	    d y + Limit[TheFunction[d amp1], t -> -t0] Delta[t + t0] /;
	    FreeQ[t0, t] && (! FreeQ[d amp1, t]) &&
	    MyFreeQ[t0, restvarlist] &&
	      ! InfinityQ[ Limit[TheFunction[d amp1], t -> -t0] ],
	  c_. Sign[b_. t] + c_ :> 2 c CStep[b t] /;
	    FreeQ[b, t],
	  ( x_. + y_. CStep[a1_. t + b1_.] ) CStep[a2_. t + b2_.] :> 
	    Block [ {ext, tl, tu},
		    ext = Extent1D[CStep[a1 t + b1] CStep[a2 t + b2], t];
		    If [ SameQ[ext, None],
			 x CStep[a2 t + b2],
			 {tl, tu} = ext;
			 y CPulse[ tu - tl, t - tl ] + x CStep[a2 t + b2] ] ],
	  CStep[a1_. t + b1_.] - CStep[a2_. t + b2_.] :>
	    Sign[a1] Sign[b1/a1 - b2/a2] *
	      CPulse[ Abs[b1/a1 - b2/a2], t - Min[-b1/a1, -b2/a2] ] /;
	    N[ Sign[a1] == Sign[a2] ],
	  CPulse[ L_, c_. ( a_. t + b_. ) ] :>			(* JMW *)
	    CPulse[ L/( a c ), t + b/a ] /;
	    FreeQ[a, t] && ! SameQ[ a c, 1],
(*
    this rule rewrites pulses of negative length to have positive length
    it should be placed after the rule above that rewrites affine pulses

	  CPulse[a_ b_., t + t1_.] :>
	    CPulse[-a b, t - a b + t1 ] /;
	    Negative[a] && FreeQ[ b, t ],
 *)
	  operator_[ params__ ][ a_ x_ ] :>			(* JMW *)
	    a operator[ params ][ x ] /;
	    MemberQ[ { Periodic, ScaleAxis, Shift, RotateAxes }, operator ] &&
	    MyFreeQ[ a, GetOperatorVariables[ operator[params] ] ],
	  Periodic[ T_, t ][ a_ x_ ] :>				(* JMW *)
	    a Periodic[ T, t ][ x ] /; FreeQ[ a, t ],

	  (* Simplify trig functions to Sinc functions when possible *)
	  Sin[a_. t]^m_. t^n_. :>				(* JMW *)
	    (a Sinc[ a t ])^(-n) Sin[a t]^(m+n) /;
	    FreeQ[ a, t ] && (Abs[m] >= Abs[n]) && (Sign[m] != Sign[n]),
	  Sin[a_. t]^m_. t^n_. :>				(* JMW *)
	    (a Sinc[ a t ])^m t^(m+n) /;
	    FreeQ[ a, t ] && (Abs[m] < Abs[n]) && (Sign[m] != Sign[n]),
	  Csc[a_. t]^m_. t^n_. :>				(* JMW *)
	    (a Sinc[ a t ])^(-n) Sin[a t]^(-m+n) /;
	    FreeQ[ a, t ] && (Abs[m] >= Abs[n]) && (Sign[m] == Sign[n]),
	  Csc[a_. t]^m_. t^n_. :>				(* JMW *)
	    (a Sinc[ a t ])^(-m) t^(-m+n) /;
	    FreeQ[ a, t ] && (Abs[m] < Abs[n]) && (Sign[m] == Sign[n]),
	  Sinc[a_ b_ t + t0_.] :>
	    Sinc[-a b t - t0] /;
	    (a < 0),						(* JMW *)
	  t^m_. Sin[a_. t]^n_. :>
	    a^n (Sinc[a t]^n) /;
	    FreeQ[a, t] && IntegerQ[n] && (n > 0) && (m == -n),

	  Sinc[a_ b_. t + t0_.] :>
	    Sinc[-a b t - t0] /;
	    Negative[a] && FreeQ[ b, t ],			(* JMW *)
	  t Cos[b_. t] Sinc[b_. t] :> Sin[2 b t] / (2 b),
	  t^m_. Sinc[t^n_. c_.] :> ( t^(m-n) Sin[t^n c] / c ) /;
	    ( IntegerQ[m] && IntegerQ[n] && ( m > n > 0 ) ) || ( m == n ),
	  Delta[-a_. t + t0_] :> Delta[a t - t0],
	  Impulse[-a_. t + t0_] :> Impulse[a t - t0],
	  Exp[Complex[0,j1_] t a1_.] * (c_ + d_. Exp[Complex[0,j2_] t a2_.]) :>
	    2 d I Sin[-j1 a1 t] /;
	    ( c == -d ) && ( -2 j1 a1 == j2 a2 ),
	  Exp[Complex[0,j1_] t a1_.] * (c_ + d_. Exp[Complex[0,j2_] t a2_.]) :>
	    2 d Cos[-j1 a1 t] /;
	    ( c == d ) && ( -2 j1 a1 == j2 a2 ),
	  (a_. Exp[Complex[0,j1_] t a1_.] + b_. Exp[Complex[0,j2_] t a2_.]) :>
	    2 a Cos[-j1 a1 t] /;
	    (j1 a1 == -j2 a2) && (a == b),			(* JMW *)
	  (a_. Exp[Complex[0,j1_] t a1_.] + b_. Exp[Complex[0,j2_] t a2_.]) :>
	    2 I b Sin[-j1 a1 t] /;
	    (j1 a1 == -j2 a2) && (a == -b),			(* JMW *)

	  CPulse[L_, t + t0_.] CStep[t + t1_.] :>
		pulseStepSimplify[ t, L, n0, n1, CPulse, CStep, 0 ],
	  Pulse[L_, t + n0_.] Step[t + n1_.] :>
		pulseStepSimplify[ t, L, n0, n1, Pulse, Step, 1 ],

	  Conjugate[a_. t] :> a t /; Negative[a] || Positive[a]
	};

	If [ TrueQ[ $VersionNumber >= 2.0 ],
	     simpRules = simpRules ~Join~
		{ Sqrt[a_. t^n_?EvenQ] :> t^(n/2) Sqrt[a],
		  c_. Exp[a_. t] + c_. Exp[b_. t] :> 2 c Cosh[a t] /;
			(a == -b) && (IntegerQ[Sign[a]] || NumberQ[N[a]]),
		  c_. Exp[a_. t] + d_. Exp[b_. t] :> 2 c Sinh[a t] /;
			(c == -d) && (a == -b) && 
			(IntegerQ[Sign[a]] || NumberQ[N[a]]) &&
			(Sign[a] == Sign[c]) && (Sign[b] == Sign[d]) } ];

	simpRules ]

pulseStepSimplify[ t_, L_, n0_, n1_, pulse_, step_, adj_ ] :=
	Block [	{pleft, pright},
		If [ TrueQ[ Negative[L] ],
		     pleft = L + adj - n0;
		     pright = -n0,
		     Assuming[ Positive[L] ];
		     pleft = -n0;
		     pright = L - adj - n0 ];
		step[pright + n1] *
		  pulse[pright - Max[pleft, -n1] + adj, t - Max[pleft, -n1]] ]

(*  Step  *)
Step/: Step[t_?Positive] := 1
Step/: Step[t_?Negative] := 0
Step/: Step[0] := 1
Step/: Step[0.] := 1

Step/: Extent1D[ Step[a_. n_ + b_.], n_ ] :=
		Extent1DOrder[-b/a, Sign[a] Infinity] /;
		FreeQ[{a,b}, n]

Step/: Step[t_, args__] := Step[t] Step[args]		(* multidimensional *)
Step/: Step[List[args__]] := Step[args]			(* separability	    *)

Step/: Step[t_]^a_ := Step[t]				(* automatic	    *)
Step/: Step[n_Symbol + m_.] Step[n_Symbol  + k_.] :=	(* simplification   *)
	Step[n + Min[m,k]] /;				(* rules	    *)
	NumberQ[m] && NumberQ[k]
Step/: Step[n_Symbol + m_.] Step[-n_Symbol + k_.] :=
	Pulse[k + m + 1, n + m] /;
	N[ -m <= k ]
Step/: Step[n_Symbol + m_.] Step[-n_Symbol + k_.] := 0 /;
	N[ -m > k ]

Step/: Format[ Step, TeXForm ] :=			(* output form      *)
	SequenceForm[ "u[", x, "]" ]

Derivative[1][Step] :=					(* derivative rule  *)
	MyMessage[ Step::noder, Impulse ]

(*  Summation  *)
Summation/: Summation[i_Symbol, n_] := Summation[i, 1, n, 1]
Summation/: Summation[i_Symbol, istart_, iend_] :=
	Summation[i, istart, iend, 1]
Summation/: Summation[i_Symbol, istart_, iend_, inc_, options___][0] := 0

Summation/:
    TheFunction[Summation[i_Symbol, istart_, iend_, inc_, options___][f_]] :=
	Block [	{tempsym},
		If [ NumberQ[N[istart]] && NumberQ[N[iend]] && NumberQ[N[inc]],
		     Sum[ TheFunction[f /. i -> tempsym],
			  {tempsym, istart, iend, inc} ],
		     Summation[i, istart, iend, inc][ TheFunction[f] ] ] ]

Summation/:
    Extent1D[ Summation[i_, istart_, iend_, inc_, options___][f_], n_ ] :=
	Block [	{extent},
		extent = Extent1D[f, n];
		Extent1DOrder[ extent /. i -> istart,
			       extent /. i -> iend ] ] /;
	FreeQ[{i,istart,iend,inc}, n]

Summation/:
    N[ Summation[i_Symbol, istart_?NumberQ, iend_?NumberQ, inc_?NumberQ, options___][f_]] :=
	Sum[ N[f], {i, istart, iend, inc} ]

Summation/:
	Format[ Summation[i_, istart_, iend_, inc_, options___] ] :=
	sumformat[ "Summation", i, istart, iend, inc, OutputForm ]

Summation/:
	Format[ Summation[i_, istart_, iend_, inc_, options___], TeXForm ] :=
	sumformat[ "\\sum\\limits", i, istart, iend, inc, TeXForm ]


	(* The following rules were suggested by John Mitchell *)

getdvars[ x_Symbol ] := x
getdvars[ {x_Symbol, k_} ] := x
Unprotect[D]
D/: D[expr_. Summation[i_, istart_, iend_, inc_][f_], x__] :=
	Summation[i, istart, iend, inc][ D[expr f, x] ] /;
	MyFreeQ[ i, Map[getdvars, {x}] ]
Protect[D]

SPIntegrationRules =
Join[ SPIntegrationRules,
      { integrate[ expr_. Summation[i_, istart_, iend_, inc_][f_], x__] :>
	   Summation[i, istart, iend, inc][ integrate[expr f, x] ] /;
	 MyFreeQ[ i, Map[getdvars, {x}] ]
      } ]

(*  TheFunction  *)
TheFunction[x_?AtomQ] := x
TheFunction[Hold[x__]] := Hold[x]
TheFunction[Sign[x_]] := CStep[x] - CStep[-x]	(* Step or CStep will work  *)
TheFunction[h_[a_]] := h [ TheFunction[a] ]
TheFunction[h_[a1_, a__]] := Apply[ h, Map[TheFunction, { a1, a }] ]

(*  ToContinuous  *)
ToContinuous[expr_] := ReplaceRepeated[expr, ToContinuousRules]

ToContinuousRules =
     {
	Impulse[n_]   :> Delta[n],
	Step[n_]      :> CStep[n],
	Pulse[l_, n_] :> CPulse[l, n],
	Convolve[n_]  :> CConvolve[n]
     }

(*  ToDiscrete  *)
ToDiscrete[expr_] := ReplaceRepeated[expr, ToDiscreteRules]

ToDiscreteRules =
     {
	CStep[t_]      :> Step[t],
	Delta[t_]      :> Impulse[t],
	CPulse[l_, t_] :> Pulse[l, t],
	CConvolve[t_]  :> Convolve[t]
     }

(*  Unit *)
Unit/: Unit[0][t_] := Delta[t]				(* definition       *)
Unit/: Unit[-1][t_] := CStep[t]
Unit/: Unit[n_][t_] := t^-(n + 1) / Factorial[-(n + 1)] CStep[t] /; N[n < 0]

Unit/: Extent1D[ Unit[n_Integer][a_. t_ + b_.], t_ ] := {-b/a, -b/a} /;
	FreeQ[{a,b}, t] && ( n <= 0 )
Unit/: Extent1D[ Unit[n_Integer][a_. t_ + b_.], t_ ] :=
	Extent1DOrder[ -b/a, Sign[a] Infinity ] /;
	FreeQ[{a,b}, t] && ( n > 0 )

Unit/: Unit[n_][List[args]] := Unit[n][args]		(* multidimensional *)
Unit/: Unit[n_][t1_, t__] := Unit[n][t1] Unit[n][t]	(* separability     *)

Unit/: TheFunction[Unit[n_][t__]] :=
	 t^-(n + 1) / Factorial[-(n + 1)] CStep[t] /; N[n < 0]

Unit/: Format[ Unit[n_] ] := Subscripted[ Unit[n] ]	(* output forms     *)
Unit/: Format[ Unit[n_], TeXForm ] := Subscripted[ Unit[n] ]

Derivative[1][Unit[n_]] := Unit[n+1]			(* derivative rule  *)

SPIntegrationRules = 					(* integration rules *)
Join[ SPIntegrationRules,
      { integrate[ expr_. Unit[n_][a_. x_ + b_.], x_Symbol ] :>
	  Limit[expr, x -> -b/a] Unit[n-1][a x + b] / a /;
	FreeQ[{a,b,c}, x],

	integrate[ expr_. Unit[n_][a_ x_ + b_.], {x_Symbol, x1_, x2_}] :>
	  integrate[expr Unit[n][x + b/a] / a, x] /;
	FreeQ[{a,b}, x],

	integrate[expr_. Unit[n_][x_ + c_.], {x_Symbol, x1_, x2_}] :>
	  Limit[expr, x -> -c] CStep[-c - x1] CStep[c + x2] Unit[n-1][x + c] /;
	FreeQ[c, x]	(* CStep[-c - x1] CStep[c + x2] <--> x1 <= -c <= x2 *)
      } ]

(*  Upsample [Myers, 218-219] [Bamberger, ch. 2]  *)

	(* Error checking of upsample parameters  *)

Upsample/: Upsample[m_List, n_List, options___][f_] :=
	MyMessage[Upsample::convert, Upsample[DiagonalMatrix[m], n][f], m] /;
	VectorQ[m] && SameQ[Length[m], Length[n]]

Upsample/: Upsample[m_List, n_List, options___][f_] :=
	MyMessage[Upsample::badmD, f] /;
	VectorQ[m] && ! SameQ[Length[m], Length[n]]

Upsample/: Upsample[m_List, n_List, options___][f_] :=
	MyMessage[Upsample::badmD, f] /;
	MatrixQ[m] && ! Apply[SameQ, Append[Dimensions[m], Length[n]]]

Upsample/: Upsample[m_List, n_Symbol, options___][f_] :=
	MyMessage[Upsample::badmD, f]

		(*  degenerate cases  *)

Upsample/: Upsample[1, n_, options___][f_] := f
Upsample/: Upsample[-1, n_, options___][f_] := Rev[n][f]

	(* convert upsample into a functional form  *)

Upsample/: N[Upsample[m_, n_, options___][f_List]] := UpsampleSequence[f, m, 0]

Upsample/: TheFunction[Upsample[m_, n_Symbol, options___][f_List]] :=
		UpsampleSequence[f, m, 0]
Upsample/: TheFunction[Upsample[m_, n_Symbol, options___][f_]] :=
		UpsampledFunction[m, 0, n, TheFunction[f] /. n -> n / m]
Upsample/: TheFunction[Upsample[m_, n_List, options___][f_]] :=
		UpsampledFunction[ m, 0, n,
			TheFunction[f] /. ReplaceWith[n, Inverse[m] . n] ]

	(* extent of upsampled function  *)

Upsample/: Extent1D[ Upsample[m_, n_, options___][f_], n_ ] := m Extent1D[f, n]

	(* formatting of the upsample operator *)

Upsample/: Format[ Upsample[m_, n_List, options___] ] :=
	Block [	{bar},
		bar = TableForm[ Table[ "  |  ", Release[Dimensions[n]] ] ];
		Format[ SequenceForm[ Upsample,
				      Subscript[ TableForm[m] ],
				      Subscript[ bar ],
				      Subscript[ TableForm[n] ] ] ] ]
Upsample/: Format[ Upsample[m_, n_Symbol, options___] ] :=
		Format[ Subscripted[Upsample[m, n]] ]

Upsample/: Format[ Upsample, TeXForm ] := "\\uparrow"
Upsample/: Format[ Upsample[m_, n_, options___], TeXForm ] :=
		inTeXformat[ Upsample, m, n ]

Upsample/: GetOperatorVariables[ Upsample[l_, n_, options___] ] := n

(*  UpsampledFunction  *)
lastMatrix = Null
lastMatrixInverse = Null

integerVectorQ[x_] := Apply[ And, Map[ IntegerQ, x ] ]

UpsampledFunction[m_?NumberQ, l_, n_?NumberQ, f_] :=
	If [ IntegerQ[Rationalize[n / m]], f, l ]

UpsampledFunction[m_List, l_, n_List, f_] :=
	Block [	{minv, mmatrix, nvector, retval},
		retval = l;
		nvector = Rationalize[n];
		If [ integerVectorQ[ nvector ],
		     mmatrix = Rationalize[m];
		     minv = If [ SameQ[mmatrix, lastMatrix],
				 lastMatrixInverse,
				 lastMatrix = mmatrix;
				 lastMatrixInverse = Inverse[mmatrix] ];
		     If [ integerVectorQ[minv . nvector], retval = f ] ];
		retval ] /;
	Apply[ And, Map[ NumberQ, n ] ] && MatrixQ[m]

(*  UpsampleFactor --  must be independent of variable z  *)
UpsampleFactor[f_, z_, extractor_:GetAllExponents] :=
	Block [	{ratgcd},
		If [ FreeQ[f, z], Return[0] ];
		ratgcd = Apply[ RationalGCD, extractor[f, z] ];
		ratgcd = discBreakup[ratgcd, z];
		If [ IntegerQ[ratgcd],
		     ratgcd,
		     Numerator[ratgcd] ] ]

(*  UpsampleSequence  *)
UpsampleSequence[seq_, m_:2, fill_:0] :=
	Block [	{fillcount, input, len, newseq},

		len = Length[seq];
		newseq = List[ seq[[1]] ];

		(*  Build the new sequence newseq by appending
		    m - 1 fill elements and then by appending
		    a value from the original sequence.  *)

		For [ input = 2, input <= len, input++,
		      For [ fillcount = 1, fillcount < m, fillcount++,
			    AppendTo[newseq, fill] ];
		      AppendTo[newseq, seq[[input]] ] ];

		newseq ]  /;
	IntegerQ[m] && ( m >= 2 )

(*  Z  *)
Z/: Z[n_] := Z[n, DummyVariables[Length[n], Global`z]]
Z/: Z[n_, z_, options___] [ InvZ[zi_, n_, ioptions___] [f_] ] := 
	If [ SameQ[z, zi], f, f /. ReplaceWith[zi, z] ] /;
	Length[z] == Length[zi]

Z/: Z^-1[a__] := InvZ[a]

Z/: Format[ Z[n_, z_, options___] ] :=
	Subscripted[ "Z"[ n ~goesto~ z ] ]
Z/: Format[ Z[n_, z_, options___], TeXForm ] :=
	Subscripted[ "{\\cal Z}"[ n ~goesto~ z ] ]

Z/: TheFunction[ Z[n_, args__][x_] ] :=
	findIt[ "SignalProcessing`Digital`ZTransform`",
		"ZTransform", x, n, args ]


(*  E N D     P A C K A G E  *)

End[]
EndPackage[]

Protect[ SPIntegrationRules ]


(*  A L I A S E S  *)

Unprotect[ { AliasedSinc, AliasSinc, ASinc } ]
Clear[ AliasedSinc, AliasSinc, ASinc ]
AliasedSinc = Dirichlet
AliasedSinc::usage = Dirichlet::usage
AliasSinc = Dirichlet
AliasSinc::usage = Dirichlet::usage
ASinc = Dirichlet
ASinc::usage = Dirichlet::usage
Protect[ { AliasedSinc, AliasSinc, ASinc } ]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell ];
     On[ General::spell1 ] ]


(*  H E L P     I N F O R M A T I O N  *)

Block [	{newfuns},
	newfuns = 
	  { DeltaIntegrate,	  DeltaPlot,		DiscreteGraphics,
	    Extent1D,		  DummyVariables,	GetDeltaFunctions,
	    IntervalsToFunction,  OperatorInOperExpr,	OperatorName,
	    ParametersInOperExpr, RewriteSummations,	ScalingFactor,
	    SequencePlot,	  SequenceToFunction,   SignalPlot,
	    SignalsInOperExpr,    "Signum",		SPSimplify,
	    TheFunction,	  ToContinuous,	  	ToDiscrete,
	    UpsampledFunction,	  UpsampleFactor,	UpsampleSequence };
	Combine[ SPfunctions, newfuns ];
	Apply[ Protect, newfuns ] ]

Block [	{newoperators},
	newoperators =
	  { CConvolve,		CircularShift,
	    Convolve,		DFT,		  	Difference,
	    Downsample,		DTFT,			FT,
	    Interleave,		InvDFT,			InvDTFT,
	    InvFT,		InvL,			InvZ,
	    L,			PolyphaseDownsample,	PolyphaseResample,
	    PolyphaseUpsample,  Periodic,		Rev,
	    RotateAxes,		ScaleAxis,		Shift,
	    Summation,		Upsample,		Z };
	Combine[ SPoperators, newoperators ];
	Apply[ Protect, newoperators ] ]

Block [	{newsignals},
	newsignals =
	  { "AliasSinc",	"ASinc",		CPulse,
	    CStep,		Delta,			Dirichlet,
	    Impulse,		LineImpulse,		Pulse,
	    Sinc,		Step,			Unit };
	Combine[ SPsignals, newsignals ];
	Apply[ Protect, newsignals ] ]



(* Under Mma 2.0, Delta is used by Integrate *)
If [ TrueQ[ $VersionNumber >= 2.0 ], Unprotect[Delta] ]

(* Protect objects from Calculus`DiracDelta` *)
If [ TrueQ[ $VersionNumber >= 2.1 ], Protect[DiracDelta, UnitStep] ]

(*    When encoded using ":=", these next rules causes Simplify to   *)
(*  carry out incorrect simplifications when an expression contains  *)
(*  an Impulse term and a term containing 1/n because Simplify will  *)
(*  group the expression over a common denominator thereby putting   *)
(*  the n in front of Impulse[n] which whould force it to zero,      *)
(*  thereby eliminating a perfectly valid term.			     *)

Unprotect[ SPSimplificationRules ]
AppendTo[ SPSimplificationRules,
	  ( f_ Impulse[n_Symbol + m_.] :> (f /. n -> -m) Impulse[n + m] /;
	    NumberQ[m] && ! FreeQ[f, n] ) ]
AppendTo[ SPSimplificationRules,
	  ( Sinc[a_ (x_ + x0_.)] :> Sinc[-a (x + x0)] /; Negative[a] ) ]
AppendTo[ SPSimplificationRules,
	  ( f_. Cos[theta_] Sinc[theta_] + g_. Sinc[phi_] :> 0 /;
	    ( f == -g ) && ( phi == 2 theta ) ) ]
AppendTo[ SPSimplificationRules,
	  ( Delta[a_ (x_ + x0_.)] :> Delta[-a (x + x0)] /; Negative[a] ) ]
AppendTo[ SPSimplificationRules, ( Delta[a_?Positive x_] :> Delta[x] / a ) ]
AppendTo[ SPSimplificationRules, ( Delta[a_?Negative x_] :> Delta[x] / (-a) ) ]
AppendTo[ SPSimplificationRules,
	  ( Impulse[a_?Negative b_. (x_ + x0_.)] :> Impulse[-a b (x + x0)] ) ]

AppendTo[ SPSimplificationRules, ( CPulse[L_, L_ - x_] :> CPulse[L, x] ) ]
AppendTo[ SPSimplificationRules,
	  ( CPulse[-L_, x_ - L_/2] :> CPulse[L, x + L/2] ) ]
AppendTo[ SPSimplificationRules,	(* JMW and BLE *)
	  ( CPulse[a_, t_Symbol + b_.] CPulse[c_, t_Symbol + d_.] :>
	Block [ {ext},
		ext = Extent1D[CPulse[a, t + b] CPulse[c, t + d], t] //.
		      SignalProcessing`Support`SupCode`Private`MinMaxRules;
		If [ SameQ[ext, None],
		     0,		(* Product of disjoint pulses disappear *)
		     CPulse[ ext[[2]] - ext[[1]], t - ext[[1]] ] ] ] /;
	    Apply[And, Map[NumberQ, N[ {a, b, c, d} ] ] ] ) ]
AppendTo[ SPSimplificationRules,      (* JMW and BLE *)
	  Periodic[ T_, -t_Symbol ][ x_ ] :> Periodic[ T, t ][ x ]  ]
PrependTo[ SPSimplificationRules,       (* JMW *)
	   ScaleAxis[ scale_, -t_Symbol ][ f_ ] :>
	     ScaleAxis[ scale, t ][ f ]  ]
PrependTo[ SPSimplificationRules,       (* JMW *)
	   ScaleAxis[ scale_, {-x_Symbol, -y_Symbol} ][ f_ ] :>
	     ScaleAxis[ scale, {x, y } ][ f ]  ]
PrependTo[ SPSimplificationRules,       (* JMW *)
	   RotateAxes[ angle_, { -x_Symbol, -y_Symbol } ][ f_ ] :>
	     RotateAxes[ angle, { x, y } ][ f ]  ]
AppendTo[ SPSimplificationRules,      (* JMW *)
	( x_. CPulse[ L_, t_ + a_. ] Periodic[ T_, t_ ][y_] :>
	Block [	{bottom, hedge, k, ledge, retval, top, wlow, whigh},

		{wlow, whigh} = Extent1D[ CPulse[ L, t + a ], t ];
		{ledge, hedge} = Extent1D[ y, t ];
		bottom = T Ceiling[ N[ (wlow - ledge) / T ] ];
		top = T Floor[ N[ (whigh - hedge) / T ] ];
		retval = If [ N[(bottom - T + hedge) >= wlow],
			      (y /. (t -> t - (bottom - T))) *
				CPulse[ bottom - T + hedge - wlow, t - wlow ],
			      0 ];
		If [ N[(top + T + ledge) <= whigh],
		     retval += (y /. t -> (t - (top + T))) *
			       CPulse[(top + T + ledge) - whigh, t - whigh] ];
		k = Unique["k"];
		retval += Summation[ k, bottom, top, T ][ y /. t -> (t - k) ];
		x retval ] /;
	NumberQ[N[L]] && NumberQ[N[a]] && NumberQ[N[T]] ) ]

(*  Simplifications for convolution  *)

AppendTo[ SPSimplificationRules,
	  h_[t_][x_, xrest___] :> Replace[h[t][x, xrest], SPConvolutionRules] /;
	  SameQ[h, Convolve] || SameQ[h, CConvolve] ]

SPConvolutionRules = {
	  h_[t_][z_?ZeroQ, x_] :> z ,		(*    the head is either *)
	  h_[t_][x_, z_?ZeroQ] :> z ,		(* CConvolve or Convolve *)

	  CConvolve[t_Symbol][f_ x_, y_] :>			(* JMW *)
	    f CConvolve[t][x, y] /;
	    FreeQ[f, t],
	  CConvolve[t_Symbol][x_, f_ y_] :>
	    f CConvolve[t][x, y] /;
	    FreeQ[f, t],

	  CConvolve[t_Symbol] [x_, c_. Delta[t_ + m_.]] :>
	    ((c x) /. t -> (t+m)) /;
	    FreeQ[m, t],
	  CConvolve[t_Symbol] [c_. Delta[t_ + m_.], x_] :>
	    ((c x) /. t -> (t+m)) /;
	    FreeQ[m, t],
	  CConvolve[tlist_List] [x_, c_. Delta[t_ + m_.]] :>
	    CConvolve[ Complement[tlist, {t}] ][x /. t -> (t+m), c] /;
	    MemberQ[tlist, t] && FreeQ[m, t],
	  CConvolve[tlist_List] [c_. Delta[t_ + m_.], x_] :>
	    CConvolve[ Complement[tlist, {t}] ][c, x /. t -> (t+m)] /;
	    MemberQ[tlist, t] && FreeQ[m, t],

(* This rule added by Roberto H. Bamberger *)
(*
	  CConvolve[t_Symbol][Periodic[period_,t_Symbol][ func1__ ],func2__] :>
		Periodic[period,t][ CConvolve[t][func1,func2] ],
	  CConvolve[t_Symbol][func2__,Periodic[period_,t_Symbol][ func1__ ]] :>
		Periodic[period,t][ CConvolve[t][func1,func2] ],
 *)

	  Convolve[n_Symbol, ___] [x_, c_. Impulse[n_ + m_.]] :>
	    ((c x) /. n -> (-m)) /;
	    FreeQ[m, n],
	  Convolve[n_Symbol, ___] [c_. Impulse[n_ + m_.], x_] :>
	    ((c x) /. n -> (-m)) /;
	    FreeQ[m, n],
	  Convolve[nlist_List] [x_, c_. Impulse[n_ + m_.]] :>
	    Convolve[ Complement[nlist, {n}] ][x /. n -> (-m), c] /;
	    MemberQ[nlist, n] && FreeQ[m, n],
	  Convolve[nlist_List] [c_. Impulse[n_ + m_.], x_] :>
	    Convolve[ Complement[nlist, {n}] ][c, x /. n -> (-m)] /;
	    MemberQ[nlist, n] && FreeQ[m, n]
}

Protect[ SPConvolutionRules ]

(*  Simplifications when an impulse is the input      *)

AppendTo[ SPSimplificationRules,
	  ( Downsample[L_, n_][a_. Impulse[n_]] :> a Impulse[n] ) ]
AppendTo[ SPSimplificationRules,
	  ( Upsample[L_, n_][a_. Impulse[n_]] :> a Impulse[n] ) ]

(*  Simplifications when an exponential is the input  *)

AppendTo[ SPSimplificationRules,
	  ( Rev[n_Symbol][Exp[ Complex[0, a_] b_. n_ ] ] :>
	    Exp[-I a b n] ) ]
AppendTo[ SPSimplificationRules,
	  ( ScaleAxis[l_,w_Symbol][Exp[Complex[0, a_] b_. w_]] :>
	    Exp[I a b w l] ) ]

(*  Other simplifications:  when these are encoded 	*)
(*  using ":=", they can cause rules in some of the	*)
(*  packages to confuse Mathematica to no end.  For	*)
(*  example, hard wiring these rules will cause		*)
(*  the "RewriteRules.m" package to hang.		*)
AppendTo[ SPSimplificationRules,
	  ( Difference[k_, n_][c_] :> 0 /; MyFreeQ[c,n] ) ]
AppendTo[ SPSimplificationRules,
	  ( Downsample[m_, n_][c_] :> c /; MyFreeQ[c, n] ) ]
AppendTo[ SPSimplificationRules,
	  ( Periodic[k_,n_][x_] :> x /; MyFreeQ[x, n] ) ]
AppendTo[ SPSimplificationRules,
	  ( Rev[n_][x_] :> x /; MyFreeQ[x, n] ) ]
AppendTo[ SPSimplificationRules,
	  ( ScaleAxis[l_, w_][c_] :> c /; MyFreeQ[c, w] ) ]
AppendTo[ SPSimplificationRules,
	  ( Shift[m_, n_][x_] :> x /; MyFreeQ[x, n] ) ]
AppendTo[ SPSimplificationRules,
	  ( Summation[k_, rest___][a_ x_] :> a Summation[k, rest][x] /;
	    FreeQ[a, k] ) ]

(*  Bizarre resampling cases  *)
AppendTo[ SPSimplificationRules,
	  ( h1_[Sign[m_], n_][ h2_[Abs[m_], n_][f_] ] :> h2[m,n][f] /;
	    ( SameQ[h1, Downsample] || SameQ[h1, Upsample] ) &&
	    ( SameQ[h2, Downsample] || SameQ[h2, Upsample] ) ) ]

AppendTo[ SPSimplificationRules,
	  ( h_[m_, n_][c_ f_] :> c h[m, n][f] /;
	    FreeQ[c, n] && ( SameQ[h, Downsample] || SameQ[h, Upsample] ) ) ]


Protect[ SPSimplificationRules ]


(*  E N D     M E S S A G E  *)

Print["The knowledge base of signal processing operators has been loaded."]
Null
