(*  :Title:	Smith Chart *)

(*  :Authors:	William Anklam (see Discussion) and Brian Evans	*)

(*
    :Summary:	This package contains functions that will produce a 
		Smith Chart (Impedance) graphics object.
*)

(*  :Context:	SignalProcessing`Support`SmithChart`	*)

(*  :PackageVersion:  SPP 2.9  @(#)SmithChart.m	1.1	2/8/94  *)

(*  :Copyright: Copyright (c) 1992 Hewlett-Packard Company

	Permission to use, copy, modify, and distribute this software
	and its documentation for any purpose and without fee is
	hereby granted, provided that the above copyright notice
	appears in all copies and that both the above copyright notice
	and this permission notice appear in supporting documentation.
	Hewlett-Packard Company makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
*)

(*
    :History:	Integrated into the signal processing packages by Brian Evans
		Revised by William Anklam, February 1993.
		Version 1.0 by William Anklam, September 1992.
*)

(*  :Keywords:	*)

(*  :Source:	*)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*
    :Discussion: 

	This package was developed by William Anklam (billa@sr.hp.com)

	    Microwave Technology Division, 2US-X
	    Hewlett-Packard Company 
	    1412 Fountaingrove Parkway
	    Santa Rosa, CA. 95403

	With permission, Brian Evans has embedded into the signal processing
	packages for Mathematica.  He added the function SmithChartPlot
	and made the existing code compatible for different versions of
	Mathematica.

	For example, the following code generates data to be plotted,
	formats it as real and imaginary pairs, assigns to s11plot the
	graphics primitives that would plot the data (but the actual
	plotting i delayed through the use of DisplayFunction -> Identity),
	the starting frequency will be highlighted by a large dot, then
	finally all the graphics are displayed in one fell swoop usin
	the Show[] function:

	data = Table[ someFunc[f], {f, fmin, fmax, fstep}];
	data = Map[ {Re[#], Im[#]}&, data ] //N;
	s11plot = ListPlot[data, PlotJoined->True, DisplayFunction->Identity];
	startPoint = Graphics[ {PointSize[0.02], Point[ data[[1]] ]} ];
	Show[SmithChartGraphics, s11plot, startPoint, PlotLabel -> "S11",
	     AspectRatio->Automatic, DisplayFunction->$DisplayFunction];
*)


BeginPackage[ "SignalProcessing`Support`SmithChart`",
	      "Graphics`Graphics`" ]


(*  U S A G E     I N F O R M A T I O N  *)

SmithChart::usage = 
	"SmithChart displays a Smith Chart by issuing the command \
	Show[SmithChartGraphics, DisplayFunction:>$DisplayFunction]. \
	Function written by William Anklam."

SmithChartGraphics::usage = 
	"SmithChartGraphics returns the -Graphics- primitives required to \
	plot a Smith Chart. \
	SmithChartGraphics is meant to be used within the Show[] command \
	function so that the Smith Chart can be displayed along with data. \
	Function written by William Anklam."

SmithChartPlot::usage =
	"SmithChartPlot[y, {f, fmin, fmax}] plots the impedance function y \
	on a Smith chart. \
	Function written by William Anklam."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin["`Private`"]

(* Parametrize the resistive r=R/Zo contours *)
SmithR[r_, t_] := { (Cos[t] + r)/(1 + r), Sin[t]/(1 + r) }

(* Parametrize the reactive x=X/Zo contours *)
SmithX[x_, t_] := { Cos[t]/x +1, (Sin[t] + 1)/x }

(* Overrides the HoldAll attribute in a version-compatible manner *)
parametricPlot[x_, i_, rest___] := ParametricPlot[x, i, rest]

SmithChartGraphics :=
Block[ {phi, r1, r2, r3, t, unitcircle, x1, x2, x3, x4, x5, x6},
	unitcircle = PolarPlot[1, {phi, 0, 2Pi}, 
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	r1 = parametricPlot[SmithR[1/2, t], {t, 0, 2Pi},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	r2 = parametricPlot[SmithR[1, t], {t, 0, 2Pi},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	r3 = parametricPlot[SmithR[2, t], {t, 0, 2Pi},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	x1 = parametricPlot[SmithX[2,t], {t, 3Pi/2, 51Pi/64},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	x2 = parametricPlot[SmithX[1,t], {t, 3Pi/2, Pi},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	x3 = parametricPlot[SmithX[1/2,t], {t, 3Pi/2, 617Pi/512},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	x4 = parametricPlot[SmithX[-2,t], {t, 13Pi/64, -Pi/2},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	x5 = parametricPlot[SmithX[-1,t], {t, 0, -Pi/2},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	x6 = parametricPlot[SmithX[-1/2,t], {t, -105Pi/512, -Pi/2},
			PlotStyle->{Thickness[0.001]},
			DisplayFunction->Identity];
	Show[unitcircle, r1, r2, r3, x1, x2, x3, x4, x5, x6,
		Graphics[ { Thickness[0.001], Line[{ {-1,0}, {1,0} }] } ],
		Axes->None,
		AspectRatio->Automatic,
		DisplayFunction->Identity]
    ]

SmithChart := Show[SmithChartGraphics, DisplayFunction:>$DisplayFunction]

SmithChartPlot[ y_, {f_, fmin_, fmax_}, options___ ] :=
	Block [	{init, pplot, y0},
		pplot = ParametricPlot[ { Re[y], Im[y] }, {f, fmin, fmax},
					DisplayFunction -> Identity,
					 options ];
		y0 = y /. f -> fmin;
		init = Graphics[ { PointSize[0.02], Point[ N[{fmin, y0}] ] } ];
		Show[SmithChartGraphics, pplot, init,
		     options,
		     AspectRatio -> Automatic,
		     DisplayFunction :> $DisplayFunction] ]

End[]

Protect[SmithChart, SmithChartGraphics, SmithChartPlot]

EndPackage[]
